using System;
using System.IO;			// for StreamReader I/O from a file
using System.Windows.Forms;	// for RichTextBox

namespace MyNamespace
{
	/// <summary>
	///		XJCalc - Extra Precision Integer Matrix Calculator
	///		Developed in TURBO Pascal 5.5 and 6.0, converted to Visual Basic 6
	///		and now in Visual C# .Net
	///
	/// ---------------- *** LEGAL NOTICE: *** ------------------
	///
	/// This program is free software; you can redistribute it and/or modify
	/// it under the terms of the GNU General Public License (GPL) as published
	/// by the Free Software Foundation. cf. the file COPYING.txt.
	///
	/// ------------ *** end of legal notice *** ------------------
	/// </summary>

	// ------- A multiple-precision named signed integer
	public class MultiN : MultiJ
	{
		public string Nm;			// name

		public MultiN()				// initialize with M = 0 and no name
			: base()				// calls MultiSI()
		{
			Nm = "";	// constructor logic
		}

		public MultiN(int nMaxI, string nm)		// initialize with M = nMaxI and new name
			: base(nMaxI)						// calls MultiSI(i)
		{
			Nm = nm;
		}

//		public static void operator = (MultiN left, MultiSI right)		// not allowed to overload =
//		{
//			Calc.SetToNM(right, left);
//		}

		public static MultiN operator & (MultiN left, MultiSI right)	// use &= for = assignment
		{
			Calc.SetToNM(right, left);
			return left;
		}
	}

	public class Calc
	{
		public Calc(int iMaxDigits)	// constructor logic
		{
			if (iMaxDigits < 49) iMaxDigits = 49;					// size for >= 49 decimal digits
			int iNMax = 1 + (iMaxDigits - 1) / MultiID.muDMax;		// convert from digits to super digits
			Common.nMax = Math.Min(iNMax, MultiID.muNMax);			// fMC <= mMax <= nMax <= muNMax
			new Calc();
		}

		public Calc()	// constructor logic
		// initialize Multi-precision Named signed Integer package
		// input: nMax muDMax
		{
			MultiID.intCalc = true;
			Common.quiet = true;
			new MultiID();				// initialize class MultiID
			new MultiJD();				// init Multiple-precision integer Matrix package
			Common.delim = ",;<=>+-!|*/&:()^@#%\\'\"";		// @ needed for Batch commands	// ,;<=>+-!|*/&:()^@#%\'"
			MultiID.mMax = Common.nMax;						// fMC <= mMax <= nMax <= muNMax
			fMC = Common.nMax;
			MultiID.muDpG = 5;
			MultiID.maxDis = Math.Min(200, MultiID.mMax * MultiID.muDMax); // set max digits to display
			CheckNMax();
			Common.mList = 1000;
			Common.nList = new MultiN[Common.mList+1];		// create the list of named numbers
			Common.InitN(1, "", 0);							// change unused name at [0] from null to ""
			Common.InitN(1, "X", 1);						// now create and add a named numbers to the list
			Common.cList = 1;								// x=0 on top of list
			Common.iTop = 1;
			Common.lastTop = 0;
			Common.comI = 0;
			xICIn = 0;
			xILOut = 0;
			xINIn = 0;
			xINOut = 0;
			Common.lrnL = 0;
			MultiID.debug = false;
			MultiID.diagOn = false;
			MultiID.timeIt = false;
			Common.autoD = true;
			Common.echoB = true;
			Common.exX = 0;
			Common.prSt = "";
			Common.oldField = "";
			Common.noChange = false;
			Common.saveTop = true;
			MultiID.softAbort = false;
			MultiID.baseI = 10;
			MultiID.baseO = 10;
			MultiID.allowFHT = true;	// allow FHT
			MultiID.forceFHT = false;	// but don't force it
			new Common();		// initialize class Common
			new FHTMult();		// initialize class FHTMult
			new XIMult();		// initialize class XIMult
			new NPrimes();		// initialize class NPrimes
			new PrimeFA();		// initialize class PrimeFA
			new XICommon();		// initialize class XICommon
			new XICalLst();		// initialize class XPCalLst
			new XJCalLst();		// initialize class XJCalLst
			Common.running = true;
			Common.quiet = false;
			CalReadCode("AutoExec.XJC");
			Common.quiet = true;
		} // Calc

		public static void CalSave() // save all values on list to disk files and make
									 // file Restore.XJC for restoring them
		{
			StreamWriter outF; // text file, for Restore.XJC
			int current;
			string fName;
			int n;
			string st;
			string savDir = Environment.CurrentDirectory.ToString();
			Directory.SetCurrentDirectory(Common.homeDir);
			if (Common.ChkWrite(out outF, "Restore.XJC", ""))
			{
				MultiID.WriteLn(null, outF, "@Echo Off");
				MultiID.WriteLn(null, outF, "Rem  Start of file Restore.XJC");
				current = Common.cList;
				n = 10000;
				while (current != 0)
				{
					fName = n.ToString();
					fName = "Save" + fName.Substring(1, 4) + ".XJN";
					MultiID.WriteLn(null, outF, "ReadN(\"" + fName + "\")");
					current--;
					n++;
				}
				MultiID.WriteLn(null, outF, Common.nList[Common.iTop].Nm + "=");
				MultiID.WriteLn(null, outF, "Rem  End of file Restore.XJC");
				if (Common.echoB) { st = "On"; } else { st = "Off"; }
				MultiID.WriteLn(null, outF, "Echo " + st);
				outF.Close();
				if (!Common.quiet) MultiID.WriteLn("File Restore.XJC closed");
				current = Common.cList;
				while (current != 0)
				{
					n--;
					fName = n.ToString();
					fName = "Save" + fName.Substring(1, 4) + ".XJN";
					CalWriteN(fName, current);
					current--;
				}
			}
			Directory.SetCurrentDirectory(savDir);
		}

		public static void ChkResSav(int resSav, string fName)
		// check for Restore, Save, or Run procedure
		{
			if (resSav == 1) // restore
			{
				CalReadConfig();
				Common.RestoreHist();
				bool saveSaveTop = Common.saveTop; // don't change LastTop after it is read back in
				Common.saveTop = false;
				CalReadCode("Restore.XJC");
				Common.saveTop = saveSaveTop;
			}
			else if (resSav == 2) // save
			{
				CalWriteConfig();
				Common.SaveHist();
				CalSave();
			}
			else if (resSav == 3) // run
			{
				CalReadCode(fName);
			}
		}

		public static void ClearN(int nI) // set value = 0
		{
			Common.nList[nI] = new MultiN(0, Common.nList[nI].Nm);
		}

		public static void SetToN(MultiSI x, int zI) // z = x, resized
		// copy everything but .Nm name
		{
			SetToNM(x, Common.nList[zI]);
		}

		public static void SetToN(MultiJ x, int zI) // z = x, resized
		// copy everything but .Nm name
		{
			SetToNM(x, Common.nList[zI]);
		}

		public static void SetToII(int xI, int zI) // z = x
		// copy everything but .Nm name
		{
			SetToNM(Common.nList[xI], Common.nList[zI]);
		}

		public static void SetToNM(MultiSI x, MultiSI z) // z = x, resized
		// copy everything but .Nm name
		{
			int m = x.N;
			if (m != z.M)
			{
				z.V = new int[m+1]; // resize V
				z.M = m;
			}
			MultiID.SetToSI(x, z);
		}

		public static void SetToNM(MultiSI x, MultiJ z) // z = x, resized
		{
			if (z.Rs != 0)
			{
				z.E = new MultiSI[0, 0];
				z.Rs = z.Cs = 0;
			}
			SetToNM(x, (MultiSI)z);
		}

		public static void SetToNM(MultiJ x, MultiJ z) // z = x, resized
		{
			if (Object.ReferenceEquals(x, z))
				return;
			SetToNM((MultiSI)x, (MultiSI)z);
			int rs = x.Rs;
			int cs = x.Cs;
			z.Rs = rs;
			z.Cs = cs;
			z.E = new MultiSI[rs, cs];
			for (int r = 0; r < rs; r++)
				for (int c = 0; c < cs; c++)
				{
					z.E[r, c] = new MultiSI();
					SetToNM(x.E[r, c], z.E[r, c]);
				}
		}

		public static void NormN(int nI) // normalize number indexed by nI on the list
		{
			int n = Common.nList[nI].N;
			if (n > fMC)
			{
				MultiID.WriteDLn("Most significant digits discarded in " + Common.nList[nI].Nm);
			}
			MultiJD.NormM(Common.nList[nI]);
		}

		public static void NormNM(MultiSI x) // normalize x with M = N, release unused memory
		{
			int m = x.M;
			int n = x.N;
			if (n > fMC)
			{
				n = fMC;
				x.N = n;
				MultiID.NormSI(x);			// get rid of any new leading zero super digits
			}
			if (n == 1 && x.V[0] == 0) x.S = false; // prevent -0
			if (m == n) return;
			Common.ReDimPreserve(ref x.V, n+1);
			x.V[n] = 0;						// be neat
			x.M = n;
		}

		public static void ValueN10(string st, out int i, int zI)
		// convert a base 10 String to z, returns i = # of character used
		// allows st to be a literal, muErr set true if overflow
		{
			int saveBaseI = MultiID.baseI;
			MultiID.baseI = 10;
			ValueN(st, out i, zI);
			MultiID.baseI = saveBaseI;
		}

		public static void ValueN(string st, out int i, int zI)
		// convert String to z, returns i = number of character used
		// allows st to be a literal, muErr set true if overflow
		{
			int m;
			m = (int)(5 + st.Length * Math.Log10(MultiID.baseI) / MultiID.muDMax);
			if (m > Common.nMax) m = Common.nMax;
			MultiSI t = new MultiSI(m);
			MultiID.ValueSI(st, out i, t);
			m = t.N;
			Common.nList[zI].V = new int[m+1];
			Common.nList[zI].M = m;
			MultiID.SetToSI(t, Common.nList[zI]);
			MultiID.DoneSI(t);
		}

		public static void WriteNV(RichTextBox outB, StreamWriter outF, MultiN x, ref int tot, int width)
		// output x name + V and a new line. x not modified
		{
			MultiID.WriteT(outB, outF, x.Nm + " =", ref tot, width);
			if (outB == null)
				MultiID.WriteTLn2(outB, outF, "", ref tot, width);
			else
				MultiID.WriteT(outB, outF, " ", ref tot, width);
			MultiJD.WriteJ(outB, outF, x, ref tot, width, true);
		}

		public static void WriteNumLn(MultiN x, ref int tot)
		// output x value and a new line. x not modified
		{
			MultiID.WriteSILn(x, ref tot);
		}

		public static void CheckNMax() // fMC <= mMax <= nMax <= muNMax
		// fMC = mMax // for the Integer Calculator
		{
			int fMCL;
			fMCL = fMC;
			int min = 1 + (34 / MultiID.muDMax); // min of 35 decimal digits
			MultiID.mMax = Math.Max(MultiID.mMax, min);
			MultiID.mMax = Math.Min(MultiID.mMax, Common.nMax);
			fMC = MultiID.mMax;
			if (fMC < fMCL) ListSetM();
		}

		public static void CalSMan(int nI)
		// set current number of digits in a result
		{
			if (Common.nList[nI].Rs != 0)
				MultiJD.ErrNotScalar();
			else
			{
				double db;
				MultiJD.GetDbInJ(out db, 0, Int32.MaxValue, Common.nList[nI]);
				db = 1 + (db - 1) / MultiID.muDMax; // set for at least x decimal digits
				fMC = (int)db;
				MultiID.mMax = fMC; // for the Integer Calculator
				CheckNMax();
				ListSetM();
			}
		}

		public static void ListSetM() // for all items, reduce M to fMC if greater than fMC
		{
			for (int cI = 1; cI <= Common.cList; cI++)
			{
				if (Common.nList[cI].N > fMC)
				{
					NormN(cI); // re-dimension the .V vector
				}
			}
		}

		public static bool GetFName(ref string st)
		// get file name from command line
		// if first parm ends in .XJC, it is changed to Run("... .XJC")
		// if first parm ends in .XJN, it is changed to ReadN("... .XJN")
		{
			bool bGetFName = false;
			int i = st.Length;
			if (i > 4)
			{
				string ext = st.Substring(i - 4, 4);
				ext = ext.ToUpper();
				if (ext == ".XJC")
				{
					st = "Run(\"" + st + "\")";
					bGetFName = true;
				}
				else if (ext == ".XJN")
				{
					st = "ReadN(\"" + st + "\")";
					bGetFName = true;
				}
			}
			if (!bGetFName) st = "";
			return bGetFName;
		}

		public static void TestExpanded() { return; } // noOp for XJCalc

		public static bool UnsignedConst(ref string st, int nI)
		// do an Unsigned Constant, Must input number like F... as 0F...
		{
			char ch;
			ch = st[0];
			if (ch >= '0' && ch <= '9')
			{
				Common.EnterN(ref st, nI);
				return true;
			}
			return false;
		}

		public static bool Constant(ref string st, int nI)
		// do a constant, Input number like F... as 0F...
		{
			char ch;
			int i = 0;
			string maxD = MultiID.MaxDig(MultiID.baseI);	// max Input digit "1" to "Z"
			if ( st != "" && (st[0] == '+' || st[0] == '-') ) i++;
			if (i >= st.Length) return false;
			ch = st[i];
			if (ch >= '0' && ch <= '9' && ch <= maxD[0])
			{
				Common.EnterN(ref st, nI);
				return true;
			}
			return false;
		}

		public static void EnterMatrixN(ref string st, int zI) // enter a matrix {a11, a12, ..., }
		{
			int tI;
			Common.InitNAdd(1, "EnterMatrixN-tI", out tI);
			Common.DelNPBls(ref st, 1);			// delete the {
			string st2;
			int i = st.IndexOf("}");
			if (i < 0)
			{
				st2 = st;
				st = "";
			}
			else
			{
				st2 = st.Substring(0, i);
				Common.DelNPBls(ref st, i + 1);
			}
			int rs = 1;
			int cs = 1;
			i = -1;
			while ((i = st2.IndexOf(";", i+1)) >= 0)
				rs++;
			string st3;
			i = st2.IndexOf(";");
			if (i < 0)
				st3 = st2;
			else
				st3 = st2.Substring(0, i+1);
			string maxD; // max Input digit "1" to "Z"
			maxD = MultiID.MaxDig(MultiID.baseI);
			i = 0;
			if (st3 != "")
			{
				while ((i = st3.IndexOf(",", i+1)) >= 0)
				{
					if (!MultiJD.aDigit(st3[i-1].ToString(), maxD) ||
						!MultiJD.aDigit(st3[i+1].ToString(), maxD))
						cs++;
				}
			}
			XJCalLst.MatrixN(rs, cs, zI);
			int r = 0;
			int c = 0;
			int o = 0;
			while (st2.Length > 0)
			{
				if (Expression(ref st2, tI))
				{
					if (Common.nList[tI].Rs != 0)
					{
						MultiJD.ErrNotScalar();
						break;
					}
					if (r < rs && c < cs)
						SetToNM(Common.nList[tI], Common.nList[zI].E[r, c]);
					else
						o++;
					if (st2.Length > 0 && st2[0] == ';')
					{
						r++;
						c = 0;
						Common.DelNPBls(ref st2, 1);
					}
					if (st2.Length > 0 && st2[0] == ',')
					{
						c++;
						Common.DelNPBls(ref st2, 1);
					}
				}
				else break;
			}
			Common.LDelete(tI);
			if (o > 0)
				MultiJD.ErrOutside();
		} // EnterMatrixN

		public static bool AFunction(ref string st, int nI) // do a Function
		{
			int tI;				// temp for the 1st argument of a function
			int t2I;			// temp for the 2nd argument of a function
			int t3I;			// temp for the 3rd argument of a function
			int t4I;			// temp for the 4th argument of a function
			if (st[0] == '{')
			{
				EnterMatrixN(ref st, nI);
				return true;
			}
			int args = 1;
			string name = Common.GetField(ref st);
			int i = name.Length;
			if (i == 0 || i == st.Length || st[i] != '(') return false;
			Common.DelNPBls(ref st, 1 + i);
			Common.InitNAdd(fMC, "AFunction-tI", out tI);
			Common.InitNAdd(fMC, "AFunction-t2I", out t2I);
			Common.InitNAdd(fMC, "AFunction-t3I", out t3I);
			Common.InitNAdd(fMC, "AFunction-t4I", out t4I);
			if (!Expression(ref st, tI))
			{
				Common.ReportSin("Error in function's argument: " + name + "(|", ref st);
				args = 0;
			}
			else
			{
				if (st != "" && (st[0] == '\'' || st[0] == ','))
				{
					Common.DelNPBls(ref st, 1);
					args = 2;
					if (!Expression(ref st, t2I))
					{
						Common.ReportSin("Error in function's 2nd argument: " + name + "(..., |", ref st);
						ClearN(t2I);
						args = 1;
					}
					else
					{
						if (st != "" && (st[0] == '\'' || st[0] == ','))
						{
							Common.DelNPBls(ref st, 1);
							args = 3;
							if (!Expression(ref st, t3I))
							{
								Common.ReportSin("Error in function's 3rd argument: " + name + "(..., |", ref st);
								ClearN(t3I);
								args = 2;
							}
							else
							{
								if (st != "" && (st[0] == '\'' || st[0] == ','))
								{
									Common.DelNPBls(ref st, 1);
									args = 4;
									if (!Expression(ref st, t4I))
									{
										Common.ReportSin("Error in function's 4th argument: " + name + "(..., |", ref st);
										ClearN(t4I);
										args = 3;
									}
								}
							}
						}
					}
				}
				SetToII(tI, nI);
				ClearN(tI);		// to release memory
				if (st != "" && st[0] == ')') Common.DelNPBls(ref st, 1);
				if (MultiID.muErr)
					args = 0;
				else if (name == "ABS")
					XJCalLst.AbsN(Common.nList[nI], nI);
				else if (name == "BERND")
					BernD(Common.nList[nI], Common.nList[nI]);
				else if (name == "BERNN")
					BernN(Common.nList[nI], Common.nList[nI]);
				else if (name == "BINO")
					Bino(Common.nList[nI], Common.nList[t2I], Common.nList[nI]);
				else if (name == "BINOS")
					BinoS(Common.nList[nI], Common.nList[t2I], Common.nList[nI]);
				else if (name == "CHIN")
					ChinN(nI, t2I, nI);
				else if (name == "CHIN1")
					Chin1N(nI, t2I, nI);
				else if (name == "CONCAT")
					XJCalLst.ConcatN(Common.nList[nI], Common.nList[t2I], nI);
				else if (name == "CONCATR")
					XJCalLst.ConcatRN(Common.nList[nI], Common.nList[t2I], nI);
				else if (name == "CURT")
					XJCalLst.CubeRtN(nI, nI);
				else if (name == "DIG")
					XJCalLst.DigN(Common.nList[nI], Common.nList[t2I], nI);
				else if (name == "DIGD")
					XJCalLst.DigDN(Common.nList[nI], nI);
				else if (name == "DEL")
					XJCalLst.DelN(Common.nList[nI], Common.nList[t2I], nI);
				else if (name == "DELR")
					XJCalLst.DelRN(Common.nList[nI], Common.nList[t2I], nI);
				else if (name == "DET")
					XJCalLst.DeterminantN(Common.nList[nI], nI);
				else if (name == "DIAGONAL")
					XJCalLst.DiagonalN(Common.nList[nI], nI);
				else if (name == "DIVINT")
					DivIntN(nI, t2I, nI);
				else if (name == "DIVREM")
					XJCalLst.DivRemN(Common.nList[nI], Common.nList[t2I], nI);
				else if (name == "EULER" || name == "EULERN")
					Euler(Common.nList[nI], Common.nList[nI]);
				else if (name == "EXTRACT")
					XJCalLst.ExtractN(Common.nList[nI], Common.nList[t2I], nI);
				else if (name == "EXTRACTR")
					XJCalLst.ExtractRN(Common.nList[nI], Common.nList[t2I], nI);
				else if (name == "FAC")
					XJCalLst.RFacN(nI);
				else if (name == "FAC2")
					XJCalLst.Fac2N(Common.nList[nI], Common.nList[t2I], nI, false);
				else if (name == "FACM")
					XJCalLst.RFacMBN(nI);
				else if (name == "FACM2")
					XJCalLst.Fac2N(Common.nList[nI], Common.nList[t2I], nI, true);
				else if (name == "FACMS")
					XJCalLst.RFacMBSN(nI);
				else if (name == "FACS")
					XJCalLst.RFacSN(nI);
				else if (name == "FIB")
					XJCalLst.FibN(Common.nList[nI], nI);
				else if (name == "GAM")
					XJCalLst.GamN(Common.nList[nI], nI);
				else if (name == "GCD")
					XJCalLst.GCDN(Common.nList[nI], Common.nList[t2I], nI);
				else if (name == "GCDE")
					XJCalLst.GCDeN(nI, t2I, nI);
				else if (name == "GET")
					XJCalLst.GetElement(Common.nList[nI], Common.nList[t2I], Common.nList[t3I], nI);
				else if (name == "INSERT")
					XJCalLst.InsertN(Common.nList[nI], Common.nList[t2I], Common.nList[t3I], nI);
				else if (name == "INSERTR")
					XJCalLst.InsertRN(Common.nList[nI], Common.nList[t2I], Common.nList[t3I], nI);
				else if (name == "INV")
					XJCalLst.InvN(Common.nList[nI], Common.nList[t2I], nI);
				else if (name == "ISDIAG")
					XJCalLst.IsDiagN(Common.nList[nI], nI);
				else if (name == "ISSQ")
					IsSq(nI);
				else if (name == "KRON")
					Kron(Common.nList[nI], Common.nList[t2I], Common.nList[nI]);
				else if (name == "LCM")
					XJCalLst.LCMN(Common.nList[nI], Common.nList[t2I], nI);
				else if (name == "MAG")
					XJCalLst.MagN(Common.nList[nI], Common.nList[t2I], nI);
				else if (name == "MAT")
					XJCalLst.MatrixN(Common.nList[nI], Common.nList[t2I], nI);
				else if (name == "MATID")
					XJCalLst.MatIdN(Common.nList[nI], nI);
				else if (name == "MAX")
					XJCalLst.MaxN(Common.nList[nI], Common.nList[t2I], nI);
				else if (name == "MEQ")
					MersenneEqM(nI);
				else if (name == "MIN")
					XJCalLst.MinN(Common.nList[nI], Common.nList[t2I], nI);
				else if (name == "MOD")
					XJCalLst.ModN(Common.nList[nI], Common.nList[t2I], nI);
				else if (name == "MORD")
					Morder(Common.nList[nI], Common.nList[t2I], Common.nList[nI]);
				else if (name == "MPG")
					MersennePGM(nI);
				else if (name == "MPP")
					MersennePPM(nI);
				else if (name == "MPRIME")
					MersennePM(nI);
				else if (name == "MU")
					MoebiusMu(nI);
				else if (name == "P")
					PrimePT(nI);
				else if (name == "PASCAL")
					XJCalLst.PascalN(Common.nList[nI], nI);
				else if (name == "PEQ")
					PerfectEqM(nI);
				else if (name == "PGT")
				{
					PrimeFA.findUp = true;
					PrimeSearch(nI);
				}
				else if (name == "PHI")
					PrimePhi(nI);
				else if (name == "PHIL")
					PhiLN(nI, t2I, nI);
				else if (name == "PI")
					PrimePiT(nI);
				else if (name == "PIL")
					PrimePiLT(nI);
				else if (name == "PIL1")
					PrimePiL1(nI);
				else if (name == "PIM")
					PrimePiM(nI);
				else if (name == "PLT")
				{
					PrimeFA.findUp = false;
					PrimeSearch(nI);
				}
				else if (name == "PNG")
					PerfectNG(nI);
				else if (name == "POW")
					XJCalLst.PowN(Common.nList[nI], Common.nList[t2I], nI);
				else if (name == "POWM")
					XJCalLst.PowMN(Common.nList[nI], Common.nList[t2I], nI);
				else if (name == "PRIME")
					Prime(nI);
				else if (name == "PRIMO")
					XJCalLst.Primorial(nI);
				else if (name == "PRIMR")
					PrimR(nI, false);
				else if (name == "PRIMRP")
					PrimR(nI, true);
				else if (name == "PRIMRQ")
					PrimRQ(nI, t2I, nI);
				else if (name == "REV")
					XJCalLst.RevN(t2I, nI);
				else if (name == "RINT")
					XJCalLst.RRIntN(t2I, nI);
				else if (name == "RN")
					RRanN(nI);
				else if (name == "SIG")
					PrimeSig(nI);
				else if (name == "SIG0")
					PrimeSig0(nI);
				else if (name == "SE")
					XJCalLst.MulEN(Common.nList[nI], Common.nList[nI], nI);
				else if (name == "SET")
					XJCalLst.SetElement(Common.nList[nI], Common.nList[t2I], Common.nList[t3I], Common.nList[t4I], nI);
				else if (name == "SETALL")
					XJCalLst.SetAll(Common.nList[nI], Common.nList[t2I], nI);
				else if (name == "SIGN")
					Sign(nI);
				else if (name == "SIZE")
					XJCalLst.RSizeN(nI);
				else if (name == "SOLVE")
					XJCalLst.SolveN(Common.nList[nI], Common.nList[t2I], Common.nList[t3I], nI);
				else if (name == "SORD")
					Sorder(Common.nList[nI], Common.nList[t2I], Common.nList[nI]);
				else if (name == "SORTC")
					XJCalLst.SortC(Common.nList[nI]);
				else if (name == "SORTCS")
					XJCalLst.SortCS(Common.nList[nI], Common.nList[t2I], Common.nList[t3I]);
				else if (name == "SORTR")
					XJCalLst.SortR(Common.nList[nI]);
				else if (name == "SORTRS")
					XJCalLst.SortRS(Common.nList[nI], Common.nList[t2I], Common.nList[t3I]);
				else if (name == "SQ")
					XJCalLst.MulN(Common.nList[nI], Common.nList[nI], nI);
				else if (name == "SQFREE")
					SqFree(nI);
				else if (name == "SQRT")
					XJCalLst.SqRootN(Common.nList[nI], nI);
				else if (name == "SUMD")
					XJCalLst.SumDN(Common.nList[nI], Common.nList[t2I], nI);
				else if (name == "SUMDD")
					SumDDNM(nI);
				else if (name == "TAU")
					PrimeTau(nI);
				else if (name == "TRACE")
					XJCalLst.TraceN(Common.nList[nI], nI);
				else if (name == "TRAN")
					XJCalLst.TransposeN(nI);
				else
				{
					Common.ReportSin("Unknown function: |" + name + "(", ref st);
					args = 0;
				}
			}
			Common.LDelete(t4I);	// delete in reverse order of creation
			Common.LDelete(t3I);
			Common.LDelete(t2I);
			Common.LDelete(tI);
			if (name == "SET")
			{
				if (args == 1)
					MultiID.WriteDLn(name + ": 2nd, 3rd and 4th argument assumed = 0.0");
				if (args == 2)
					MultiID.WriteDLn(name + ": 3rd and 4th argument assumed = 0.0");
				if (args == 3)
					MultiID.WriteDLn(name + ": 4th argument assumed = 0.0");
			}
			else if (name == "GET" || name == "SOLVE" || name == "SORTCS" || name == "SORTRS")
			{
				if (args == 2)
						MultiID.WriteDLn(name + ": 3rd argument assumed = 0");
				if (args == 1)
						MultiID.WriteDLn(name + ": 2nd and 3rd argument assumed = 0");
			}
			else if (name == "BINO" || name == "BINOS" || name == "CHIN" || name == "CHIN1" ||
				name == "CONCAT" || name == "CONCATR" || name == "DEL" || name == "DELR" ||
				name == "DIG" || name == "DIVINT" || name == "DIVREM" || name == "EXTRACT" ||
				name == "EXTRACTR" || name == "FAC2" ||	name == "FACM2" || name == "GCD" ||
				name == "GCDE" || name == "INV" || name == "KRON" || name == "LCM" ||
				name == "MAG" || name == "MAT" || name == "MAX" || name == "MIN" ||
				name == "MOD" || name == "MORD" || name == "PHIL" || name == "POW" ||
				name == "POWM" || name == "PRIMRQ" || name == "REV" || name == "RINT" || name == "SETALL" ||
				name == "SORD" || name == "SUMD")
			{
				if (args == 1 && name != "REV")
					MultiID.WriteDLn(name + ": Second argument assumed = 0");
				if (args > 2)
					MultiID.WriteDLn(name + ": Only two arguments used");
			}
			else
			{
				if (args == 2)
					MultiID.WriteDLn(name + ": Second argument ignored");
			}
			return true;
		} // AFunction

		public static bool Factor(ref string st, int nI) // do a Factor
		{
			int tI;
			if (st == "") return false;
			if (UnsignedConst(ref st, nI)) return true;
			if (AFunction(ref st, nI)) return true;
			if (Variable(ref st, out tI))
			{
				SetToII(tI, nI);
				return true;
			}
			if (st != "" && st[0] == '(')
			{
				Common.DelNPBls(ref st, 1);
				if (Expression(ref st, nI))
				{
					if (st != "" && st[0] == ')') Common.DelNPBls(ref st, 1);
					return true;
				}
				else
				{
					Common.ReportSin("Expression expected: (|", ref st);
					return true;
				}
			}
			if (st != "" && st[0] == '!')
			{
				Common.DelNPBls(ref st, 1);
				if (Factor(ref st, nI))
				{
					XJCalLst.RNotN(nI);
					return true;
				}
				else
				{
					Common.ReportSin("Factor expected: !|", ref st);
					return true;
				}
			}
			return false;
		} // Factor

		public static bool Power(ref string st, int nI) // do a Power
		{
			string sign;
			string op;
			int tI;
			int t2I;
			if (!Factor(ref st, nI)) return false;
			while (true)
			{
				if (st == "") return true;
				op = st.Substring(0, 1);
				if (op != "^") return true;
				Common.DelNPBls(ref st, 1);
				if (st == "") return true;
				sign = st.Substring(0, 1);
				if (sign == "+" || sign == "-")
				{
					Common.DelNPBls(ref st, 1);
					if (st == "") return true;
				}
				else
				{
					sign = "";
				}
				Common.InitNAdd(fMC, "Power-tI", out tI);
				// 3^3^3 gave (3^3)^3 = 19683, should give 3^(3^3) = 762,55974,84987, fixed
				// by changing Factor to Power on next line 2003-11-22 HJS
				if (Power(ref st, tI))
				{
					if (sign == "-") Common.ChSN(tI);
					Common.InitNAdd(fMC, "Power-t2I", out t2I);
					if (op == "^")
					{
						XJCalLst.PowN(Common.nList[nI], Common.nList[tI], t2I);
					}
					SetToII(t2I, nI);
					Common.LDelete(t2I);
					NormN(nI);
				}
				else
				{
					Common.ReportSin("Exponent expected:" + "^" + sign + "|", ref st);
				}
				Common.LDelete(tI);
			}
		} // Power

		public static bool Term(ref string st, int nI) // do a Term
		{
			char op;
			int tI;
			int t2I;
			if (!Power(ref st, nI)) return false;
			while (true)
			{
				if (st == "") return true;
				op = st[0];
				if ((op != '*') && (op != '/') && (op != '@') && (op != '&') &&
					 (op != '%') && (op != '\\') && (op != '#') ) return true;
				Common.DelNPBls(ref st, 1);
				Common.InitNAdd(fMC, "Term-tI", out tI);
				if (!Power(ref st, tI))
				{
					Common.ReportSin("Factor expected: " + op + "|", ref st);
				}
				else
				{
					Common.InitNAdd(fMC, "Term-t2I", out t2I);
					if (op == '*')
						XJCalLst.MulN(Common.nList[nI], Common.nList[tI], t2I);
					else if (op == '@')
						XJCalLst.MulEN(Common.nList[nI], Common.nList[tI], t2I);
					else if (op == '/')
						XJCalLst.DivRemN(Common.nList[nI], Common.nList[tI], t2I);
					else if (op == '#')
						XJCalLst.MagN(Common.nList[nI], Common.nList[tI], t2I);
					else if (op == '%')
						XJCalLst.ModN(Common.nList[nI], Common.nList[tI], t2I);
					else if (op == '\\')
						XJCalLst.GCDN(Common.nList[nI], Common.nList[tI], t2I);
					else if (op == '&')
						XJCalLst.AndN(Common.nList[nI], Common.nList[tI], t2I);
					SetToII(t2I, nI);
					Common.LDelete(t2I);
					NormN(nI);
				}
				Common.LDelete(tI);
			}
		} // Term

		public static bool SimpExpr(ref string st, int nI)
		// do a Simple Expression
		{
			char sign = ' ';
			string op;
			string op2;
			int tI;
			if (st != "") sign = st[0];
			if (sign == '+' || sign == '-')
			{
				Common.DelNPBls(ref st, 1);
			}
			else
			{
				sign = ' ';
			}
			if (!Term(ref st, nI))
			{
				if (sign == ' ') return false;
				Common.ReportSin("Term expected: " + sign + "|", ref st);
				return true;
			}
			if (sign == '-') XJCalLst.ChSMN(nI);
			while (true)
			{
				if (st == "") return true;
				op = st.Substring(0, 1);
				op2 = "";
				if (st.Length > 1) op2 = st.Substring(0, 2);
				if ( (op != "+") && (op != "-") && (op != "|") && (op2 != "--") )
					return true;
				if (op2 == "--")
				{
					Common.DelNPBls(ref st, 2);
					op = op2;
				}
				else
				{
					Common.DelNPBls(ref st, 1);
				}
				Common.InitNAdd(fMC, "SimpExpr-tI", out tI);
				if (!Term(ref st, tI))
				{
					Common.ReportSin("Term expected: " + op + "|", ref st);
				}
				else
				{
					if (op == "+") XJCalLst.RAddN(Common.nList[tI], nI);
					else if (op == "-")	XJCalLst.RSubN(Common.nList[tI], nI);
					else if (op == "|")
						XJCalLst.OrN(Common.nList[nI], Common.nList[tI], nI);
					else if (op == "--")
						XJCalLst.RepUnitTN(Common.nList[nI], Common.nList[tI], nI);
				}
				Common.LDelete(tI);
			}
		} // SimpExpr

		public static bool Variable(ref string st, out int zI) // do a Variable
		// passed By Value is default, st is input and output, zI is output, points to Variable on the list
		{
			bool state;
			int i;
			int iL;
			char ch = ' ';
			string name;
			state = false;
			zI = 0;							// zI = 0 for returns of false
			name = Common.GetField(ref st);
			iL = name.Length;
			if (iL > 0) ch = name[0];
			if (ch >= 'A' && ch <= 'Z')		// first ch is a letter
			{
				i = 0;
				state = true;
				while (i < iL-1)
				{
					i++;
					ch = name[i];
					if ( (ch >= 'A' && ch <= 'Z') ||
						(ch >= '0' && ch <= '9') )		// and all ch are numbers or letters
					{
						; // noOp
					}
					else
					{
						state = false;		// not a Variable name
						i = iL;				// exit while
					}
				}
			}
			if (state)
			{
				name = st.Substring(0, iL); // get original lower case letters
				Common.DelNPBls(ref st, iL);
				Common.LFindAdd(name, out zI);		// add it to the list if not there and point to it
			}
			return state;
		} // Variable

		public static void SumDDNM(int zI) // z = sum of decimal digits in z
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
				XICommon.SumDDN(zI);
			else
			{
				int cs = Common.nList[zI].Cs;
				int tI;
				Common.InitNAdd(1, "SumDDNM-tI", out tI);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
					{
						SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tI]);
						XICommon.SumDDN(tI);
						SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // SumDDNM

		public static void RepUnitN(int zI) // z = 111...111, 1 repeated z times in baseO
			// R2, R19, R23, R317, and R1031 are the only known repunit primes in base 10
			// the next one is larger than R20000 (1993)
			// Repunit n = (base^n - 1)/(base-1), The Output base is used for this
		{
			double r;	// Rep number
			MultiID.GetDbSI(out r, Common.nList[zI]);
			if (r < 1)
			{
				ClearN(zI);
				return;
			}
			MultiSI b = new MultiSI(1);
			MultiID.SetTo1SI(MultiID.baseO, b); // base
			MultiSI t = new MultiSI(2 + (int)(r * Math.Log10(MultiID.baseI) / MultiID.muDMax));
			MultiID.ClearSI(MultiID.fMB);
			MultiID.PowMBSI(b, Common.nList[zI], t);
			MultiID.RAdd1SI(-1, t);
			MultiID.RDiv1SI(MultiID.baseO - 1, t);
			MultiID.SetToSI(t, Common.nList[zI]);
			MultiID.DoneSI(t);
			MultiID.DoneSI(b);
			NormN(zI);
		} // RepUnitN

		public static void IsSq(int zI) // set z true if z is a perfect square, else false
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
			{
				if (Common.nList[zI].S)
					SetToN(MultiID.sZero, zI);
				else
					PrimeFA.IsSq(zI);
			}
			else
			{
				int cs = Common.nList[zI].Cs;
				int tI;
				Common.InitNAdd(1, "IsSqM-tI", out tI);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
					{
						SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tI]);
						if (Common.nList[tI].S)
							SetToN(MultiID.sZero, tI);
						else
							PrimeFA.IsSq(tI);
						SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // IsSq

		public static void MoebiusMu(int zI) // z = Moebius Mu(z) function
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
				PrimeFA.MoebiusMu(zI);
			else
			{
				int cs = Common.nList[zI].Cs;
				int tI;
				Common.InitNAdd(1, "MoebiusMuM-tI", out tI);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
					{
						SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tI]);
						PrimeFA.MoebiusMu(tI);
						SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // MoebiusMu

		public static void PrimeSearch(int zI) // prime Search
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
				PrimeFA.PrimeSearch(zI);
			else
			{
				int cs = Common.nList[zI].Cs;
				int tI;
				Common.InitNAdd(1, "PrimeSearchM-tI", out tI);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
					{
						SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tI]);
						PrimeFA.PrimeSearch(tI);
						SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // PrimeSearch

		public static void PrimePhi(int zI) // z = Phi(z)
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
				PrimeFA.PrimePhi(zI);
			else
			{
				int cs = Common.nList[zI].Cs;
				int tI;
				Common.InitNAdd(1, "PrimePhiM-tI", out tI);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
					{
						SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tI]);
						PrimeFA.PrimePhi(tI);
						SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // PrimePhi

		public static void PhiLN(int xI, int yI, int zI) // z = PhiL(x, y) =
		// Legendre's formula i.e., the number of integers in [1, x]
		// not divisible by any of the first y primes.
		{
			int xRs = Common.nList[xI].Rs;
			int xCs = Common.nList[xI].Cs;
			int yRs = Common.nList[yI].Rs;
			int yCs = Common.nList[yI].Cs;
			if (xRs == 0 && yRs == 0)
				PrimeFA.PhiLN(xI, yI, zI);
			else if ((xRs != 0 && yRs != 0) && (xRs != yRs || xCs != yCs))
				MultiJD.ErrIncompatible();
			else
			{
				int tI, t2I;
				Common.InitNAdd(1, "PhiLN-tI", out tI);
				Common.InitNAdd(1, "PhiLN-t2I", out t2I);
				if (xRs == 0)
				{
					int xLI;
					Common.InitNAdd(1, "PhiLN-xLI", out xLI);
					SetToII(xI, xLI);
					if (Common.nList[zI].Rs != yRs || Common.nList[zI].Cs != yCs)
						XJCalLst.MatrixN(yRs, yCs, zI);
					for (int r = 0; r < yRs; r++)
						for (int c = 0; c < yCs; c++)
						{
							SetToNM(Common.nList[yI].E[r, c], (MultiSI)Common.nList[t2I]);
							PrimeFA.PhiLN(xLI, t2I, tI);
							SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
					Common.LDelete(xLI);
				}
				else if (yRs == 0)
				{
					int yLI;
					Common.InitNAdd(1, "PhiLN-yLI", out yLI);
					SetToII(yI, yLI);
					if (Common.nList[zI].Rs != xRs || Common.nList[zI].Cs != xCs)
						XJCalLst.MatrixN(xRs, xCs, zI);
					for (int r = 0; r < xRs; r++)
						for (int c = 0; c < xCs; c++)
						{
							SetToNM(Common.nList[xI].E[r, c], (MultiSI)Common.nList[tI]);
							PrimeFA.PhiLN(tI, yLI, tI);
							SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
					Common.LDelete(yLI);
				}
				else
				{
					if (Common.nList[zI].Rs != xRs || Common.nList[zI].Cs != xCs)
						XJCalLst.MatrixN(xRs, xCs, zI);
					for (int r = 0; r < xRs; r++)
						for (int c = 0; c < xCs; c++)
						{
							SetToNM(Common.nList[xI].E[r, c], (MultiSI)Common.nList[tI]);
							SetToNM(Common.nList[yI].E[r, c], (MultiSI)Common.nList[t2I]);
							PrimeFA.PhiLN(tI, t2I, tI);
							SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				Common.LDelete(t2I);
				Common.LDelete(tI);
			}
		} // PhiLN

		public static void PrimRQ(int xI, int yI, int zI) // return z = 1 (true) if x is a primitive root of y, else 0 (false)
		{
			int xRs = Common.nList[xI].Rs;
			int xCs = Common.nList[xI].Cs;
			int yRs = Common.nList[yI].Rs;
			int yCs = Common.nList[yI].Cs;
			if (xRs == 0 && yRs == 0)
				PrimeFA.PrimRQ(xI, yI, zI);
			else if ((xRs != 0 && yRs != 0) && (xRs != yRs || xCs != yCs))
				MultiJD.ErrIncompatible();
			else
			{
				int tI, t2I;
				Common.InitNAdd(1, "PrimRQ-tI", out tI);
				Common.InitNAdd(1, "PrimRQ-t2I", out t2I);
				if (xRs == 0)
				{
					int xLI;
					Common.InitNAdd(1, "PrimRQ-xLI", out xLI);
					SetToII(xI, xLI);
					if (Common.nList[zI].Rs != yRs || Common.nList[zI].Cs != yCs)
						XJCalLst.MatrixN(yRs, yCs, zI);
					for (int r = 0; r < yRs; r++)
						for (int c = 0; c < yCs; c++)
						{
							SetToNM(Common.nList[yI].E[r, c], (MultiSI)Common.nList[t2I]);
							PrimeFA.PrimRQ(xLI, t2I, tI);
							SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
					Common.LDelete(xLI);
				}
				else if (yRs == 0)
				{
					int yLI;
					Common.InitNAdd(1, "PrimRQ-yLI", out yLI);
					SetToII(yI, yLI);
					if (Common.nList[zI].Rs != xRs || Common.nList[zI].Cs != xCs)
						XJCalLst.MatrixN(xRs, xCs, zI);
					for (int r = 0; r < xRs; r++)
						for (int c = 0; c < xCs; c++)
						{
							SetToNM(Common.nList[xI].E[r, c], (MultiSI)Common.nList[tI]);
							PrimeFA.PrimRQ(tI, yLI, tI);
							SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
					Common.LDelete(yLI);
				}
				else
				{
					if (Common.nList[zI].Rs != xRs || Common.nList[zI].Cs != xCs)
						XJCalLst.MatrixN(xRs, xCs, zI);
					for (int r = 0; r < xRs; r++)
						for (int c = 0; c < xCs; c++)
						{
							SetToNM(Common.nList[xI].E[r, c], (MultiSI)Common.nList[tI]);
							SetToNM(Common.nList[yI].E[r, c], (MultiSI)Common.nList[t2I]);
							PrimeFA.PrimRQ(tI, t2I, tI);
							SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				Common.LDelete(t2I);
				Common.LDelete(tI);
			}
		} // PrimRQ

		public static void PrimePiLT(int zI) // z = PiL(z) =
		// Pi(z) = number of primes <= z by Lehmer's formula (Top level called by calculator)
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
				PrimeFA.PrimePiLT(zI);
			else
			{
				int cs = Common.nList[zI].Cs;
				int tI;
				Common.InitNAdd(1, "PrimePiLTM-tI", out tI);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
					{
						SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tI]);
						PrimeFA.PrimePiLT(tI);
						SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // PrimePiLT

		public static void PrimePiL1(int zI) // z = PiL1(z) =
		// Pi(z) = number of primes <= z by Legendre's formula
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
				PrimeFA.PrimePiL1(zI);
			else
			{
				int cs = Common.nList[zI].Cs;
				int tI;
				Common.InitNAdd(1, "PrimePiL1M-tI", out tI);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
					{
						SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tI]);
						PrimeFA.PrimePiL1(tI);
						SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // PrimePiL1

		public static void PrimePiM(int zI) // z = PiM(z) =
		// Pi(z) = number of primes <= z by Meissel's formula
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
				PrimeFA.PrimePiM(zI);
			else
			{
				int cs = Common.nList[zI].Cs;
				int tI;
				Common.InitNAdd(1, "PrimePiMM-tI", out tI);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
					{
						SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tI]);
						PrimeFA.PrimePiM(tI);
						SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // PrimePiM

		public static void Prime(int zI) // return z = 1 if |z| is prime, else 0
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
				PrimeFA.Prime(zI);
			else
			{
				int cs = Common.nList[zI].Cs;
				int tI;
				Common.InitNAdd(1, "PrimeM-tI", out tI);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
					{
						SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tI]);
						PrimeFA.Prime(tI);
						SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // Prime

		public static void PrimR(int zI, bool rPrime) // return z = gen = least primitive root of z modulo z or zero
		// rRrime true => return least primitive prime root of z modulo z or zero
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
				PrimeFA.PrimR(zI, rPrime);
			else
			{
				int rI;
				Common.LFindAdd("Re", out rI);
				if (rI == zI || rI == Common.iTop)
				{
					MultiID.MuWriteErr("Cannot set same location to both cube root and remainder, continuing...");
					return;
				}
				int cs = Common.nList[zI].Cs;
				int tI;
				Common.InitNAdd(1, "PrimeM-tI", out tI);
				MultiJ re = new MultiJ(1, rs, cs);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
					{
						SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tI]);
						PrimeFA.PrimR(tI, rPrime);
						SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						SetToNM((MultiSI)Common.nList[rI], re.E[r, c]);
					}
				Calc.SetToN(re, rI);
				Common.LDelete(tI);
			}
		} // PrimR

		public static void PrimeSig(int zI) // z = Sig(z)
		// Prime Sig function = Sum of the divisors of x
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
				PrimeFA.PrimeSig(zI);
			else
			{
				int cs = Common.nList[zI].Cs;
				int tI;
				Common.InitNAdd(1, "PrimeSigM-tI", out tI);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
					{
						SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tI]);
						PrimeFA.PrimeSig(tI);
						SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // PrimeSig

		public static void PrimeSig0(int zI) // z = Sig(z) - z
		// Prime Sig0 function = Sum of the divisors of x (-x)
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
				PrimeFA.PrimeSig0(zI);
			else
			{
				int cs = Common.nList[zI].Cs;
				int tI;
				Common.InitNAdd(1, "PrimeSig0M-tI", out tI);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
					{
						SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tI]);
						PrimeFA.PrimeSig0(tI);
						SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // PrimeSig0

		public static void SqFree(int zI)
		// set z true if z is a square free, else false 
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
				PrimeFA.SqFree(zI);
			else
			{
				int cs = Common.nList[zI].Cs;
				int tI;
				Common.InitNAdd(1, "SqFreeM-tI", out tI);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
					{
						SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tI]);
						PrimeFA.SqFree(tI);
						SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // SqFree

		public static void PrimeTau(int zI) // z = Tau(z)
		// Prime Tau function = Number of divisors of x
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
				PrimeFA.PrimeTau(zI);
			else
			{
				int cs = Common.nList[zI].Cs;
				int tI;
				Common.InitNAdd(1, "PrimeTauM-tI", out tI);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
					{
						SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tI]);
						PrimeFA.PrimeTau(tI);
						SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // PrimeTau

		public static void RRanMN(int zI) // z = random numbers in matrix from seed RN
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
				RRanN(zI);
			else
			{
				int cs = Common.nList[zI].Cs;
				int tI;
				int rNI;
				Common.LFindAdd("RN", out rNI);
				if (Common.nList[rNI].Rs != 0)
					ClearN(rNI);
				Common.InitNAdd(1, "RRanMN-tI", out tI);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
					{
						SetToNM(Common.nList[rNI], (MultiSI)Common.nList[tI]);
						RRanN(tI);
						if (MultiID.muErr)
						{
							Common.LDelete(tI);
							return;
						}
						SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // RRanMN

		public static void RRanN(int zI) // z = random number from seed z
		{
			int rs = Common.nList[zI].Rs;
			if (rs != 0)
				RRanMN(zI);
			else
			{
				int rNA;
				int rNC;	// constants for the random number generator
				int i;
				int saveFMC;
				int aN = (34 / MultiID.muDMax) + 1;		// Super digits in rNA
				Common.LFind("RNA", out rNA);
				if (rNA > 0)
				{
					if (Common.nList[rNA].N < aN)
						rNA = 0;
				}
				Common.LFind("RNC", out rNC);
				if (rNC > 0)
				{
					if (Common.nList[rNC].N < aN)
						rNC = 0;
				}
				if (rNA == 0)
				{
					Common.LFindAdd("RNA", out rNA);
					ValueN10("1843624,8305725,0753463,7429192,0765341", out i, rNA);
				}
				if (rNC == 0)
				{
					Common.LFindAdd("RNC", out rNC);
					ValueN10("8634647,9732047,9456723,8254463,9625443", out i, rNC);
				}
				saveFMC = fMC;
				fMC = 2 * aN;
				MultiID.mMax = fMC;
				int e = Common.nList[zI].N - aN;
				if (e > 0)
					for (i = 0; i < aN; i++)
						Common.nList[zI].V[i] = Common.nList[zI].V[i + e];
				Common.nList[zI].N = Math.Min(Common.nList[zI].N, aN);
				XICommon.MulN(rNA, zI, zI);	// z = (rNA * z + rNC) % (base^aN)
				XICommon.RAddN(rNC, zI);		// only tested for muDMax = 4, 5, 7, and 8
				if (MultiID.muDMax == 8 && Common.nList[zI].N > 4)
					Common.nList[zI].V[4] %= 1000;
				else if (MultiID.muDMax == 4 && Common.nList[zI].N > 8)
					Common.nList[zI].V[8] %= 1000;
				Common.nList[zI].N = aN;
				Common.nList[zI].S = false;
				MultiID.NormSI(Common.nList[zI]);
				fMC = saveFMC;
				MultiID.mMax = fMC;
				NormN(zI);
				int rNI;						// update seed for the N command
				Common.LFindAdd("RN", out rNI);
				if (rNI != zI) SetToN(Common.nList[zI], rNI);
			}
		} // RRanN

		public static void RanN(int zI) // z = random number from seed RN
		{
			int rNI;		// random number
			Common.LFindAdd("RN", out rNI);
			RRanN(rNI);		// rN = (rNA * rN + rNC) % (10^35)
			SetToII(rNI, zI);
		}

		public static void Ran() // Top = random number from random seed
		{
			int rNI;		// random number
			Common.LFindAdd("RN", out rNI);
			DateTimePicker dateTime = new DateTimePicker();
			string timeSt = dateTime.Value.ToString();
			string st = "";
			double t = MultiID.TimerF();
			timeSt += t.ToString();
			int i;
			for (i = 0; i < timeSt.Length; i++)
			{
				if ('0' <= timeSt[i] && timeSt[i] <= '9')
					st += timeSt[i];
			}
			ValueN10("1" + st + st, out i, rNI);
			XICommon.MulN(rNI, rNI, rNI);
			RRanN(rNI);		// rN = (rNA * rN + rNC) % (10^35)
			SetToII(rNI, Common.iTop);
		} // Ran

		public static void RRIntN(int yI, int zI) // z = Random Integer between y and z inclusive
		{
			int yLI;		// y local
			int zLI;		// z local
			int rNI;		// random number
			int rLI;		// random number local
			int i;
			Common.InitNAdd(Common.nList[yI].N, "RRIntN-yLI", out yLI);
			Common.InitNAdd(Common.nList[zI].N, "RRIntN-zLI", out zLI);
			Common.nList[yLI] &= Common.nList[yI];
			Common.nList[zLI] &= Common.nList[zI];
			if (Common.nList[yI] < Common.nList[zI])
			{
				int tmp = yLI;
				yLI = zLI;
				zLI = tmp;
			}
			Common.LFindAdd("RN", out rNI);
			RRanN(rNI);		// rN = (rNA * rN + rNC) % (10^35)
			Common.InitNAdd(Common.nList[rNI].N, "RRIntN-rLI", out rLI);
			Common.nList[rLI] &= Common.nList[rNI];
			if (rNI == zI || rNI == Common.iTop)
				MultiID.WriteLn("RInt: Warning, changing RN by RInt(X, Y), continuing...");
			XICommon.RSubN(zLI, yLI);
			MultiID.Add1SI(Common.nList[yLI], 1, Common.nList[zI]);
			XICommon.RMulN(rLI, zI);
			ValueN10("1,0000000,0000000,0000000,0000000,0000000", out i, yLI);
			XICommon.DivN(zI, yLI, zI);
			XICommon.RAddN(zLI, zI);		// z = (RN * (y - z + 1) / 10^35) + z, y >= z
			Common.LDelete(rLI);
			if (zLI > yLI)
			{
				Common.LDelete(zLI);
				Common.LDelete(yLI);
			}
			else
			{
				Common.LDelete(yLI);
				Common.LDelete(zLI);
			}
			NormN(zI);
		} // RRIntN

		public static bool Expression(ref string st, int nI) // do an Expression
		{
			string op;
			string op2;
			int tI;
			if (!SimpExpr(ref st, nI)) return false;
			if (st == "") return true;
			op = st.Substring(0, 1);
			op2 = "";
			if (st.Length > 1) op2 = st.Substring(0, 2);
			if (op != "<" && op != "!" && op != "=" && op != ">")
				return true;
			if (op2 == "==" || op2 == "<=" || op2 == "<>" || op2 == "!=" || op2 == ">=")
			{
				Common.DelNPBls(ref st, 2);
			}
			else
			{
				if (op == "!") return true;
				Common.DelNPBls(ref st, 1);
			}
			Common.InitNAdd(fMC, "Expression-tI", out tI);
			if (!SimpExpr(ref st, tI))
			{
				Common.ReportSin("Simple Expression expected: " + op2 + "|", ref st);
			}
			else
			{
				XJCalLst.CompN(Common.nList[tI], Common.nList[nI], op, op2, nI);
			}
			Common.LDelete(tI);
			return true;
		} // Expression

		public static void ReadFNmB(StreamReader inF, out bool oK, int zI, ref string fName, string st)
		// input st is a possible initial input line
		// read in z (baseI) from a file, oK = true; if no I/O errors
		{
			int i;
			char ch;
			bool fini = false;
			string stN = "";
			string st2 = "";
			string maxD = MultiID.MaxDig(MultiID.baseI);
			oK = true;
			try
			{
				do
				{
					if (st == "") st = inF.ReadLine();	// get next line of input
					if (st == null) break;				// null => EOF
					for (i = 1; i <= st.Length; i++)
					{
						ch = st[i-1];
						if (ch >= '0' && ch <= maxD[0] && (ch <= '9' || ch >= 'A')) // get value
						{
							st2 += ch;
							if (st2.Length >= 100)
							{
								stN += st2;
								st2 = "";
							}
						}
						else if (ch != ',')
						{
							if (ch != ' ')
							{
								fini = true;
								break;				// stop if garbage after number
							}
						}
					}
					st = "";
					MultiID.MuInterrupt();
				}
				while (!fini && !MultiID.muAbort);	// not fini ...
				stN += st2;
				ValueN(stN, out i, zI);
			}
			catch (Exception ex)
			{
				// trap read error
				MultiID.WriteLn(" Read error file \"" + fName + '"');
				Common.WCurDir(fName);
				Common.ReportError("ReadFNmB", ex.Message);
				oK = false;
			}
		} // ReadFNmB

		public static void ReadFNm10(StreamReader inF, ref bool oK, int zI, ref string fName, string st)
		// read in z (base 10) from an already opened file, oK = true if no I/O errors
		{
			int i;
			int j = 0;
			int k = 0;
			string ch;
			string stN = "";
			bool fini = false;
			int sD;				// Super digit
			try
			{
				do
				{
					if (st == "") st = inF.ReadLine();			// get next line of input
					if (st == null) break;						// null => EOF
					for (i = 1; i <= st.Length; i++)
					{
						ch = st.Substring(i-1, 1);
						if ('0' <= ch[0] && ch[0] <= '9')	// get value
						{
							j++;
							stN += ch;
							if (j == MultiID.muDMax)
							{
								sD = System.Int32.Parse(stN);
								stN = "";
								j = 0;
								Common.nList[zI].V[k] = sD;
								k++;
							}
						}
						else if (ch != ",")
						{
							if (ch != " ")
							{
								fini = true;
								break;		// stop if garbage after number
							}
						}
					}
					st = "";
					MultiID.MuInterrupt();
				}
				while (!fini && !MultiID.muAbort);
				for (i = 0; i < k / 2; i++)
				{
					sD = Common.nList[zI].V[i];
					Common.nList[zI].V[i] = Common.nList[zI].V[k - 1 - i];
					Common.nList[zI].V[k - 1 - i] = sD;
				}
				if (k > 0) Common.nList[zI].N = k;
				if (j > 0)
				{
					sD = (int)Math.Pow(10, j);
					MultiID.RMul1SI(sD, Common.nList[zI]);
					sD = System.Int32.Parse(stN);
					Common.nList[zI].V[0] += sD;
				}
			}
			catch (Exception ex)
			{
				// trap read error
				MultiID.WriteLn(" Read error file \"" + fName + '"');
				Common.WCurDir(fName);
				Common.ReportError("ReadFNm10", ex.Message);
				oK = false;
			}
		} // ReadFNm10

		public static void ReadFNm(StreamReader inF, out string name, out bool oK, int zI, ref string fName)
		// read in z from an already opened file
		// get its name and base from file, oK = true; if no i/O errors
		{
			name = "";
			oK = false;
			string st;
			st = inF.ReadLine();				// get first line of input file
			if (st == null) return;				// null => EOF, file is empty
			if (st.Length > 0 && st[0] == '@')
			{
				inF.Close();
				int saveBaseI = MultiID.baseI;
				MultiID.baseI = 10;
				CalReadCode(fName);
				MultiID.baseI = saveBaseI;
				name = Common.nList[Common.iTop].Nm;
				oK = false;
				return;
			}
			char ch;
			int i;
			int j = 0;
			int baseIS;
			int baseIL = 0;
			string maxD = "9";
			string stN = "";
			bool fini = false;
			bool vStarted = false;	// false if still looking for first digit
			bool sFound = false;	// false if still looking for a sign
			bool sign = false;
			oK = false;
			name = "";
			MultiID.ReSizeSI(fMC, Common.nList[zI]);	// z = 0, don't change name
			try
			{
				do
				{
					st = inF.ReadLine();	// get next line of input
					if (st == null) break;	// null => EOF
					if (baseIL == 0)
					{
						j = st.ToUpper().IndexOf("BASE("); // look for base(xx
						if (j >= 0)
						{
							stN = st.Substring(j + 5, 2);
							if (stN[1] < '0' || stN[1] > '9')
							{
								stN = stN.Substring(0, 1);
								j--;
							}
							if (stN[0] < '0' || stN[0] > '9')
							{
								baseIL = 10;
								st = "";
							}
							else
							{
								baseIL = System.Int32.Parse(stN);
								if (baseIL < 2 || baseIL > 36) baseIL = 10;
								if (st[j+7] == ')') j++;
								MultiID.Delete(ref st, 0, j + 7);
							}
							maxD = MultiID.MaxDig(baseIL);
						}
					}
					if (name == "" && !vStarted) // no name in file, no problem
					{
						i = st.IndexOf("=");
						if (i > 0)
						{
							name = st.Substring(0, i).Trim(' '); // get name from file just before "="
							MultiID.Delete(ref st, 0, i+1);
						}
					}
					for (i = 1; i <= st.Length; i++)
					{
						ch = st[i-1];
						if (!vStarted) // get sign
						{
							if (!sFound)
							{
								if (ch == '+' || ch == '-')
								{
									sFound = true;
									if (ch == '-') sign = true;
									Common.ReplaceAt(ref st, i-1, ",");
								}
							}
						}
						if (ch >= '0' && ch <= maxD[0] &&
										(ch <= '9' || ch >= 'A')) // get value
						{
							if (i > 1) MultiID.Delete(ref st, 0, i - 1);
							if (baseIL == 0) baseIL = 10;
							oK = true;
							if (baseIL == 10)
							{
								ReadFNm10(inF, ref oK, zI, ref fName, st); // read in base 10
							}
							else
							{
								baseIS = MultiID.baseI;
								MultiID.baseI = baseIL;
								if (!Common.quiet) MultiID.Write("Base " + MultiID.baseI + " ");
								ReadFNmB(inF, out oK, zI, ref fName, st); // read in base baseI from file
								MultiID.baseI = baseIS;
							}
							fini = true;
							break; // exit for
						}
					}
				}
				while (!fini);
				if (!oK) MultiID.WriteLn(" No named number found");
				Common.nList[zI].S = sign;
				NormN(zI);
			}
			catch (Exception ex)
			{
				// trap read error
				MultiID.WriteLn(" Read error file \"" + fName + '"');
				Common.WCurDir(fName);
				Common.ReportError("ReadFNm", ex.Message);
				oK = false;
			}
		} // ReadFNm

		public static bool ReadFile(string fName, int xI)
		// read in a number from a file after opening it
		{
			StreamReader inF;		// file number to read from
			bool oK;
			string name;
			int tI;
			int t2I;
			string dir;
			if (fName == "") fName = "NoName.XJN";
			if (!Common.OpenRead(out inF, fName, out dir)) return false;
			if (!Common.quiet) MultiID.WriteLn("Reading number file \"" + fName + "\" ");
			if (xI != 0)
			{
				ReadFNm(inF, out name, out oK, xI, ref fName); // I Primitive op code
			}
			else
			{
				Common.InitNAdd(1, "ReadFile-tI", out tI);
				ReadFNm(inF, out name, out oK, tI, ref fName); // readN Procedure
				if (oK)
				{
					if (name != "")
					{
						Common.LFindAdd(name, out t2I);
						SetToII(tI, t2I);
					}
					else
					{
						SetToII(tI, Common.iTop);
					}
				}
				Common.LDelete(tI);
			}
			inF.Close();
			if (!Common.quiet)
			{
				if (name != "" && oK) MultiID.Write("containing " + name);
				MultiID.WriteLn();
				MultiID.WriteLn("File \"" + fName + "\" closed");
			}
			return true;
		} // ReadFile

		public static void CalRead(string fName, int xI)
		// determine name and read in a number from a file
		{
			if (fName == "" && xINIn != 0)
				fName = Common.nList[xINIn].Nm.Substring(7);		// remove "XJNIn: "
			if (fName == "" && Common.comI != 0)
				fName = Common.nList[Common.comI].Nm.Substring(6);	// remove "File: "
			if (fName == "") fName = "NoName.XJN";
			Common.FixFName(ref fName, "XJN");
			Common.CalReadN(fName, xI);
		}

		public static void CalWriteN(string fName, int xI)
		// output a number on the list to a file with name fName
		{
			StreamWriter outF; // text file output for Number
			string nm = Common.nList[xI].Nm;
			string st;
			st = " with " + Common.nList[xI].Nm;
			if (fName == "") fName = "NoName.XJN";
			if (!Common.ChkWrite(out outF, fName, st)) return;			// open file, exit if not opened
			XJCalLst.FormatN(outF, nm, Common.nList[xI]);				// output x to file #outF
			outF.Close();												// close the file
			// output status to the screen
			if (!Common.quiet) MultiID.WriteLn("File \"" + fName + "\" closed");
		}

		public static void CalWrite(string fName, int xI)
		// output a number on the list to a file after checking its name
		{
			if (fName == "" && xINOut != 0)
				fName = Common.nList[xINOut].Nm.Substring(8);		// remove "XJNOut: "
			if (fName == "" && Common.comI != 0)
				fName = Common.nList[Common.comI].Nm.Substring(6);	// remove "File: "
			if (fName == "") fName = "NoName.XJN";
			Common.FixFName(ref fName, "XJN");
			CalWriteN(fName, xI);
		}

		public static void CalLFile(ref string fName) // establish log file name
		{
			if (fName == "" && xILOut != 0)
				fName = Common.nList[xILOut].Nm.Substring(8);		// remove "XJLOut: "
			if (fName == "" && Common.comI != 0)
				fName = Common.nList[Common.comI].Nm.Substring(6);	// remove "File: "
			if (fName == "") fName = "NoName.XJL";
			Common.FixFName(ref fName, "XJL");
		}

		public static void CalFileNm(string stNm, ref string st, ref int nI)
		// get File name from command line in st and put it on the list in nI
		{
			string stF = Common.FileNm(ref st);
			string stL = stNm + ": " + stF;
			if (nI == 0)
				Common.InitNAdd(1, stL, out nI);
			else
				Common.nList[nI].Nm = stL;
			if (!Common.quiet)
			{
				MultiID.WriteDLn(Common.nList[nI].Nm);
			}
		}

		public static void CalReadConfig() // read and execute file Config.XJC
		{
			CalReadCode("Config.XJC");
		}

		public static void CalWriteConfig() // write file Config.XJC
		{
			StreamWriter outF; // text file: configuration output
			string st;
			int fMBI;
			string savDir = Environment.CurrentDirectory.ToString();
			Directory.SetCurrentDirectory(Common.homeDir);
			if (Common.ChkWrite(out outF, "Config.XJC", ""))
			{
				MultiID.WriteLn(null, outF, "@Echo Off");
				MultiID.WriteLn(null, outF, "Rem  Start of file Config.XJC");
				MultiID.WriteLn(null, outF, "BaseI(0)");
				MultiID.WriteLn(null, outF, "SetMax(" + MultiID.mMax * MultiID.muDMax + ")");
				MultiID.Write(null, outF, "LastTop=;  ");
				MultiID.Write(null, outF, fMC * MultiID.muDMax + " M;  ");
				MultiID.WriteLn(null, outF, MultiID.muDpG + " G;");
				MultiID.WriteLn(null, outF, "SetC(" + Common.mComs + ")");
				MultiID.WriteLn(null, outF, "SetD(" + MultiID.maxDis + ")");
				if (Common.saveTop) { st = "1"; } else { st = "0"; };
				MultiID.WriteLn(null, outF, "SaveTop(" + st + ")");
				if (MultiID.allowFHT) { st = "1"; } else { st = "0"; };
				MultiID.WriteLn(null, outF, "AllowFHT(" + st + ")");
				if (MultiID.forceFHT) { st = "1"; } else { st = "0"; };
				MultiID.WriteLn(null, outF, "ForceFHT(" + st + ")");
				if (MultiID.diagOn) { st = "1"; } else { st = "0"; };
				if (MultiID.debug) st = "-1";
				MultiID.WriteLn(null, outF, "Diag(" + st + ")");
				if (MultiID.timeIt && !MultiID.diagOn)
					MultiID.WriteLn(null, outF, "Time");
				if (Common.autoD) { st = "1"; } else { st = "0";};
				MultiID.WriteLn(null, outF, "AutoDisplay(" + st + ")");
				MultiID.WriteLn(null, outF, "ChDir(\"" + savDir + "\")");
				Common.LFind("FMB", out fMBI);
				if (fMBI != 0)
				{
					CalWriteN("FMB.XJN", fMBI);
					MultiID.WriteLn(null, outF, "ReadN(\"FMB.XJN\")");
				}
				if (xICIn != 0)
				{
					st = Common.nList[xICIn].Nm.Substring(7);	// remove "XJCIn: "
					MultiID.WriteLn(null, outF, "XJCIn(\"" + st + "\")");
				}
				if (xILOut != 0)
				{
					st = Common.nList[xILOut].Nm.Substring(8);	// remove "XJLOut: "
					MultiID.WriteLn(null, outF, "XJLOut(\"" + st + "\")");
				}
				if (xINIn != 0)
				{
					st = Common.nList[xINIn].Nm.Substring(7);	// remove "XJNIn: "
					MultiID.WriteLn(null, outF, "XJNIn(\"" + st + "\")");
				}
				if (xINOut != 0)
				{
					st = Common.nList[xINOut].Nm.Substring(8);	// remove "XJNOut: "
					MultiID.WriteLn(null, outF, "XJNOut(\"" + st + "\")");
				}
				if (Common.comI != 0)
				{
					st = Common.nList[Common.comI].Nm.Substring(6);	// remove "File: "
					MultiID.WriteLn(null, outF, "\"" + st + "\"");
				}
				if (Common.lrnL != 0)
				{
					st = Common.nList[Common.lrnL].Nm.Substring(5); // remove "Lrn: "
					MultiID.WriteLn(null, outF, "X " + st);
				}
				if (MultiID.logg) { st = "1"; } else { st = "0"; }
				MultiID.WriteLn(null, outF, "LogScreen(" + st + ")");
				MultiID.WriteLn(null, outF, "BaseO(" + MultiID.baseO + ")");
				MultiID.WriteLn(null, outF, "BaseI(" + MultiID.baseI + ")");
				MultiID.WriteLn(null, outF, Common.nList[Common.iTop].Nm + "=");
				if (Common.quiet) { st = "1"; } else { st = "0"; }
				MultiID.WriteLn(null, outF, "Quiet(" + st + ")");
				MultiID.WriteLn(null, outF, "Pri(" + Common.pri + ")");
				MultiID.WriteLn(null, outF, "Rem  End of file Config.XJC");
				if (Common.echoB) { st = "On"; } else { st = "Off"; }
				MultiID.WriteLn(null, outF, "Echo " + st);
				outF.Close();
				if (!Common.quiet) MultiID.WriteLn("File Config.XJC closed");
			}
			Directory.SetCurrentDirectory(savDir);
		} // CalWriteConfig

		public static bool AProcedure(ref string st, ref int resSav, ref string fName) // do a procedure
		{
			int po;
			int tot;
			string stL;
			int tI;
			double db;
			string name = Common.GetField(ref st);
			int i = name.Length;
			resSav = 0;
			fName = "";
			if (i == 0 || MultiID.muErr) return false;
			if (st.Length > i && st[i] == '=') return false; // if name= assignment
			if (name != "ALLD" &&
				name != "ALLOWFHT" &&
				name != "AUTODISPLAY" &&
				name != "BASE" &&
				name != "BASEI" &&
				name != "BASEO" &&
				name != "BERNM" &&
				name != "CHDIR" &&
				name != "CLEARBERN" &&
				name != "CLEAREULER" &&
				name != "CLEARHIST" &&
				name != "CLEARLOG" &&
				name != "D1LINE" &&
				name != "DBOOL" &&
				name != "DDIG" &&
				name != "DIAG" &&
				name != "ECHOSCREEN" &&	// same as LogScreen
				name != "EULERM" &&
				name != "EXIT" &&
				name != "FORCEFHT" &&
				name != "GENBERN" &&
				name != "GENEULER" &&
				name != "HISTH" &&
				name != "HISTW" &&
				name != "HELPH" &&
				name != "HELPW" &&
				name != "LOGSCREEN" &&
				name != "LT" &&
				name != "LX" &&
				name != "NEXT" &&		// "PAUSE" is in batch processing
				name != "PFA" &&
				name != "PFB" &&
				name != "PFE" &&
				name != "PTAB" &&
				name != "PRI" &&
				name != "QUIET" &&
				name != "RAN" &&
				name != "READN" &&
				name != "RESTORE" &&
				name != "RUN" &&
				name != "RUNH" &&
				name != "RUNW" &&
				name != "SAVE" &&
				name != "SAVETOP" &&
				name != "SETC" &&
				name != "SETD" &&
				name != "SETM" &&
				name != "SETMAX" &&
				name != "TIME" &&
				name != "WRITE" &&
				name != "WRITELN" &&
				name != "WRITEN" &&
				name != "XJCIN" &&
				name != "XJLOUT" &&
				name != "XJNIN" &&
				name != "XJNOUT")
			{
				return false;
			}
			Common.DelNPBls(ref st, i);		// if name never uses ()
			if (name == "BERNM" || name == "CLEARBERN" || name == "CLEAREULER" ||
				name == "CLEARHIST" || 	name == "CLEARLOG" || name == "EULERM" ||
				name == "EXIT" || name == "LT" || name == "LX" ||
				name == "NEXT" || name == "RAN" || name == "RESTORE" ||
				name == "SAVE" || name == "TIME")
			{
				if (st != "" && st[0] == '(') // allow use of ( or () but not (x)
				{
					Common.DelNPBls(ref st, 1);
					if (st != "" && st[0] == ')') Common.DelNPBls(ref st, 1);
				}
			}
			if (name == "BERNM")	// return x = index of max Bernoulli number in storage
			{
				MultiJD.SetTo1J(2 * Common.nBern, Common.nList[Common.iTop]);
				NormN(Common.iTop);
				return true;
			}
			if (name == "CLEARBERN")
			{
				Common.nBern = -1; // force message output
				Common.ClearBern();
				return true;
			}
			if (name == "CLEAREULER")
			{
				Common.nEuler = -1; // force message output
				Common.ClearEuler();
				return true;
			}
			if (name == "CLEARHIST")
			{
				Common.ClearHist();
				return true;
			}
			if (name == "CLEARLOG")
			{
				Common.ClearLog();
				return true;
			}
			if (name == "EULERM")	// return x = index of max Euler number in storage
			{
				MultiJD.SetTo1J(2 * Common.nEuler, Common.nList[Common.iTop]);
				NormN(Common.iTop);
				return true;
			}
			if (name == "EXIT")	// totally quit and exit the program
			{
				MultiID.muAbort = true;
				Common.tQuitReq = true;
				Common.tExitFlag = true;
				return true;
			}
			if (name == "LX" || name == "LT")
			{
				Common.LFindAdd("LastTop", out Common.lastTop);
				SetToII(Common.lastTop, Common.iTop);
				return true;
			}
			if (name == "NEXT")
			{
				Common.iTop--;
				if (Common.iTop < 1) Common.iTop = Common.cList;
				return true;
			}
			if (name == "RAN")
			{
				Ran();
				return true;
			}
			if (name == "RESTORE")
			{
				resSav = 1;
				return true;
			}
			if (name == "SAVE")
			{
				resSav = 2;
				return true;
			}
			if ( (name == "XJCIN") || (name == "XJLOUT") || (name == "CHDIR") ||
				 (name == "XJNIN") || (name == "XJNOUT") ||
				 (name == "READN") || (name == "WRITEN") || (name == "RUN") )
			{
				tI = 0;
				if (st != "" && st[0] == '(')
				{
					Common.DelNPBls(ref st, 1);
					if (st != "" && st[0] == ')')
						Common.DelNPBls(ref st, 1);
					else if (st != "" && st[0] != '\"')
					{
						if (name == "CHDIR")
							Common.ReportSin("Directory name expected: " + name + "(|", ref st);
						else
							Common.ReportSin("File name expected: " + name + "(|", ref st);
						return true;
					}
					else
					{
						if (name == "CHDIR") Common.ChangeDir(ref st);
						if (name == "XJCIN") CalFileNm("XJCIn", ref st, ref xICIn);
						if (name == "XJLOUT") CalFileNm("XJLOut", ref st, ref xILOut);
						if (name == "XJNIN") CalFileNm("XJNIn", ref st, ref xINIn);
						if (name == "XJNOUT") CalFileNm("XJNOut", ref st, ref xINOut);
						if (name == "READN") CalRead(Common.FileNm(ref st), tI); // tI is nil
						if (name == "WRITEN") CalWrite(Common.FileNm(ref st), Common.iTop);
						if (name == "RUN")
						{
							fName = Common.FileNm(ref st);
							resSav = 3;
						}
						Common.DelNPBls(ref st, 1);
						if (st != "" && st[0] == ')') Common.DelNPBls(ref st, 1);
						return true;
					}
				}
				if (Common.comI == 0) stL = "\"";
				else stL = "\"" + Common.nList[Common.comI].Nm.Substring(5) + "\"";
				if (name == "CHDIR") Common.ChangeDir(ref stL);
				if (name == "XJCIN") CalFileNm("XJCIn", ref stL, ref xICIn);
				if (name == "XJLOUT") CalFileNm("XJLOut", ref stL, ref xILOut);
				if (name == "XJNIN") CalFileNm("XJNIn", ref stL, ref xINIn);
				if (name == "XJNOUT") CalFileNm("XJNOut", ref stL, ref xINOut);
				if (name == "READN") CalRead(Common.FileNm(ref stL), tI);
				if (name == "WRITEN") CalWrite(Common.FileNm(ref stL), Common.iTop);
				if (name == "RUN")
				{
					fName = Common.FileNm(ref stL);
					resSav = 3;
				}
				return true;
			}
			if (name != "WRITE" && name != "WRITELN")
			{
				if (st == "" || st[0] != '(')
				{
					if (name == "DIAG")
					{
						if (!MultiID.timeIt) Common.DoubleSpace();
						MultiID.debug = false;
						MultiID.diagOn = true;
						MultiID.timeIt = true;
					}
					else if (name == "TIME")
					{
						if (!MultiID.timeIt) Common.DoubleSpace();
						MultiID.debug = false;
						MultiID.diagOn = false;
						MultiID.timeIt = true;
					}
					else if (name == "PRI")
						Common.DisplayPriority(); // display priority status
					else if (name == "QUIET")
						Common.quiet = true;
					else
						Common.ReportSin("( expected: " + name + "|", ref st);
					return true;
				}
				else
				{
					Common.DelNPBls(ref st, 1);
					Common.InitNAdd(fMC, "AProcedure-tI-1", out tI);
					if (!Expression(ref st, tI))
					{
						Common.LDelete(tI);
						Common.ReportSin("Expression expected: " + name + "(|", ref st);
						return true;
					}
					if (st != "" && st[0] == ')') Common.DelNPBls(ref st, 1);
					if (MultiID.muErr)
					{
						Common.LDelete(tI);
						return true;
					}
					if (name == "ALLD")
					{
						if (Common.nList[tI].Rs != 0)
							MultiJD.ErrNotScalar();
						else
							PrimeFA.PrimeAllD(tI);
					}
					else if (name == "ALLOWFHT")
					{
						if (Common.nList[tI].Rs != 0)
							MultiJD.ErrNotScalar();
						else
						{
							MultiID.allowFHT = !Common.ZTestN(tI);
							if (!MultiID.allowFHT) MultiID.forceFHT = false;
						}
					}
					else if (name == "AUTODISPLAY")
						if (Common.nList[tI].Rs != 0)
							MultiJD.ErrNotScalar();
						else
							Common.autoD = !Common.ZTestN(tI);
					else if (name == "BASE")
					{
						CalSBaseI(tI);
						CalSBaseO(tI);
					}
					else if (name == "BASEI")
						CalSBaseI(tI);
					else if (name == "BASEO")
						CalSBaseO(tI);
					else if (name == "D1LINE")
						MultiID.d1LineOn = !Common.ZTestN(tI);
					else if (name == "DBOOL")
						MultiID.dBoolOn = !Common.ZTestN(tI);
					else if (name == "DDIG")
						MultiID.dDigOn = !Common.ZTestN(tI);
					else if (name == "DIAG")
					{
						if (Common.nList[tI].Rs != 0)
							MultiJD.ErrNotScalar();
						else
						{
							if (!Common.ZTestN(tI) && !MultiID.timeIt) Common.DoubleSpace();
							MultiID.diagOn = !Common.ZTestN(tI);
							MultiID.debug = Common.nList[tI].S;
							MultiID.timeIt = MultiID.diagOn;
						}
					}
					else if (name == "FORCEFHT")
					{
						if (Common.nList[tI].Rs != 0)
							MultiJD.ErrNotScalar();
						else
						{
							MultiID.forceFHT = !Common.ZTestN(tI);
							if (MultiID.forceFHT) MultiID.allowFHT = true;
						}
					}
					else if (name == "GENBERN")
						GenBern(tI);
					else if (name == "GENEULER")
						GenEuler(tI);
					else if (name == "HELPH")
					{
						if (Common.nList[tI].Rs != 0)
							MultiJD.ErrNotScalar();
						else
						{
							MultiJD.GetDbInJ(out db, 100, 5000, Common.nList[tI]);
							Global.objRunForm.SetHelpHeight((int)db);
						}
					}
					else if (name == "HELPW")
					{
						if (Common.nList[tI].Rs != 0)
							MultiJD.ErrNotScalar();
						else
						{
							MultiJD.GetDbInJ(out db, 100, 5000, Common.nList[tI]);
							Global.objRunForm.SetHelpWidth((int)db);
						}
					}
					else if (name == "HISTH")
					{
						if (Common.nList[tI].Rs != 0)
							MultiJD.ErrNotScalar();
						else
						{
							MultiJD.GetDbInJ(out db, 100, 5000, Common.nList[tI]);
							Global.objRunForm.SetHistHeight((int)db);
						}
					}
					else if (name == "HISTW")
					{
						if (Common.nList[tI].Rs != 0)
							MultiJD.ErrNotScalar();
						else
						{
							MultiJD.GetDbInJ(out db, 100, 5000, Common.nList[tI]);
							Global.objRunForm.SetHistWidth((int)db);
						}
					}
					else if (name == "LOGSCREEN" || name == "ECHOSCREEN")
					{
						if (Common.nList[tI].Rs != 0)
							MultiJD.ErrNotScalar();
						else
							if (MultiID.logg == Common.ZTestN(tI)) Common.ToggleLogg();
					}
					else if (name == "PFA")
						if (Common.nList[tI].Rs != 0)
							MultiJD.ErrNotScalar();
						else
						{
							PrimeFA.PrimeFAA(tI);
						}
					else if (name == "PFB")
						if (Common.nList[tI].Rs != 0)
							MultiJD.ErrNotScalar();
						else
						{
							XICommon.PrimeFAB(tI);
						}
					else if (name == "PFE")
						if (Common.nList[tI].Rs != 0)
							MultiJD.ErrNotScalar();
						else
						{
							PrimeFA.PrimeFAE(tI);
						}
					else if (name == "PTAB")
					{
						if (Common.nList[tI].Rs != 0)
							MultiJD.ErrNotScalar();
						else
						{
							MultiJD.GetDbInJ(out db, 2, Int32.MaxValue, Common.nList[tI]);
							XICommon.GenPrimeTable((int)db);
						}
					}
					else if (name == "PRI")
					{
						if (Common.nList[tI].Rs != 0)
							MultiJD.ErrNotScalar();
						else
						{
							MultiJD.GetDbInJ(out db, -3, 3, Common.nList[tI]); // 0 for normal priority
							Common.SetPriority((int)db);
							Common.DisplayPriority();
						}
					}
					else if (name == "QUIET")
						if (Common.nList[tI].Rs != 0)
							MultiJD.ErrNotScalar();
						else
							Common.quiet = !Common.ZTestN(tI);
					else if (name == "RUNH")
					{
						if (Common.nList[tI].Rs != 0)
							MultiJD.ErrNotScalar();
						else
						{
							MultiJD.GetDbInJ(out db, 100, 5000, Common.nList[tI]);
							Global.objRunForm.SetRunHeight((int)db);
						}
					}
					else if (name == "RUNW")
					{
						if (Common.nList[tI].Rs != 0)
							MultiJD.ErrNotScalar();
						else
						{
							MultiJD.GetDbInJ(out db, 100, 5000, Common.nList[tI]);
							Global.objRunForm.SetRunWidth((int)db);
						}
					}
					else if (name == "SAVETOP")
						if (Common.nList[tI].Rs != 0)
							MultiJD.ErrNotScalar();
						else
							Common.saveTop = !Common.ZTestN(tI);
					else if (name == "SETC")
					{
						if (Common.nList[tI].Rs != 0)
							MultiJD.ErrNotScalar();
						else
						{
							MultiJD.GetDbInJ(out db, 0, 100000, Common.nList[tI]); // 0 for no change status
							Common.ReSizeHist((int)db);
						}
					}
					else if (name == "SETD")
					{
						if (Common.nList[tI].Rs != 0)
							MultiJD.ErrNotScalar();
						else
						{
							MultiJD.GetDbInJ(out db, 14, Common.nMax * MultiID.muDMax, Common.nList[tI]);
							MultiID.maxDis = (int)db;
						}
					}
					else if (name == "SETM")
						CalSMan(tI);
					else if (name == "SETMAX")
						CalSMan(tI);
					Common.LDelete(tI);
					return true;
				}
			}
			if (st != "" && st[0] == '(') // process Write and WriteLn
			{
				Common.DelNPBls(ref st, 1);
				if (st != "")
				{
					tot = 0;
					if (st != "" && st[0] == ')')
						Common.DelNPBls(ref st, 1);
					else if (st != "" && st[0] == '\"')
					{
						po = st.IndexOf("\"", 1);
						if (po < 0) po = st.Length;
						MultiID.WriteT(st.Substring(1, po - 1), ref tot);
						Common.DelNPBls(ref st, po+1);	// 1 too many ok for DelNPBls
					}
					else
					{
						Common.InitNAdd(fMC, "AProcedure-tI-2", out tI);
						if (!Expression(ref st, tI))
						{
							Common.LDelete(tI);
							Common.ReportSin("Expression expected: " + name + "(|", ref st);
							return true;
						}
						MultiJD.WriteJ(Common.nList[tI], ref tot);
						Common.LDelete(tI);
					}
					if (st != "" && st[0] == ')') Common.DelNPBls(ref st, 1);
				}
			}
			if (name == "WRITELN") MultiID.WriteLn();
			return true;
		} // AProcedure

		public static bool AssignmentOp(string op) // is this an assignment?
		{
			return (op == "+=" || op == "-=" || op == "*=" || op == "@=" || op == "/=" || op == "%=");
		}

		public static bool Assignment(ref string st, ref int nI)
		// do an assignment, nI unchanged if not an assignment
		{
			int pI; // pointer to left member of assignment statement
			int tI;
			string name = Common.GetField(ref st);
			int iL = name.Length;
			if (iL == 0 || iL == st.Length) return false;
			if (st.Length > iL+2)
			{
				string op = st.Substring(iL, 2);
				if (AssignmentOp(op))
				{
					op = op[0] + "(";
					st = name + "=" + name + op + st.Substring(iL+2);
				}
			}
			if (st[iL] == '=')
			{
				if (Variable(ref st, out pI)) // pI points to new top, may be the same as old top
				{
					Common.DelNPBls(ref st, 1);		// delete the = sign
					nI = pI;
					if (st.Length == 0)
					{
						return true;				// merely change top
					}
					Common.InitNAdd(fMC, "Assignment-tI", out tI);
					if (Expression(ref st, tI))
					{
						SetToII(tI, nI);
						Common.LDelete(tI);
						return true;
					}
					Common.LDelete(tI);
				}
			}
			return false;
		} // Assignment

		public static void ListReport() // report the contents of the nList
		// output name and value for each item on the list
		{
			int nI;
			int mI;
			int uI;
			int tot;
			listOn = true;
			int uB = Common.cList;
			int cI = uB;
			MultiID.WriteDLn(Common.StringOf(40, "-"));
			while (cI >= 1)
			{
				tot = 0;
				MultiID.WriteT(cI + ": ", ref tot);
				Common.WriteNV(Common.nList[cI], ref tot);
				if (MultiID.diagOn)
				{
					if (Common.nList[cI].Rs > 0)
					{
						tot = 0;
						MultiID.WriteLn();
						MultiID.WriteT("Scalar Part = ", ref tot);
						MultiID.WriteSI(Common.nList[cI], ref tot);
					}
					nI = Common.nList[cI].N;
					mI = Common.nList[cI].M;
					uI = Common.nList[cI].V.GetUpperBound(0);
					MultiID.WriteT(" N=" + nI + " M=" + mI + " U=" + uI, ref tot);
				}
				MultiID.WriteLn();
				if (cI > 1) MultiID.WriteLn(); // double space between values
				cI--;
			}
			MultiID.WriteLn(Common.StringOf(40, "-"));
			listOn = false;
		}

		public static void CalReadCode(string fName) // read in a Cal code file and Run it
		{
			if (fName == "" && xICIn != 0)
				fName = Common.nList[xICIn].Nm.Substring(7);		// remove "XJCIn: "
			if (fName == "" && Common.comI != 0)
				fName = Common.nList[Common.comI].Nm.Substring(6);	// remove "File: "
			if (fName == "") fName = "NoName.XJC";
			Common.FixFName(ref fName, "XJC");
			if (!Common.RunFile(fName))
			{
				if (Common.MakeReady() == DialogResult.OK)			// pause for operator to make file/drive ready
				{
					if (!Common.RunFile(fName))
						MultiID.WriteLn("Aborting file input...");
				}
				MultiID.WriteLn();
			}
		}

		public static void CalDel(int nI) // delete nI from list
		// don't ever delete x at nI = 1, careful, nI and iTop may be the same Integer
		{
			if (nI > 1)
			{
				if (nI == Common.comI) Common.comI = 0;
				if (nI == Common.lrnL) Common.lrnL = 0;
				if (nI == xICIn) xICIn = 0;
				if (nI == xILOut) xILOut = 0;
				if (nI == xINIn) xINIn = 0;
				if (nI == xINOut) xINOut = 0;
				Common.LDelete(nI);
				if (Common.comI > nI) Common.comI--;
				if (Common.lrnL > nI) Common.lrnL--;
				if (xICIn > nI) xICIn--;
				if (xILOut > nI) xILOut--;
				if (xINIn > nI) xINIn--;
				if (xINOut > nI) xINOut--;
				if (Common.iTop >= nI) Common.iTop--;
			}
			else
			{
				if (Common.iTop == 1) Common.iTop = Common.cList;
			}
		}

		public static void CalComment(ref string st) // get Comment from command line in st
		{
			CalFileNm("File", ref st, ref Common.comI);
		}

		public static void CalSDpG(int nI)
		// set digits per group in number output display [0, nMax * muDMax], 0 => no ,'s
		{
			if (Common.nList[nI].Rs != 0)
				MultiJD.ErrNotScalar();
			else
			{
				double db;
				MultiJD.GetDbInJ(out db, 0, Common.nMax * MultiID.muDMax, Common.nList[nI]);
				MultiID.muDpG = (int)db;
			}
		}

		public static void CalSBaseO(int nI)
		// set Display base in input and output [2, 36], 0 => 10, 'A' = 10, 'Z' = 35
		{
			if (Common.nList[nI].Rs != 0)
				MultiJD.ErrNotScalar();
			else
			{
				double db;
				string maxD; // max Input digit "1" to "Z"
				MultiJD.GetDbInJ(out db, 0, 36, Common.nList[nI]);
				if (db < 2) db = 10;
				MultiID.baseO = (int)db;
				maxD = MultiID.MaxDig(MultiID.baseO);
				if (!Common.quiet) MultiID.WriteLn("BaseO = " + MultiID.baseO + " base ten, Max digit = " + maxD);
			}
		}

		public static void CalSBaseI(int nI)
		// set Display base in input and output [2, 36], 0 => 10, 'A' = 10, 'Z' = 35
		{
			if (Common.nList[nI].Rs != 0)
				MultiJD.ErrNotScalar();
			else
			{
				double db;
				string maxD; // max Input digit "1" to "Z"
				MultiJD.GetDbInJ(out db, 0, 36, Common.nList[nI]);
				if (db < 2) db = 10;
				MultiID.baseI = (int)db;
				maxD = MultiID.MaxDig(MultiID.baseI);
				if (!Common.quiet) MultiID.WriteLn("BaseI = " + MultiID.baseI + " base ten, Max digit = " + maxD);
			}
		}

		public static void SetNFMB(int nI) // set Modulo base FMB on Common.nList = value of nI
		{
			int fMBI;
			Common.LFindAdd("FMB", out fMBI);
			if (fMBI == nI) return;
			SetToII(nI, fMBI);
		}

		public static void SetFMB() // set Modulo base FMB in MultiID package to FMB on the Common.nList
		{
			int fMBI;
			Common.LFindAdd("FMB", out fMBI);
			MultiID.SetToSIN(Common.nList[fMBI], MultiID.fMB);
		}

		public static void CalExxLrn(int nI) // setup to execute learned line x times
		{
			if (Common.nList[nI].Rs != 0)
				MultiJD.ErrNotScalar();
			else
			{
				double db;
				MultiJD.GetDbInJ(out db, 0, Int32.MaxValue, Common.nList[nI]);
				Common.exX = (int)db;
			}
		}

		public static bool Primitive(ref string st, int nI) // do operation
		{
			if (MultiID.muErr) return false;
			string op = st.Substring(0, 1).ToUpper();	// operation code character "+", "S", etc.
			if (op == " " || op == ";")					// " " and ";" never get this far
				{} // noOp
			else if (op == "A")
				Common.autoD = !Common.autoD;
			else if (op == "B")
				Common.CalDLrn(ref st);
			else if (op == "C")
				XJCalLst.ChSMN(nI);
			else if (op == "F" || op == "!")
				XJCalLst.RFacN(nI);
			else if (op == "#")
				XJCalLst.Primorial(nI);
			else if (op == "G")
				CalSDpG(nI);
			else if (op == "H")
				Common.ToggleLogg();
			else if (op == "I")
				CalRead("", Common.iTop);
			else if (op == "J")
				CalReadCode("");
			else if (op == "K")
				CalExxLrn(nI);
			else if (op == "M")
				CalSMan(nI);
			else if (op == "N")
				RanN(nI);
			else if (op == "Q")
			{
				MultiID.muAbort = true;
				Common.tQuitReq = true;
			}
			else if (op == "R")
				XJCalLst.SqRootN(Common.nList[nI], nI);
			else if (op == "S")
				XJCalLst.MulN(Common.nList[nI], Common.nList[nI], nI);
			else if (op == "@")
				XJCalLst.MulEN(Common.nList[nI], Common.nList[nI], nI);
			else if (op == "W")
				CalWrite("", Common.iTop);
			else if (op == "X")
				Common.CalExLrn(ref st);
			else if (op == "Y")
				CalDel(nI);
			else if (op == "Z")
				ListReport();
			else if (op == "\"")
				CalComment(ref st);
			else if (op == "%")
				SetNFMB(nI);
			else if (op == "/")
				XJCalLst.ModFMBN(Common.nList[nI], nI);
			else if (op == @"\")
			{
				CalSBaseI(nI);
				CalSBaseO(nI);
			}
			else if (op == "$")
			{
				RunForm.ClearOutput();
				new Calc();				// re-initialize class Calc
			}
			else if (op == ">")
				CalWriteConfig();
			else if (op == "<")
			{
				st = "";
				CalReadConfig();
			}
			else if (op == "]")
				Common.SaveHist();
			else if (op == "[")
				Common.RestoreHist();
			else if (op == "?")
				RunForm.HelpForm();
			else
			{
				return false;
			}
			Common.DelNPBls(ref st, 1);
			return true;
		} // Primitive

		public static void GamN(MultiSI x, int zI) // z = Gamma(x) = (x-1)! = 1*2*3*...*(x-1)
		{
			MultiID.ClearSI(MultiID.fMB);
			MultiID.Add1SI(x, -1, Common.nList[zI]);
			MultiID.FacMBiSI(Common.nList[zI], Common.nList[zI]);
		}

		public static void FibN(MultiSI x, int zI) // z = FibonacciNumber(x)
		{
			FibSI(x, Common.nList[zI]);
		}

		public static void FibSI(MultiSI x, MultiJ z)
		// z = FibonacciNumber(x), Fib(0) = 0
		// x may have the same address in memory as z
		// x is not changed unless it shares memory with z
		// sets muErr true if overflow or error
		{
			int maxFib = Int32.MaxValue;
			bool sign = x.S;
			x.S = false;
			double db;
			MultiID.GetDbInSI(out db, 0, (double)(maxFib) + 1.0, x);
			x.S = sign;
			if (db > maxFib)	// n too large
			{
				Common.ReportError("FibSI:", "n > " + maxFib + ", too large for Fibonacci number");
				MultiID.muErr = true;
				z &= 0;			// n too large
				return;
			}
			int n = (int)db;
			if (n == 0) { z &= 0; return; }
			if (n == 1) { z &= 1; return; }
			if (n == 2) { z &= 1; z.S = sign; return; }
			MultiSI a = new MultiSI(MultiID.mMax);
			MultiSI b = new MultiSI(MultiID.mMax);
			MultiSI c = new MultiSI(MultiID.mMax);
			a &= 1;
			b &= 1;
			for (int i = 2; i < n; i++)
			{
				MultiID.AddSI(a, b, c);
				MultiID.SetToSI(b, a);
				MultiID.SetToSI(c, b);
				MultiID.MuInterrupt();
				if (MultiID.muAbort) break; // exit for
				if (MultiID.statReq2)
				{
					MultiID.nDone = i - 1;
					MultiID.nToGo = n - i - 1;
					MultiID.WritenDone("FibSI:");
				}
			}
			if ((n & 1) == 1) sign = false;		// n odd => answer > 0
			z &= c;
			z.S = sign;
			if (MultiID.muAbort) z &= 0;
			MultiID.DoneSI(c);
			MultiID.DoneSI(b);
			MultiID.DoneSI(a);
		} // FibSI

		public static void SignM(int zI) // z = sign(z) = -1 if z < 0, 0 if z = 0, +1 if z > 0
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
				Sign(zI);
			else
			{
				int cs = Common.nList[zI].Cs;
				int tI;
				Common.InitNAdd(1, "SignM-tI", out tI);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
					{
						SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tI]);
						Sign(tI);
						SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // SignM

		public static void Sign(int zI)	// z = sign(z) = -1 if z < 0, 0 if z = 0, +1 if z > 0
		{
			int rs = Common.nList[zI].Rs;
			if (rs != 0)
				SignM(zI);
			else
			{
				int sign = MultiID.SignSI(Common.nList[zI]);
				MultiID.SetTo1SI(sign, Common.nList[zI]);
				NormN(zI);
			}
		} // Sign

		public static void PrimePTM(int zI) // P(z) = The z'th prime (Called from calculator)
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
				PrimePT(zI);
			else
			{
				int cs = Common.nList[zI].Cs;
				int tI;
				Common.InitNAdd(1, "PrimePTM-tI", out tI);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
					{
						SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tI]);
						PrimePT(tI);
						SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // PrimePTM

		public static void PrimePT(int zI)	// P(z) = The z'th prime (Called from calculator)
		{
			int rs = Common.nList[zI].Rs;
			if (rs != 0)
				PrimePTM(zI);
			else
				XICommon.PrimePT(zI);
		} // PrimePT

		public static void PrimePiTM(int zI)
		// prime counting function Pi(z) = number of primes <= z (Called from calculator)
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
				PrimePiT(zI);
			else
			{
				int cs = Common.nList[zI].Cs;
				int tI;
				Common.InitNAdd(1, "PrimePiTM-tI", out tI);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
					{
						SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tI]);
						PrimePiT(tI);
						SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
					Common.LDelete(tI);
			}
		} // PrimePiTM
		
		public static void PrimePiT(int zI)
		// prime counting function Pi(z) = number of primes <= z (Called from calculator)
		{
			int rs = Common.nList[zI].Rs;
			if (rs != 0)
				PrimePiTM(zI);
			else
			{
				MultiID.ADebug("Prime Pi Function:");
				if (!PrimeFA.pTabFilled) PrimeFA.FillPTab();
				Common.nList[zI].S = false;
				XICommon.PrimePi(zI);
				NormN(zI);
			}
		} // PrimePiT

		public static void GenBern(int nI)
		// generate even numbered Bernoulli numbers B(0) thru B(nI) = bernN[i/2] / bernD[i/2]
		{
			if (Common.nList[nI].Rs != 0)
				MultiJD.ErrNotScalar();
			else
			{
				double db;
				MultiJD.GetDbInJ(out db, 4, Common.maxBern + 1, Common.nList[nI]);
				int n = (int)db;
				if (n > Common.maxBern) // n too large
					Common.ReportError("GenBern", "n > " + Common.maxBern + ", too large for Bernoulli number");
				else
					Common.GenBernI(n);
				MultiID.muErr = false; // because no value is returned to nList[nI]
			}
		}

		public static void GenEuler(int nI)
		// generate even numbered Euler numbers E(0) thru E(nI) = euler[i/2]
		{
			if (Common.nList[nI].Rs != 0)
				MultiJD.ErrNotScalar();
			else
			{
				double db;
				MultiJD.GetDbInJ(out db, 4, Common.maxEuler + 1, Common.nList[nI]);
				int n = (int)db;
				if (n > Common.maxEuler) // n too large
					Common.ReportError("GenEuler", "n > " + Common.maxEuler + ", too large for Euler number");
				else
					Common.GenEulerI(n);
				MultiID.muErr = false; // because no value is returned to nList[nI]
			}
		}

		public static void BernD(MultiJ x, MultiJ z)
		// z = denominator of B(n), the n'th Bernoulli number
		{
			if (x.Rs == 0)
				BernD((MultiSI)x, (MultiSI)z);
			else
			{
				int tI;
				Common.InitNAdd(1, "BernD-tI", out tI);
				XJCalLst.MatrixN(x.Rs, x.Cs, tI);
				for (int r = 0; r < x.Rs; r++)
					for (int c = 0; c < x.Cs; c++)
					{
						BernD(x.E[r, c], Common.nList[tI].E[r, c]);
						if (MultiID.muErr)
						{
							Common.LDelete(tI);
							return;
						}
					}
				SetToNM(Common.nList[tI], z);
				Common.LDelete(tI);
			}
		} // BernD J

		public static void BernD(MultiSI nSI, MultiSI z)
		// z = denominator of B(n), the n'th Bernoulli number
		{
			if (nSI.S) { z &= 1; return; }	// n negative
			int n;
			if (nSI.N > 1)
				n = Common.maxBern + 1;		// n too large
			else
				MultiID.Get1SI(out n, nSI);
			if (n > Common.maxBern)
			{
				Common.ReportError("BernD", "n > " + Common.maxBern + ", too large for Bernoulli number");
				MultiID.muErr = true;
				z &= 1;						// n too large
				return;
			}
			if (n == 0) { z &= 1; return; }
			if (n == 1) { z &= 2; return; }
			if ((n & 1) == 1) { z &= 1; return; }	// n odd > 1
			if (n/2 > Common.nBern) Common.GenBernI(n);
			if (MultiID.muErr || MultiID.muAbort)
				z &= 1;
			else
				z &= Common.bernD[n/2];
		} // BernD SI

		public static void BernN(MultiJ x, MultiJ z)
		// z = denominator of B(n), the n'th Bernoulli number
		{
			if (x.Rs == 0)
				BernN((MultiSI)x, (MultiSI)z);
			else
			{
				int tI;
				Common.InitNAdd(1, "BernN-tI", out tI);
				XJCalLst.MatrixN(x.Rs, x.Cs, tI);
				for (int r = 0; r < x.Rs; r++)
					for (int c = 0; c < x.Cs; c++)
					{
						BernN(x.E[r, c], Common.nList[tI].E[r, c]);
						if (MultiID.muErr)
						{
							Common.LDelete(tI);
							return;
						}
					}
				SetToNM(Common.nList[tI], z);
				Common.LDelete(tI);
			}
		} // BernN J

		public static void BernN(MultiSI nSI, MultiSI z)
		// z = numerator of B(n), the n'th Bernoulli number
		{
			if (nSI.S) { z &= 0; return; }	// n negative
			int n;
			if (nSI.N > 1)
				n = Common.maxBern + 1;		// n too large
			else
				MultiID.Get1SI(out n, nSI);
			if (n > Common.maxBern)
			{
				Common.ReportError("BernN", "n > " + Common.maxBern + ", too large for Bernoulli number");
				MultiID.muErr = true;
				z &= 0;						// n too large
				return;
			}
			if (n == 0) { z &= 1; return; }
			if (n == 1) { z &= -1; return; }
			if ((n & 1) == 1) { z &= 0; return; } // n odd > 1
			if (n/2 > Common.nBern) Common.GenBernI(n);
			if (MultiID.muErr || MultiID.muAbort)
				z &= 0;
			else
				z &= Common.bernN[n/2];
		} // BernN SI

		public static void Euler(MultiJ x, MultiJ z)
		// z = denominator of B(n), the n'th Bernoulli number
		{
			if (x.Rs == 0)
				Euler((MultiSI)x, (MultiSI)z);
			else
			{
				int tI;
				Common.InitNAdd(1, "Euler-tI", out tI);
				XJCalLst.MatrixN(x.Rs, x.Cs, tI);
				for (int r = 0; r < x.Rs; r++)
					for (int c = 0; c < x.Cs; c++)
					{
						Euler(x.E[r, c], Common.nList[tI].E[r, c]);
						if (MultiID.muErr)
						{
							Common.LDelete(tI);
							return;
						}
					}
				SetToNM(Common.nList[tI], z);
				Common.LDelete(tI);
			}
		} // Euler J

		public static void Euler(MultiSI nSI, MultiSI z)
		// z = E(n), the n'th Euler number
		{
			if (nSI.S) { z &= 0; return; }	// n negative
			double db;
			MultiID.GetDbInSI(out db, 0, Common.maxEuler + 1, nSI);
			int n = (int)db;
			if (n > Common.maxEuler)		// n too large
			{
				Common.ReportError("Euler", "n > " + Common.maxEuler + ", too large for Euler number");
				MultiID.muErr = true;
				z &= 0;						// n too large
				return;
			}
			if ((n & 1) == 1) { z &= 0; return; }	// n odd > 0
			if (n == 0) { z &= 1; return; }
			if (n == 2) { z &= -1; return; }
			if (n == 4) { z &= 5; return; }
			if (n/2 > Common.nEuler) Common.GenEulerI(n);
			if (MultiID.muErr || MultiID.muAbort)
				z &= 0;
			else
				z &= Common.euler[n/2];
		} // Euler SI

		public static void Kron(MultiJ x, MultiJ y, MultiJ z)
		// z = Kronecker symbol (x/y)
		{
			if (x.Rs == 0 && y.Rs == 0)
				PrimeFA.KronSI(x, y, z);
			else if ((x.Rs != 0 && y.Rs != 0) && (x.Rs != y.Rs || x.Cs != y.Cs))
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "Kron-tI", out tI);
				if (x.Rs == 0)
				{
					XJCalLst.MatrixN(y.Rs, y.Cs, tI);
					for (int r = 0; r < y.Rs; r++)
						for (int c = 0; c < y.Cs; c++)
						{
							PrimeFA.KronSI(x, y.E[r, c], Common.nList[tI].E[r, c]);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
						}
				}
				else if (y.Rs == 0)
				{
					XJCalLst.MatrixN(x.Rs, x.Cs, tI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							PrimeFA.KronSI(x.E[r, c], y, Common.nList[tI].E[r, c]);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
						}
				}
				else
				{
					XJCalLst.MatrixN(x.Rs, x.Cs, tI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							PrimeFA.KronSI(x.E[r, c], y.E[r, c], Common.nList[tI].E[r, c]);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
						}
				}
				SetToNM(Common.nList[tI], z);
				Common.LDelete(tI);
			}
			MultiJD.NormM(z);
		} // Kron J

		public static void Sorder(MultiJ x, MultiJ y, MultiJ z)
		// z = Sord(x, y) = Multiplicative suborder of base x (mod y) or 0
		{
			if (x.Rs == 0 && y.Rs == 0)
				PrimeFA.SorderSI(x, y, z);
			else if ((x.Rs != 0 && y.Rs != 0) && (x.Rs != y.Rs || x.Cs != y.Cs))
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "Sorder-tI", out tI);
				if (x.Rs == 0)
				{
					XJCalLst.MatrixN(y.Rs, y.Cs, tI);
					for (int r = 0; r < y.Rs; r++)
						for (int c = 0; c < y.Cs; c++)
						{
							PrimeFA.SorderSI(x, y.E[r, c], Common.nList[tI].E[r, c]);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
						}
				}
				else if (y.Rs == 0)
				{
					XJCalLst.MatrixN(x.Rs, x.Cs, tI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							PrimeFA.SorderSI(x.E[r, c], y, Common.nList[tI].E[r, c]);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
						}
				}
				else
				{
					XJCalLst.MatrixN(x.Rs, x.Cs, tI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							PrimeFA.SorderSI(x.E[r, c], y.E[r, c], Common.nList[tI].E[r, c]);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
						}
				}
				SetToNM(Common.nList[tI], z);
				Common.LDelete(tI);
			}
			MultiJD.NormM(z);
		} // Sorder J

		public static void Morder(MultiJ x, MultiJ y, MultiJ z) // z = Mord(x, y)
		// multiplicative order of base x (mod y) or zero if it does not exist. From Henri Cohen's book,
		// A Course in Computational Algebraic Number Theory, Springer, 1996, Algorithm 1.4.3, page 25.
		{
			if (x.Rs == 0 && y.Rs == 0)
				PrimeFA.MorderSI(x, y, z);
			else if ((x.Rs != 0 && y.Rs != 0) && (x.Rs != y.Rs || x.Cs != y.Cs))
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "Morder-tI", out tI);
				if (x.Rs == 0)
				{
					XJCalLst.MatrixN(y.Rs, y.Cs, tI);
					for (int r = 0; r < y.Rs; r++)
						for (int c = 0; c < y.Cs; c++)
						{
							PrimeFA.MorderSI(x, y.E[r, c], Common.nList[tI].E[r, c]);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
						}
				}
				else if (y.Rs == 0)
				{
					XJCalLst.MatrixN(x.Rs, x.Cs, tI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							PrimeFA.MorderSI(x.E[r, c], y, Common.nList[tI].E[r, c]);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
						}
				}
				else
				{
					XJCalLst.MatrixN(x.Rs, x.Cs, tI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							PrimeFA.MorderSI(x.E[r, c], y.E[r, c], Common.nList[tI].E[r, c]);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
						}
				}
				SetToNM(Common.nList[tI], z);
				Common.LDelete(tI);
			}
			MultiJD.NormM(z);
		} // Morder J

		public static void Bino(MultiJ x, MultiJ y, MultiJ z)
		// z = Binomial coefficient (a, b) by binary splitting
		{
			if (x.Rs == 0 && y.Rs == 0)
				Bino((MultiSI)x, (MultiSI)y, (MultiSI)z);
			else if ((x.Rs != 0 && y.Rs != 0) && (x.Rs != y.Rs || x.Cs != y.Cs))
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "Bino-tI", out tI);
				if (x.Rs == 0)
				{
					XJCalLst.MatrixN(y.Rs, y.Cs, tI);
					for (int r = 0; r < y.Rs; r++)
						for (int c = 0; c < y.Cs; c++)
						{
							Bino(x, y.E[r, c], Common.nList[tI].E[r, c]);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
						}
				}
				else if (y.Rs == 0)
				{
					XJCalLst.MatrixN(x.Rs, x.Cs, tI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							Bino(x.E[r, c], y, Common.nList[tI].E[r, c]);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
						}
				}
				else
				{
					XJCalLst.MatrixN(x.Rs, x.Cs, tI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							Bino(x.E[r, c], y.E[r, c], Common.nList[tI].E[r, c]);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
						}
				}
				SetToNM(Common.nList[tI], z);
				Common.LDelete(tI);
			}
			MultiJD.NormM(z);
		} // Bino J

		public static void Bino(MultiSI a, MultiSI b, MultiSI z)
		// z = Binomial coefficient (a, b) by binary splitting
		// a, b are not modified unless they share memory with z
		// will write to standard output if diagOn is true
		{
			MultiID.ClearSI(MultiID.fMB);
			MultiID.BinoSI(a, b, z);
		} // Bino SI

		public static void BinoS(MultiJ x, MultiJ y, MultiJ z)
		// z = Binomial coefficient (a, b) by binary splitting
		{
			if (x.Rs == 0 && y.Rs == 0)
				XICommon.BinoS((MultiSI)x, (MultiSI)y, (MultiSI)z);
			else if ((x.Rs != 0 && y.Rs != 0) && (x.Rs != y.Rs || x.Cs != y.Cs))
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "BinoS-tI", out tI);
				XJCalLst.MatrixN(x.Rs, x.Cs, tI);
				if (x.Rs == 0)
				{
					for (int r = 0; r < y.Rs; r++)
						for (int c = 0; c < y.Cs; c++)
						{
							XICommon.BinoS(x, y.E[r, c], Common.nList[tI].E[r, c]);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
						}
				}
				else if (y.Rs == 0)
				{
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICommon.BinoS(x.E[r, c], y, Common.nList[tI].E[r, c]);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
						}
				}
				else
				{
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICommon.BinoS(x.E[r, c], y.E[r, c], Common.nList[tI].E[r, c]);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
						}
				}
				SetToNM(Common.nList[tI], z);
				Common.LDelete(tI);
			}
			MultiJD.NormM(z);
		} // BinoS J

		public static void GCDeSIr(MultiSI x, MultiSI y, MultiSI x1, MultiSI y1, MultiSI z)
		// extended GCD, z = GCD(x, y), also computes x1 and x1 such that x*x1 + y*y1 = GCD(x, y)
		// assumes that x >= 0, y >= 0. z will be >= 0 but x1 or y1 may be <= 0.
		// assumes x, y, x1, y1, and z are distinct objects, recursive version
		{
			if (y == 0)
			{
				z &= x;
				x1 &= 1;
				y1 &= 0;
				return;
			}
			MultiSI x1p = new MultiSI(1);
			MultiSI y1p = new MultiSI(1);
			XICommon.GCDeSI(y, x % y, x1p, y1p, z);
			x1 &= y1p;
			y1 &= x1p - y1p * (x/y);
			MultiID.DoneSI(x1p);
			MultiID.DoneSI(y1p);
		} // GCDeSIr

		public static void Chin1N(int xI, int yI, int zI) // Initialize the Chinese remainder problem
		// z = X1 = x mod y, Y1 = |y|
		{
			if (Common.nList[xI].Rs != 0 || Common.nList[yI].Rs != 0)
				MultiJD.ErrNotScalar();
			else
				XICommon.Chin1N(xI, yI, zI);
		} // Chin1N

		public static void ChinN(int xI, int yI, int zI) // add x mod y to the Chinese remainder problem
		// z = X1 = new x, Y1 = new y
		{
			if (Common.nList[xI].Rs != 0 || Common.nList[yI].Rs != 0)
				MultiJD.ErrNotScalar();
			else
				XICommon.ChinN(xI, yI, zI);
		} // ChinN

		public static void MersennePGM(int zI) // Mersenne equation z = 2^z - 1
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
				XICommon.MersennePG(zI);
			else
			{
				int cs = Common.nList[zI].Cs;
				int tI;
				Common.InitNAdd(1, "MersennePGM-tI", out tI);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
					{
						SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tI]);
						XICommon.MersennePG(tI);
						SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // MersennePGM

		public static void MersenneEqM(int zI) // Mersenne equation z = 2^z - 1
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
				XICommon.MersenneEq(zI);
			else
			{
				int cs = Common.nList[zI].Cs;
				int tI;
				Common.InitNAdd(1, "MersenneEqM-tI", out tI);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
					{
						SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tI]);
						XICommon.MersenneEq(tI);
						SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // MersenneEqM

		public static void MersennePM(int zI) // is 2^z -1 a Mersenne prime?
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
				XICommon.MersenneP(zI);
			else
			{
				int cs = Common.nList[zI].Cs;
				int tI;
				Common.InitNAdd(1, "MersennePM-tI", out tI);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
					{
						SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tI]);
						XICommon.MersenneP(tI);
						SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // MersennePM

		public static void MersennePPM(int zI) // Mersenne Prime Power, MPP(1) = 2
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
				XICommon.MersennePP(zI);
			else
			{
				int cs = Common.nList[zI].Cs;
				int tI;
				Common.InitNAdd(1, "MersennePPM-tI", out tI);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
					{
						SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tI]);
						XICommon.MersennePP(tI);
						SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // MersennePPM

		public static void PerfectNG(int zI) // Perfect Number Generator PEq(MPP(z))
		{
			MersennePPM(zI);
			if (MultiID.muErr) return;
			PerfectEqM(zI);
		} // PerfectNG

		public static void PerfectEqM(int zI) // Perfect equation z = (2^z - 1) * 2^(z-1)
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
				XICommon.PerfectEq(zI);
			else
			{
				int cs = Common.nList[zI].Cs;
				int tI;
				Common.InitNAdd(1, "PerfectEqM-tI", out tI);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
					{
						SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tI]);
						XICommon.PerfectEq(tI);
						SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // PerfectEqM

		public static void MaxSIN(MultiSI x, MultiSI y, int zI)	// z = max of x and y
		{
			if (x >= y)
				SetToN(x, zI);
			else
				SetToN(y, zI);
			NormN(zI);
		} // MaxSIN

		public static void MinSIN(MultiSI x, MultiSI y, int zI)	// z = min of x and y
		{
			if (x >= y)
				SetToN(y, zI);
			else
				SetToN(x, zI);
			NormN(zI);
		} // MinSIN

		public static void MagN(MultiSI x, MultiSI y, int zI)	// z = SqRt(Sq(x) + Sq(y)), Re = remainder
		{
			int tI;
			Common.InitNAdd(1, "MagN-tI", out tI);
			SetToN(x, tI);
			XICommon.MulN(tI, tI, tI);
			SetToN(y, zI);
			XICommon.MulN(zI, zI, zI);
			XICommon.RAddN(tI, zI);
			Common.LDelete("MagN-tI");
			XICommon.SqRtRemN(zI, zI);
		} // MagN

		public static void MagNRem(MultiSI x, MultiSI y, int zI, int rI)	// z = SqRt(Sq(x) + Sq(y)), r = remainder
		{
			int tI;
			Common.InitNAdd(1, "MagNRem-tI", out tI);
			SetToN(x, tI);
			XICommon.MulN(tI, tI, tI);
			SetToN(y, zI);
			XICommon.MulN(zI, zI, zI);
			XICommon.RAddN(tI, zI);
			Common.LDelete("MagNRem-tI");
			XICommon.ReSizeN(4, zI, rI);
			XICommon.ReSizeN(4, zI, zI);
			MultiID.SqRtRemSI(Common.nList[zI], Common.nList[rI], Common.nList[zI]);
		} // MagN

		public static void DivIntN(int xI, int yI, int zI)	// z = Floor(x/y)
		// sets muErr true if overflow
		{
			int tI;
			int m = Common.nList[xI].N;
			Common.InitNAdd(m, "DivIntN-tI", out tI);
			MultiJD.DivM(Common.nList[xI], Common.nList[yI], Common.nList[tI]);
			SetToII(tI, zI);
			Common.LDelete(tI);
			NormN(zI);
		} // DivIntN

		public static void Cleanup() // cleanup named list after a run-time error
		{
			Common.LDelete("AFunction-t2I");
			Common.LDelete("AFunction-t3I");
			Common.LDelete("AFunction-t4I");
			Common.LDelete("AFunction-tI");
			Common.LDelete("AProcedure-tI-1");
			Common.LDelete("AProcedure-tI-2");
			Common.LDelete("Assignment-tI");
			Common.LDelete("Condition-tI");
			Common.LDelete("DivN-tI");
			Common.LDelete("Expression-tI");
			Common.LDelete("MulN-tI");
			Common.LDelete("Power-t2I");
			Common.LDelete("Power-tI");
			Common.LDelete("PrimeFAB-dI");
			Common.LDelete("PrimeFAB-qI");
			Common.LDelete("PrimeFAB-rI");
			Common.LDelete("Primorial-tI");
			Common.LDelete("RAddN-tI");
			Common.LDelete("RAddSIN-tI");
			Common.LDelete("ReadFile-tI");
			Common.LDelete("SimpExpr-tI");
			Common.LDelete("Term-t2I");
			Common.LDelete("Term-tI");
			GC.Collect();	// force garbage collection of all generations
		} // Cleanup

		public static bool listOn;		// true if ListReport is running
		public static int fMC;			// current max # of super digits in a result
		public static int xICIn;		// code input list item, its name has the File name
		public static int xILOut;		// log output list item, its name has the File name
		public static int xINIn;		// number input list item, its name has the File name
		public static int xINOut;		// number output list item, its name has the File name
	}
}
