using System;
using System.IO;			// for StreamWriter I/O to a file
using System.Windows.Forms;	// for RichTextBox

namespace MyNamespace
{
	/// <summary>
	///		Extra precision Integer List algorithms
	///		Developed in TURBO Pascal 5.5 and 6.0, converted to Visual Basic 6
	///		and now in Visual C# .Net
	///
	/// ---------------- *** LEGAL NOTICE: *** ------------------
	///
	/// This program is free software; you can redistribute it and/or modify
	/// it under the terms of the GNU General Public License (GPL) as published
	/// by the Free Software Foundation. cf. the file COPYING.txt.
	///
	/// ------------ *** end of legal notice *** ------------------
	/// </summary>

	public class XICalLst
	{
		public XICalLst()
		{
			Common.saveI = 1;
		}

		public static void SetTo1N(int d1, int zI) // z = d1 % baseM
		// allows a literal d1, d1 an integer, |d1| < baseM, d1 not changed
		{
			MultiSI t = new MultiSI(2);
			MultiID.SetTo1SI(d1, t);
			Calc.SetToN(t, zI);
			MultiID.DoneSI(t);
		}

		public static void SetToDbN(double db, int zI) // z = db % baseM^4
		// allows a literal db, db not changed, muErr set true if overflow
		{
			MultiSI t = new MultiSI(4);
			MultiID.SetToDbSI(db, t);
			Calc.SetToN(t, zI);
			MultiID.DoneSI(t);
		}

		public static void RAdd1N(int d1, int zI) // z = z + d1
		// allows literal d1, d1 is not changed, sets muErr true if overflow
		{
			MultiSI t = new MultiSI(2); // temp
			MultiID.SetTo1SI(d1, t);
			XICommon.RAddSIN(t, zI);
			MultiID.DoneSI(t);
		}

		public static void Add1SIN(MultiSI x, int d1, int zI) // z = x + d1
		// allows literal d1, d1 is not changed, sets muErr true if overflow
		{
			Calc.SetToN(x, zI);
			RAdd1N(d1, zI);
		}

		public static void FormatN(StreamWriter outF, string name, MultiSI z)
		// output z as a formatted set of lines and pages
		// z is not modified
		{
			int tot = 0;
			MultiID.WriteLn(null, outF); // one blank line
			if (MultiID.baseO != 10)
				MultiID.WriteLn2(null, outF, "Base(" + MultiID.baseO.ToString() + ")");
			MultiID.WriteLn2(null, outF, name + " =");	// output name to file #outF
			MultiID.WriteSILn(null, outF, z, ref tot);	// output x to file #outF
		} // FormatN

		public static void RepUnitTSIN(MultiSI x, MultiSI y, int zI) // z = x -- y
		{
			int tI;
			Common.InitNAdd(1, "RepUnitTN-tI", out tI);
			Calc.SetToN(y, tI);
			Calc.RepUnitN(tI);
			MulSIN(x, Common.nList[tI], zI);
			Calc.NormN(zI);
			Common.LDelete(tI);
		} // RepUnitTSIN

		public static void CompSIN(MultiSI x, MultiSI y, string op, string op2, int zI) // z = y op x
		{
			int sign;
			bool met;
			sign = MultiID.CompSI(x, y);
			if (op2 == "<=")
				met = (sign <= 0);
			else if (op2 == "<>" || op2 == "!=")
				met = (sign != 0);
			else if (op2 == ">=")
				met = (sign >= 0);
			else if (op2 == "==")
				met = (sign == 0);
			else if (op == "<")
				met = (sign < 0);
			else if (op == "=")
				met = (sign == 0);
			else if (op == ">")
				met = (sign > 0);
			else
				met = false;	// should not get here
			Calc.ClearN(zI);
			if (met) Common.nList[zI].V[0] = 1;	// n is initially zero
		} // CompSIN

		public static void AndSIN(MultiSI x, MultiSI y, int zI) // z = x & y
		{
			if (!MultiID.ZTestSI(x) && !MultiID.ZTestSI(y))
			{
				Calc.ClearN(zI);
				Common.nList[zI].V[0] = 1;
			}
			else
				Calc.ClearN(zI);
		} // AndSIN

		public static void OrSIN(MultiSI x, MultiSI y, int zI) // z = x | y
		{
			if (!MultiID.ZTestSI(x) || !MultiID.ZTestSI(y))
			{
				Calc.ClearN(zI);
				Common.nList[zI].V[0] = 1;
			}
			else
				Calc.ClearN(zI);
		} // OrSIN

		public static void MulSIN(MultiSI x, MultiSI y, int zI) // z = x * y
		// x or y or both may have the same address in memory as z
		// x, y are not changed unless they share memory with z
		// sets muErr true if overflow
		{
			Common.nList[zI] &= x * y;
		}

		public static void RMulSIN(MultiSI x, int zI) // z = z * x
		// x may have the same address in memory as z
		// x is not changed unless it shares memory with z
		// sets muErr true if overflow
		{
			MulSIN(Common.nList[zI], x, zI);
		}

		public static void SqSIN(MultiSI x, int zI) // z = x * x
		// x may have the same address in memory as z
		// x is not changed unless it shares memory with z
		// sets muErr true if overflow
		{
			MulSIN(x, x, zI);
		}

		public static void RSqN(int zI) // z = z * z
		// sets muErr true if overflow
		{
			MulSIN(Common.nList[zI], Common.nList[zI], zI);
		}

		public static void PowISI(int yI, MultiSI b, MultiSI p, bool MFlag, int fMBI)
		// y = b^p % FMB (% only if MFlag is true), p an integer
		{
			int bL;
			int pL;
			int iL;
			bool fini;
			bool signP;
			signP = p.S;
			if (MultiID.ZTestSI(b)) // if b == 0
			{
				Calc.ClearN(yI);
				if (signP)
					MultiID.MuWriteErr("Cannot raise zero to a negative power");
				else if (MultiID.ZTestSI(p)) // p = 0
					SetTo1N(1, yI); // can raise zero to the zero power, 0^0 = 1
				return;
			}
			if (MultiID.ZTestSI(p) || b == MultiID.sOne) // if p == 0 or b == 1
			{
				SetTo1N(1, yI);
				return;
			}
			if (b == MultiID.sMOne)
			{
				SetTo1N(1, yI);
				if ((p.V[0] & 1) == 1)
					SetTo1N(-1, yI);
				return;
			}
			if (signP)
			{
				Calc.ClearN(yI);
				return;
			}
			fini = false;
			Common.InitNAdd(1, "PowIF-bL", out bL);
			Calc.SetToN(b, bL);
			Common.InitNAdd(1, "PowIF-pL", out pL);
			Calc.SetToN(p, pL);
			Common.nList[pL].S = false;
			SetTo1N(1, yI);
			do
			{
				iL = Common.nList[pL].V[0];
				if ((iL & 1) != 0) // Odd(iL)
					MulSIN(Common.nList[bL], Common.nList[yI], yI);
				if (MFlag) RModSIN(Common.nList[fMBI], yI);
				RDiv1N(2, pL);
				if (Common.ZTestN(pL))
					fini = true;
				else
				{
					MulSIN(Common.nList[bL], Common.nList[bL], bL);
					if (MFlag) RModSIN(Common.nList[fMBI], bL);
				}
				if (MultiID.muAbort)
				{
					Calc.ClearN(yI);
					fini = true;
				}
			}
			while (!fini);
			Common.LDelete(pL);
			Common.LDelete(bL);
			Calc.NormN(yI);
		} // PowISI

		public static void Pow10SI(MultiSI p, MultiSI z)    // z = (10^p)
		// sets muErr true if overflow or error
		{
			long pL;
			long rem;
			if (p > MultiID.max64)
				MultiID.GetExSI(out pL, MultiID.max64);
			else
				MultiID.GetExSI(out pL, p);
			long k = Math.DivRem(pL, MultiID.muDMax, out rem);   // k = p/decimal digits in a super digit
			z &= (int)Math.Pow(10.0, (double)rem);
			int m = Math.Min(MultiID.mMax, (int)k + z.N);
			if (z.M < m)
			{
				z.M = m;
				Common.ReDimPreserve(ref z.V, m + 1);
			}
			XICommon.ShiftL(k, z);
		}

		public static void PowMSI(int yI, MultiSI b, MultiSI p, bool MFlag, int fMBI)
		// y = b^p % FMB (% only if MFlag is true)
		{
			if ((b == 10 || b == -10) && !p.S && !MFlag)
			{
				int lsbP = p.V[0] & 1;
				bool sign = (b.S && lsbP == 1);
				Pow10SI(p, Common.nList[yI]);
				if (MultiID.muErr)
					Calc.ClearN(yI);
				else
					Common.nList[yI].S = sign;
				return;
			}
			int tI;
			// if b is zero or (both fMB and p are integers)
			if (MultiID.ZTestSI(b) || !MFlag)
			{
				PowISI(yI, b, p, MFlag, fMBI);
				Calc.NormN(yI);
				return;
			}		
			if (b.S) // b < 0 and (p not an integer)
			{
				MultiID.MuWriteErr("Cannot raise negative number to a non-integer power");
				Calc.ClearN(yI);
				return;
			}
			Common.InitNAdd(1, "PowMF-tI", out tI);
			bool signY = b.S && (p.V[0] & 1) == 1; // if negative to odd integer
			bool signB = b.S;
			b.S = false;
			b.S = signB;
			RMulSIN(p, tI);
			Common.nList[yI].S = signY;
			if (MFlag) RModSIN(Common.nList[fMBI], yI);
			Common.LDelete(tI);
			Calc.NormN(yI);
		} // PowMSI

		public static void PowSIN(MultiSI b, MultiSI p, int zI) // z = b^p
		{
			int fMBI = 0;
			PowMSI(zI, b, p, false, fMBI);	// FMB not used
		}

		public static void PowNSI(MultiSI b, MultiSI p, MultiSI z) // z = b^p
		{
			int fMBI = 0;
			int zI;
			Common.InitNAdd(1, "PowSI-zI", out zI);
			PowMSI(zI, b, p, false, fMBI);	// FMB not used
			MultiID.SetToSI(Common.nList[zI], z);
			Common.LDelete(zI);
		}

		public static void RPowSIN(MultiSI p, int zI) // z = z^p
		{
			PowSIN(Common.nList[zI], p, zI);
		}

		public static void PowMSIN(MultiSI b, MultiSI p, int zI) // z = b^p % FMB
		{
			int fMBI;
			if (!Common.FMBIok(zI, out fMBI))
				return;
			if (Common.nList[fMBI].Rs != 0)
				MultiJD.ErrFMBNotScalar();
			else
			{
				MultiSI t = new MultiSI(2);
				Calc.SetFMB();
				MultiID.ReSizeSI(Calc.fMC, t); // fMC needed
				MultiID.PowMBSI(b, p, t);
				Calc.SetToN(t, zI);
			}
		} // PowMSIN

		public static void RPowMSIN(MultiSI p, int zI) // z = z^p % FMB
		{
			PowMSIN(Common.nList[zI], p, zI);
		}

		public static void RDiv1N(int d1, int zI) // z = z / d1
		// Multi-precision one or two super digit divide, d1 is not changed
		// allows literal d1, muErr set true if d1 = 0
		{
			MultiSI t = new MultiSI(2);
			MultiID.SetTo1SI(d1, t);
			RDivSIN(t, zI);
			MultiID.DoneSI(t);
		}

		public static void Div1SIN(MultiSI u, int d1, int zI) // z = u / d1
		// Multi-precision one or two super digit divide, d1 and u not changed
		// u and z may have same address in memory, then u is changed
		// allows literal d1, muErr set true if d1 = 0
		{
			Calc.SetToN(u, zI);
			RDiv1N(d1, zI);
		}

		public static void DivRemSIN(MultiSI u, MultiSI d, int zI) // z = Floor(u / d), Re = remainder
		// u or d or both may have the same address in memory as z
		// u, d are not changed unless they share memory with z
		// sets muErr true if d = 0
		{
			int rI;
			Common.LFindAdd("Re", out rI);
			if (rI == zI)
			{
				MultiID.MuWriteErr(
				"Cannot set same location to both quotient and remainder, continuing...");
				return;
			}
			MultiSI t = new MultiSI();
			t &= u / d;							// t = integer quotient
			if (MultiID.muErr)
			{
				Calc.SetToN(MultiJD.mZero, rI);
				Calc.SetToN(MultiJD.mZero, zI);
				return;
			}
			Common.nList[rI] &= u - t * d;		// Re = remainder
			Common.nList[zI] &= t;				// z = integer quotient
		} // DivRemSIN

		public static void DivIntSIN(MultiSI u, MultiSI d, int zI) // z = Floor(u / dZ
		// u or d or both may have the same address in memory as z
		// u, d are not changed unless they share memory with z
		// sets muErr true if d = 0
		{
			Common.nList[zI] &= u / d;
		} // DivIntSIN

		public static void RDivSIN(MultiSI d, int zI) // z = z / d
		// d may have the same address in memory as z
		// d is not changed unless it shares memory with z
		// sets muErr true if d = 0
		{
			DivIntSIN(Common.nList[zI], d, zI);
		}

		public static void FacNSI(MultiSI x, int zI, bool MFlag, int fMBI) // z = x! = 1*2*3*...*x
		// x may have the same address in memory as z
		// x is not changed unless it shares memory with z
		// sets muErr true if overflow or error
		// z must be large enough ... (See FacN)
		{
			MultiSI xL = new MultiSI(4);
			xL &= x;
			if (!MFlag)
			{
				int saveFMC = Calc.fMC;
				Calc.fMC = FacFMC(xL, false);
				MultiSI lim = new MultiSI(4);
				FacSNSI(xL, zI, MFlag, fMBI);
				MultiID.DoneSI(lim);
				Calc.fMC = saveFMC;
			}
			else
				FacSNSI(xL, zI, MFlag, fMBI);
			MultiID.DoneSI(xL);
		} // FacNSI

		public static int FacFMC(MultiSI x, bool MFlag) // compute fMC needed for Fac(x)
		{
			int fMC;
			if (x - 14 < MultiID.sZero)
				fMC = 2;
			else
			{
				fMC = Calc.fMC;
				int tI;
				Common.InitNAdd(5, "FacFMC-tI", out tI);
				Calc.fMC = 4;
				RMulSIN(x, tI);
				RDiv1N(MultiID.muDMax, tI);
				double db;
				MultiID.GetDbSI(out db, Common.nList[tI]);
				Calc.fMC = fMC;
				if (db < fMC || MFlag)
					fMC = (int)db + 1;
				if (fMC < 2)
					fMC = 2;
				Common.LDelete(tI);
			}
			MultiID.ADiag("FacFMC: FMC = " + fMC);
			return fMC;
		} // FacFMC

		public static void FacSNSI(MultiSI x, int zI, bool MFlag, int fMBI) // z = x! = 1*2*3*...*x
		// x may have the same address in memory as z
		// x is not changed unless it shares memory with z
		// sets muErr true if overflow or error
		// z must be large enough ... (See FacN)
		{
			MultiSI xL = new MultiSI(4);
			MultiSI lim = new MultiSI(4);
			xL &= x;
			MultiSI nL = new MultiSI(4);
			nL &= 2;
			SetTo1N(1, zI);
			MultiID.nDone = 0;
			MultiID.nToGo = Int64.MaxValue;
			lim &= Int64.MaxValue;
			while (nL <= xL)
			{
				if (MFlag)
				{
					int newN = nL.N + Common.nList[zI].N;
					if (Calc.fMC < newN)
						Calc.fMC = newN;
				}
				MulSIN(nL, Common.nList[zI], zI);	// 99% of processor time of FacF is here
				if (MFlag)
				{
					ModSIN(Common.nList[zI], Common.nList[fMBI], zI);
					if (MultiID.ZTestSI(Common.nList[zI])) break;
				}
				if (MultiID.muAbort)
				{
					MultiID.WriteLn("Factorial function aborted by operator, " + MultiID.nToGo +
						" multiplies to go");
					Calc.ClearN(zI);
					break;
				}
				nL += 1;
				MultiID.nDone++;
				if (MultiID.statReq2)
				{
					if (xL - nL < lim)
					{
						long n;
						MultiID.GetExSI(out n, xL - nL);
						MultiID.nToGo = n + 1;
					}
					MultiID.WritenDone("FacSNF:");
				}
			}
			MultiID.nToGo = 0;
			MultiID.DoneSI(nL);
			MultiID.DoneSI(lim);
			MultiID.DoneSI(xL);
		} // FacSNSI

		public static MultiSI PMr(MultiSI a, MultiSI b, int fMBI)
		// computes a!/b! % fMB by binary splitting (a >= b >= 0 and integers)
		{
			if (MultiID.muErr || MultiID.muAbort) return MultiID.sZero;
			MultiSI dF = new MultiSI(a.N);
			dF &= a - b;
			long d;
			MultiID.GetExSI(out d, dF);
			MultiID.DoneSI(dF);
			MultiSI p = new MultiSI(1);
			MultiSI p1 = new MultiSI(1);
			MultiSI p2 = new MultiSI(1);
			if (d > 3)
			{
				MultiSI m = new MultiSI(a.N + 1);
				m &= (a + b) / 2;
				if (fMBI != 0)
				{
					Calc.SetToNM(PMr(a, m, fMBI), p1);
					Calc.SetToNM(PMr(m, b, fMBI), p2);
					int newN = p1.N + p2.N;
					if (Calc.fMC < newN)
						Calc.fMC = newN;
					Calc.SetToNM(p1 * p2, p);
				}
				else
					Calc.SetToNM(PMr(a, m, fMBI) * PMr(m, b, fMBI), p);
				if (fMBI != 0) Mod(p, Common.nList[fMBI], p);
				if (MultiID.currK > (MultiID.maxK - d)) // currK += d could overflow
					MultiID.currK = MultiID.maxK;
				else
					MultiID.currK += d;
			}
			else
			{
				p &= a;
				if (fMBI != 0) Mod(p, Common.nList[fMBI], p);
				if (MultiID.currK > (MultiID.maxK - 1)) MultiID.currK = MultiID.maxK;
				else MultiID.currK += 1;
				if (d > 1)
				{
					if (fMBI != 0)
					{
						int newN = p.N + a.N;
						if (Calc.fMC < newN)
							Calc.fMC = newN;
					}
					p *= (a - 1);
					if (fMBI != 0) Mod(p, Common.nList[fMBI], p);
					if (MultiID.currK > (MultiID.maxK - 3)) MultiID.currK = MultiID.maxK;
					else MultiID.currK += 3;
				}
				if (d > 2)
				{
					if (fMBI != 0)
					{
						int newN = p.N + a.N;
						if (Calc.fMC < newN)
							Calc.fMC = newN;
					}
					p *= (a - 2);
					if (fMBI != 0) Mod(p, Common.nList[fMBI], p);
					if (MultiID.currK > (MultiID.maxK - 4)) MultiID.currK = MultiID.maxK;
					else MultiID.currK += 4;
				}
			}
			MultiID.ReportPct();
			return p;
		} // PMr % fMB

		public static void Fac2SIN(MultiSI a, MultiSI b, int zI, bool MFlag)
		// z = Fac(a) / Fac(b) % fMB by binary splitting
		// a or b or both may have the same address in memory as z
		// a, b are not changed unless they share memory with z
		// sets muErr true if overflow or error
		{
			int fMBI = 0;
			if (MFlag && !Common.FMBIok(zI, out fMBI))
				return;
			if (MFlag && Common.nList[fMBI].Rs != 0)
			{
				MultiJD.ErrFMBNotScalar();
				return;
			}
			if (MFlag)
				Calc.SetFMB();
			else
				MultiID.ClearSI(MultiID.fMB);
			MultiID.Fac2SI(a, b, Common.nList[zI]);
			Calc.NormN(zI);
		} // Fac2SIN

		public static void FacSIN(MultiSI x, int zI, bool MFlag, int fMBI)
		// z = x! = 1*2*3*...*x % FMB
		// x may have the same address in memory as zI
		// x is not changed unless it shares memory with zI
		// sets muErr true if overflow or error
		{
			if (MFlag && !Common.FMBIok(zI, out fMBI))
				return;
			if (MFlag && Common.nList[fMBI].Rs != 0)
			{
				MultiJD.ErrFMBNotScalar();
				return;
			}
			if (MFlag)
				Calc.SetFMB();
			else
				MultiID.ClearSI(MultiID.fMB);
			MultiID.FacMBiSI(x, Common.nList[zI]);
			Calc.NormN(zI);
		} // FacSIN

		public static void FacSSIN(MultiSI x, int zI, bool MFlag, int fMBI)
		// z = x! = 1*2*3*...*x % FMB
		// x may have the same address in memory as zI
		// x is not changed unless it shares memory with zI
		// sets muErr true if overflow or error
		{
			if (MFlag && !Common.FMBIok(zI, out fMBI))
				return;
			if (MFlag && Common.nList[fMBI].Rs != 0)
			{
				MultiJD.ErrFMBNotScalar();
				return;
			}
			if (MFlag)
				Calc.SetFMB();
			else
				MultiID.ClearSI(MultiID.fMB);
			MultiID.FacMBSI(x, Common.nList[zI]);
			Calc.NormN(zI);
		} // FacSSIN

		public static void Mod(MultiSI x, MultiSI y, MultiSI z)	// z = Mod(x, y))
		// same as FORTRAN definition: z = Mod(x, y) = x - (Floor(x/y) * y)
		// sign(z) = sign(y) or z = 0
		{
			MultiSI ySave = new MultiSI(y.N);
			ySave &= y;
			int tI;
			Common.InitNAdd(1, "ModN-tI", out tI);
			Calc.SetToN(x / y, tI);
			RMulSIN(-y, tI);
			XICommon.RAddSIN(x, tI);
			Calc.SetToNM(Common.nList[tI], z); // z could over store y
			if (z.S != ySave.S)
				MultiID.ClearSI(z);
			else
			{
				ySave.S = false;
				Common.nList[tI].S = false;
				if (Common.nList[tI] >= ySave)
					MultiID.ClearSI(z);
			}
			Common.LDelete(tI);
		} // Mod

		public static void ModSIN(MultiSI x, MultiSI y, int zI)	// z = Mod(x, y))
		// same as FORTRAN definition: z = Mod(x, y) = x - (Floor(x/y) * y)
		// sign(z) = sign(y) or z = 0
		{
			MultiSI zz = new MultiSI();
			MultiID.ModSI(x, y, zz);
			Calc.SetToN(zz, zI);
		}
		// ModSIN

		public static void RModSIN(MultiSI x, int zI) // z = Mod(z, x))
		{
			ModSIN(Common.nList[zI], x, zI);
		}

		public static void GCD(MultiSI a, MultiSI b, MultiSI z) // z = GCD(a, b))
		{
			if (MultiID.ZTestSI(a))
				Calc.SetToNM(b, z);
			else if (MultiID.ZTestSI(b))
				Calc.SetToNM(a, z);
			else
			{
				bool fini = false;
				int tot = 0;
				int nn = 0;
				MultiSI aL = new MultiSI();
				MultiSI bL = new MultiSI();
				Calc.SetToNM(a, aL);
				Calc.SetToNM(b, bL);
				aL.S = false;
				bL.S = false;
				if (MultiID.debug) Common.SingleSpace();
				while (!fini)
				{
					nn++;
					if ((nn & 1) != 0)			// Odd(nn)
					{
						MultiID.ModSI(aL, bL, aL);
						if (MultiID.debug)
						{
							MultiID.WriteT(nn + " ", ref tot);
							MultiID.WriteSI(aL, ref tot);
							MultiID.WriteLn();
							tot = 0;
						}
						if (MultiID.ZTestSI(aL))
						{
							Calc.SetToNM(bL, z);
							fini = true;
						}
					}
					else
					{
						MultiID.ModSI(bL, aL, bL);
						if (MultiID.debug)
						{
							MultiID.WriteT(nn + " ", ref tot);
							MultiID.WriteSI(bL, ref tot);
							MultiID.WriteLn();
							tot = 0;
						}
						if (MultiID.ZTestSI(bL))
						{
							Calc.SetToNM(aL, z);
							fini = true;
						}
					}
				}
			}
		} // GCD

		public static void GCDSIN(MultiSI a, MultiSI b, int zI) // z = GCD(a, b))
		{
			MultiSI zz = new MultiSI();
			GCD(a, b, zz);
			Calc.SetToN(zz, zI);
		} // GCDSIN

		public static void LCM(MultiSI a, MultiSI b, MultiSI z) // z = LCM(a, b))
		// LCM(a, b) = a * b / GCD(a, b)
		{
			MultiSI zz = new MultiSI();
			GCD(a, b, zz);
			Calc.SetToNM((a * b) / zz, z);
		} // LCM

		public static void LCMSIN(MultiSI a, MultiSI b, int zI) // z = LCM(a, b))
		{
			MultiSI zz = new MultiSI();
			LCM(a, b, zz);
			Calc.SetToN(zz, zI);
		} // LCMSIN

		public static void NotSIN(MultiSI x, int zI)	// z = !x
		{
			bool zip;
			zip = MultiID.ZTestSI(x);
			Calc.ClearN(zI);
			if (zip) Common.nList[zI].V[0] = 1;
		} // NotSIN
	}
}
