using System;
using System.IO;			// for StreamWriter I/O to a file
using System.Windows.Forms;	// for RichTextBox

namespace MyNamespace
{
	/// <summary>
	///		Extra precision signed integer List algorithms
	///		Developed in TURBO Pascal 5.5 and 6.0, converted to Visual Basic 6
	///		and now in Visual C# .Net
	///
	/// ---------------- *** LEGAL NOTICE: *** ------------------
	///
	/// This program is free software; you can redistribute it and/or modify
	/// it under the terms of the GNU General Public License (GPL) as published
	/// by the Free Software Foundation. cf. the file COPYING.txt.
	///
	/// ------------ *** end of legal notice *** ------------------
	/// </summary>

	public class XJCalLst
	{
		public XJCalLst()
		{
			uFirst = 1;
		}

		public static void Trace(MultiJ x, MultiJ z) // z = sum of elements on the principal diagonal
		{
			if (x.Rs == 0)
				MultiJD.ErrNotMatrix();
			else
			{
				MultiSI sum = new MultiSI();
				int n = Math.Min(x.Rs, x.Cs);
				for (int i = 0; i < n; i++)
					sum += x.E[i, i];
				Calc.SetToNM(sum, z);
			}
		} // Trace

		public static void TraceN(MultiJ x, int zI) // z = sum of elements on the principal diagonal
		{
			if (x.Rs == 0)
				MultiJD.ErrNotMatrix();
			else
			{
				MultiJ z = new MultiJ();
				Trace(x, z);
				Calc.SetToN(z, zI);			}
		} // TraceN

		public static void IsDiagN(MultiJ x, int zI) // is x a diagonal matrix 1 (True), 0 (False
		{
			if (x.Rs == 0 || x.Rs != x.Cs)
				Calc.ClearN(zI);
			else
			{
				int i = 1;
				for (int r = 0; r < x.Rs; r++)
				{
					for (int c = 0; c < x.Cs; c++)
					{
						if (r != c && x.E[r, c] != MultiJD.mZero)
						{
							i = 0;
							break;
						}
					}
					if (i == 0)
						break;
				}
				Calc.ClearN(zI);
				Common.nList[zI].V[0] = i;
			}
		} // IsDiagN

		public static void MatIdN(MultiJ x, int zI) // z = x by x identity matrix
		{
			int n;
			MultiJD.Get1J(out n, x);
			if (x.Rs != 0)
				MultiJD.ErrNotScalar();
			else if (n < 1)
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "MatIdN-tI", out tI);
				MatrixN(n, n, tI);
				for (int r = 0; r < n; r++)
					for (int c = 0; c < n; c++)
						Common.nList[tI].E[r, c] &= MultiJD.mZero;
				for (int i = 0; i < n; i++)
					Common.nList[tI].E[i, i] &= MultiJD.mOne;
				Calc.SetToII(tI, zI);
				Common.LDelete(tI);
			}
		} // MatIdN

		public static void PascalN(MultiJ x, int zI) // z = x by x Pascal triangle
		// zij = Bino(i, j)
		{
			int n;
			MultiJD.Get1J(out n, x);
			if (x.Rs != 0)
				MultiJD.ErrNotScalar();
			else if (n < 1)
				MultiJD.ErrIncompatible();
			else
			{
				MultiSI iS = new MultiSI(1);
				MultiSI jS = new MultiSI(1);
				MultiSI tS = new MultiSI(1);
				MatrixN(n, n, zI);
				for (int i = 0; i < n; i++)
				{
					iS &= i;
					for (int j = 0; j < n; j++)
					{
						jS &= j;
						Calc.Bino(iS, jS, tS);
						if (MultiID.muErr)
							return;
						Calc.SetToNM(tS, Common.nList[zI].E[i, j]);
					}
				}
			}
		} // PascalN

		public static void DiagonalN(MultiJ x, int zI) // z = diagonal matrix from vector x
		{
			Calc.SetToN(MultiJD.Diagonal(x), zI);
		} // DiagonalN

		public static void Del(MultiJ x, int c, MultiJ z) // delete column c from matrix x
		{
			if (c < 0 || c >= x.Cs)
				Calc.SetToNM(x, z);
			else if (x.Cs == 1)
				Calc.SetToNM(MultiJD.mZero, z);
			else
			{
				MultiJ zz = new MultiJ(1, x.Rs, x.Cs - 1);
				for (int r = 0; r < x.Rs; r++)
					for (int ci = 0; ci < c; ci++)
						Calc.SetToNM(x.E[r, ci], zz.E[r, ci]);
				for (int r = 0; r < x.Rs; r++)
					for (int ci = c + 1; ci < x.Cs; ci++)
						Calc.SetToNM(x.E[r, ci], zz.E[r, ci - 1]);
				Calc.SetToNM(zz, z);
			}
		} // Del

		public static void DelN(MultiJ x, MultiJ y, int zI) // delete column y from matrix x
		{
			int c;
			MultiJD.Get1J(out c, y);
			if (x.Rs == 0)
				MultiJD.ErrNotMatrix();
			else if (y.Rs != 0)
				MultiJD.ErrNotScalar();
			else if (c < 1 || c > x.Cs)
				MultiJD.ErrIncompatible();
			else
			{
				c -= uFirst;
				MultiJ zz = new MultiJ();
				Del(x, c, zz);
				Calc.SetToN(zz, zI);
			}
		} // DelN

		public static void DelR(MultiJ x, int r, MultiJ z) // delete row r from matrix x
		{
			if (r < 0 || r >= x.Rs)
				Calc.SetToNM(x, z);
			else if (x.Rs == 1)
				Calc.SetToNM(MultiJD.mZero, z);
			else
			{
				MultiJ zz = new MultiJ(1, x.Rs - 1, x.Cs);
				for (int ri = 0; ri < r; ri++)
					for (int c = 0; c < x.Cs; c++)
						Calc.SetToNM(x.E[ri, c], zz.E[ri, c]);
				for (int ri = r + 1; ri < x.Rs; ri++)
					for (int c = 0; c < x.Cs; c++)
						Calc.SetToNM(x.E[ri, c], zz.E[ri - 1, c]);
				Calc.SetToNM(zz, z);
			}
		} // DelR

		public static void DelRN(MultiJ x, MultiJ y, int zI) // delete row y from matrix x
		{
			int r;
			MultiJD.Get1J(out r, y);
			if (x.Rs == 0)
				MultiJD.ErrNotMatrix();
			else if (y.Rs != 0)
				MultiJD.ErrNotScalar();
			else if (r < 1 || r > x.Rs)
				MultiJD.ErrIncompatible();
			else
			{
				r -= uFirst;
				MultiJ zz = new MultiJ();
				DelR(x, r, zz);
				Calc.SetToN(zz, zI);
			}
		} // DelRN

		public static void ExtractN(MultiJ x, MultiJ y, int zI) // extract column y from matrix x
		{
			int c;
			MultiJD.Get1J(out c, y);
			if (x.Rs == 0)
				MultiJD.ErrNotMatrix();
			else if (y.Rs != 0)
				MultiJD.ErrNotScalar();
			else if (c < 1 || c > x.Cs)
				MultiJD.ErrIncompatible();
			else
			{
				c -= uFirst;
				int tI;
				Common.InitNAdd(1, "ExtractN-tI", out tI);
				MatrixN(x.Rs, 1, tI);
				for (int r = 0; r < x.Rs; r++)
					Calc.SetToNM(x.E[r, c], Common.nList[tI].E[r, 0]);
				Calc.SetToII(tI, zI);
				Common.LDelete(tI);
			}
		} // ExtractN

		public static void ExtractRN(MultiJ x, MultiJ y, int zI) // extract row y from matrix x
		{
			int r;
			MultiJD.Get1J(out r, y);
			if (x.Rs == 0)
				MultiJD.ErrNotMatrix();
			else if (y.Rs != 0)
				MultiJD.ErrNotScalar();
			else if (r < 1 || r > x.Rs)
				MultiJD.ErrIncompatible();
			else
			{
				r -= uFirst;
				int tI;
				Common.InitNAdd(1, "ExtractRN-tI", out tI);
				MatrixN(1, x.Cs, tI);
				for (int c = 0; c < x.Cs; c++)
					Calc.SetToNM(x.E[r, c], Common.nList[tI].E[0, c]);
				Calc.SetToII(tI, zI);
				Common.LDelete(tI);
			}
		} // ExtractRN

		public static void ConcatN(MultiJ x, MultiJ y, int zI) // z = [x y]
		{
			if (x.Rs == 0 || y.Rs == 0)
				MultiJD.ErrNotMatrix();
			else if (x.Rs != y.Rs)
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "ConcatN-tI", out tI);
				MatrixN(x.Rs, x.Cs + y.Cs, tI);
				for (int r = 0; r < x.Rs; r++)
					for (int c = 0; c < x.Cs; c++)
						Calc.SetToNM(x.E[r, c], Common.nList[tI].E[r, c]);
				for (int r = 0; r < y.Rs; r++)
					for (int c = 0; c < y.Cs; c++)
						Calc.SetToNM(y.E[r, c], Common.nList[tI].E[r, c+x.Cs]);
				Calc.SetToII(tI, zI);
				Common.LDelete(tI);
			}
		} // ConcatN

		public static void ConcatRN(MultiJ x, MultiJ y, int zI) // z =	[x]
		{														//		[y]
			if (x.Rs == 0 || y.Rs == 0)
				MultiJD.ErrNotMatrix();
			else if (x.Cs != y.Cs)
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "ConcatRN-tI", out tI);
				MatrixN(x.Rs + y.Rs, x.Cs, tI);
				for (int r = 0; r < x.Rs; r++)
					for (int c = 0; c < x.Cs; c++)
						Calc.SetToNM(x.E[r, c], Common.nList[tI].E[r, c]);
				for (int r = 0; r < y.Rs; r++)
					for (int c = 0; c < y.Cs; c++)
						Calc.SetToNM(y.E[r, c], Common.nList[tI].E[r+x.Rs, c]);
				Calc.SetToII(tI, zI);
				Common.LDelete(tI);
			}
		} // ConcatRN

		public static void InsertN(MultiJ x, MultiJ y, MultiJ cF, int zI)
		// z = x with y inserted at column c
		{
			int c;
			MultiJD.Get1J(out c, cF);
			if (x.Rs == 0 || y.Rs == 0)
				MultiJD.ErrNotMatrix();
			else if (cF.Rs != 0)
				MultiJD.ErrNotScalar();
			else if (x.Rs != y.Rs)
				MultiJD.ErrIncompatible();
			else if (c < 1 || c > x.Cs + 1)
				MultiJD.ErrIncompatible();
			else
			{
				c -= uFirst;
				int tI;
				Common.InitNAdd(1, "InsertN-tI", out tI);
				MatrixN(x.Rs, x.Cs + y.Cs, tI);
				for (int r = 0; r < x.Rs; r++)
					for (int ci = 0; ci < c; ci++)
						Calc.SetToNM(x.E[r, ci], Common.nList[tI].E[r, ci]);
				for (int r = 0; r < y.Rs; r++)
					for (int ci = 0; ci < y.Cs; ci++)
						Calc.SetToNM(y.E[r, ci], Common.nList[tI].E[r, ci + c]);
				for (int r = 0; r < y.Rs; r++)
					for (int ci = c; ci < x.Cs; ci++)
						Calc.SetToNM(x.E[r, ci], Common.nList[tI].E[r, ci + y.Cs]);
				Calc.SetToII(tI, zI);
				Common.LDelete(tI);
			}
		} // InsertN

		public static void InsertRN(MultiJ x, MultiJ y, MultiJ rF, int zI)
		// z = x with y inserted at row r
		{
			int r;
			MultiJD.Get1J(out r, rF);
			if (x.Rs == 0 || y.Rs == 0)
				MultiJD.ErrNotMatrix();
			else if (rF.Rs != 0)
				MultiJD.ErrNotScalar();
			else if (x.Cs != y.Cs)
				MultiJD.ErrIncompatible();
			else if (r < 1 || r > x.Rs + 1)
				MultiJD.ErrIncompatible();
			else
			{
				r -= uFirst;
				int tI;
				Common.InitNAdd(1, "InsertRN-tI", out tI);
				MatrixN(x.Rs + y.Rs, x.Cs, tI);
				for (int ri = 0; ri < r; ri++)
					for (int c = 0; c < x.Cs; c++)
						Calc.SetToNM(x.E[ri, c], Common.nList[tI].E[ri, c]);
				for (int ri = 0; ri < y.Rs; ri++)
					for (int c = 0; c < y.Cs; c++)
						Calc.SetToNM(y.E[ri, c], Common.nList[tI].E[ri + r, c]);
				for (int ri = r; ri < x.Rs; ri++)
					for (int c = 0; c < y.Cs; c++)
						Calc.SetToNM(x.E[ri, c], Common.nList[tI].E[ri + y.Rs, c]);
				Calc.SetToII(tI, zI);
				Common.LDelete(tI);
			}
		} // InsertRN

		public static void RSizeN(int zI) // z = size of z
		{
			int rs = Common.nList[zI].Rs;
			if (rs == 0)
				Calc.ClearN(zI);
			else
			{
				int cs = Common.nList[zI].Cs;
				MatrixN(1, 2, zI);
				Common.nList[zI].E[0, 0] &= rs;
				Common.nList[zI].E[0, 1] &= cs;
			}
		} // RSizeN

		public static void RevN(int yI, int zI) // z = reversed digits of z base y
		{
			int yrs = Common.nList[yI].Rs;
			int zrs = Common.nList[zI].Rs;
			int ycs = Common.nList[yI].Cs;
			int zcs = Common.nList[zI].Cs;
			if (yrs == 0 && zrs == 0)
				XICommon.RevN(zI, yI, zI);
			else if (yrs == 0)
			{
				int tzI;
				Common.InitNAdd(1, "RevN-tzI", out tzI);
				for (int r = 0; r < zrs; r++)
					for (int c = 0; c < zcs; c++)
					{
						Calc.SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tzI]);
						XICommon.RevN(tzI, yI, tzI);
						if (MultiID.muErr)
						{
							Common.LDelete(tzI);
							return;
						}
						Calc.SetToNM((MultiSI)Common.nList[tzI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tzI);
			}
			else if (zrs == 0)
			{
				int tyI;
				int tzI;
				int szI;
				Common.InitNAdd(1, "RevN-tyI", out tyI);
				Common.InitNAdd(1, "RevN-tzI", out tzI);
				Common.InitNAdd(1, "RevN-tzI", out szI);
				Calc.SetToNM(Common.nList[zI], (MultiSI)Common.nList[szI]);
				MatrixN(yrs, ycs, zI);
				for (int r = 0; r < yrs; r++)
					for (int c = 0; c < ycs; c++)
					{
						Calc.SetToNM(Common.nList[yI].E[r, c], (MultiSI)Common.nList[tyI]);
						Calc.SetToNM(Common.nList[szI], (MultiSI)Common.nList[tzI]);
						XICommon.RevN(tzI, tyI, tzI);
						if (MultiID.muErr)
						{
							Common.LDelete(szI);
							Common.LDelete(tzI);
							Common.LDelete(tyI);
							return;
						}
						Calc.SetToNM((MultiSI)Common.nList[tzI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(szI);
				Common.LDelete(tzI);
				Common.LDelete(tyI);
			}
			else if (yrs != zrs || ycs != zcs)
				MultiJD.ErrIncompatible();
			else
			{
				int tyI;
				int tzI;
				Common.InitNAdd(1, "RevN-tyI", out tyI);
				Common.InitNAdd(1, "RevN-tzI", out tzI);
				for (int r = 0; r < yrs; r++)
					for (int c = 0; c < ycs; c++)
					{
						Calc.SetToNM(Common.nList[yI].E[r, c], (MultiSI)Common.nList[tyI]);
						Calc.SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tzI]);
						XICommon.RevN(tzI, tyI, tzI);
						if (MultiID.muErr)
						{
							Common.LDelete(tzI);
							Common.LDelete(tyI);
							return;
						}
						Calc.SetToNM((MultiSI)Common.nList[tzI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tzI);
				Common.LDelete(tyI);
			}
		} // RevN

		public static void RRIntN(int yI, int zI) // z = Random Integer between y and z inclusive
		{
			int yrs = Common.nList[yI].Rs;
			int zrs = Common.nList[zI].Rs;
			int ycs = Common.nList[yI].Cs;
			int zcs = Common.nList[zI].Cs;
			if (yrs == 0 && zrs == 0)
				Calc.RRIntN(yI, zI);
			else if (yrs == 0)
			{
				int tzI;
				Common.InitNAdd(1, "RRIntN-tzI", out tzI);
				for (int r = 0; r < zrs; r++)
					for (int c = 0; c < zcs; c++)
					{
						Calc.SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tzI]);
						Calc.RRIntN(yI, tzI);
						if (MultiID.muErr)
						{
							Common.LDelete(tzI);
							return;
						}
						Calc.SetToNM((MultiSI)Common.nList[tzI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tzI);
			}
			else if (zrs == 0)
			{
				int tyI;
				int tzI;
				int szI;
				Common.InitNAdd(1, "RRIntN-tyI", out tyI);
				Common.InitNAdd(1, "RRIntN-tzI", out tzI);
				Common.InitNAdd(1, "RRIntN-tzI", out szI);
				Calc.SetToNM(Common.nList[zI], (MultiSI)Common.nList[szI]);
				MatrixN(yrs, ycs, zI);
				for (int r = 0; r < yrs; r++)
					for (int c = 0; c < ycs; c++)
					{
						Calc.SetToNM(Common.nList[yI].E[r, c], (MultiSI)Common.nList[tyI]);
						Calc.SetToNM(Common.nList[szI], (MultiSI)Common.nList[tzI]);
						Calc.RRIntN(tyI, tzI);
						if (MultiID.muErr)
						{
							Common.LDelete(szI);
							Common.LDelete(tzI);
							Common.LDelete(tyI);
							return;
						}
						Calc.SetToNM((MultiSI)Common.nList[tzI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(szI);
				Common.LDelete(tzI);
				Common.LDelete(tyI);
			}
			else if (yrs != zrs || ycs != zcs)
				MultiJD.ErrIncompatible();
			else
			{
				int tyI;
				int tzI;
				Common.InitNAdd(1, "RRIntN-tyI", out tyI);
				Common.InitNAdd(1, "RRIntN-tzI", out tzI);
				for (int r = 0; r < yrs; r++)
					for (int c = 0; c < ycs; c++)
					{
						Calc.SetToNM(Common.nList[yI].E[r, c], (MultiSI)Common.nList[tyI]);
						Calc.SetToNM(Common.nList[zI].E[r, c], (MultiSI)Common.nList[tzI]);
						Calc.RRIntN(tyI, tzI);
						if (MultiID.muErr)
						{
							Common.LDelete(tzI);
							Common.LDelete(tyI);
							return;
						}
						Calc.SetToNM((MultiSI)Common.nList[tzI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tzI);
				Common.LDelete(tyI);
			}
		} // RRIntN

		public static void RAddN(MultiJ x, int zI)	// z = z + x
		// sets muErr true if overflow
		{
			AddN(Common.nList[zI], x, zI);
		}

		public static void RSubN(MultiJ x, int zI) // z = z - x
		// sets muErr true if overflow
		{
			SubN(Common.nList[zI], x, zI);
		}

		public static void InvN(MultiJ x, MultiJ y, int zI) // z = Inverse of x mod y (x*z = 1 mod y)
		// sets muErr true if overflow
		{
			if (x.Rs == 0 && y.Rs == 0)
				XICommon.InvSI(x, y, Common.nList[zI]);
			else if (x.Rs == 0)
			{
				MultiSI xL = new MultiSI();
				Calc.SetToNM(x, xL);
				if (Common.nList[zI].Rs != y.Rs || Common.nList[zI].Cs != y.Cs)
					MatrixN(y.Rs, y.Cs, zI);
				for (int r = 0; r < y.Rs; r++)
					for (int c = 0; c < y.Cs; c++)
						XICommon.InvSI(xL, y.E[r, c], Common.nList[zI].E[r, c]);
			}
			else if (y.Rs == 0)
			{
				MultiSI yL = new MultiSI();
				Calc.SetToNM(y, yL);
				if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
					MatrixN(x.Rs, x.Cs, zI);
				for (int r = 0; r < x.Rs; r++)
					for (int c = 0; c < x.Cs; c++)
						XICommon.InvSI(x.E[r, c], yL, Common.nList[zI].E[r, c]);
			}
			else if (x.Rs != y.Rs || x.Cs != y.Cs)
				MultiJD.ErrIncompatible();
			else
			{
				if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
					MatrixN(x.Rs, x.Cs, zI);
				for (int r = 0; r < x.Rs; r++)
					for (int c = 0; c < x.Cs; c++)
						XICommon.InvSI(x.E[r, c], y.E[r, c], Common.nList[zI].E[r, c]);
			}
		} // InvN

		public static void AddN(MultiJ x, MultiJ y, int zI) // z = x + y
		// sets muErr true if overflow
		{
			if (x.Rs == 0 && y.Rs == 0)
				XICommon.AddSIN(x, y, zI);
			else if (x.Rs == 0)
			{
				MultiSI xL = new MultiSI();
				Calc.SetToNM(x, xL);
				if (Common.nList[zI].Rs != y.Rs || Common.nList[zI].Cs != y.Cs)
					MatrixN(y.Rs, y.Cs, zI);
				for (int r = 0; r < y.Rs; r++)
					for (int c = 0; c < y.Cs; c++)
						Calc.SetToNM(xL + y.E[r, c], Common.nList[zI].E[r, c]);
			}
			else if (y.Rs == 0)
			{
				MultiSI yL = new MultiSI();
				Calc.SetToNM(y, yL);
				if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
					MatrixN(x.Rs, x.Cs, zI);
				for (int r = 0; r < x.Rs; r++)
					for (int c = 0; c < x.Cs; c++)
						Calc.SetToNM(x.E[r, c] + yL, Common.nList[zI].E[r, c]);
			}
			else if (x.Rs != y.Rs || x.Cs != y.Cs)
				MultiJD.ErrIncompatible();
			else
			{
				if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
					MatrixN(x.Rs, x.Cs, zI);
				for (int r = 0; r < x.Rs; r++)
					for (int c = 0; c < x.Cs; c++)
						Calc.SetToNM(x.E[r, c] + y.E[r, c], Common.nList[zI].E[r, c]);
			}
		} // AddN

		public static void SubN(MultiJ x, MultiJ y, int zI) // z = x - y
		{
			if (x.Rs == 0 && y.Rs == 0)
				XICommon.SubSIN(x, y, zI);
			else if (x.Rs == 0)
			{
				MultiSI xL = new MultiSI();
				Calc.SetToNM(x, xL);
				if (Common.nList[zI].Rs != y.Rs || Common.nList[zI].Cs != y.Cs)
					MatrixN(y.Rs, y.Cs, zI);
				for (int r = 0; r < y.Rs; r++)
					for (int c = 0; c < y.Cs; c++)
						Calc.SetToNM(xL - y.E[r, c], Common.nList[zI].E[r, c]);
			}
			else if (y.Rs == 0)
			{
				MultiSI yL = new MultiSI();
				Calc.SetToNM(y, yL);
				if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
					MatrixN(x.Rs, x.Cs, zI);
				for (int r = 0; r < x.Rs; r++)
					for (int c = 0; c < x.Cs; c++)
						Calc.SetToNM(x.E[r, c] - yL, Common.nList[zI].E[r, c]);
			}
			else if (x.Rs != y.Rs || x.Cs != y.Cs)
				MultiJD.ErrIncompatible();
			else
			{
				if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
					MatrixN(x.Rs, x.Cs, zI);
				for (int r = 0; r < x.Rs; r++)
					for (int c = 0; c < x.Cs; c++)
						Calc.SetToNM(x.E[r, c] - y.E[r, c], Common.nList[zI].E[r, c]);
			}
		} // SubN

		public static void RAdd1N(int d1, int zI) // z = z + d1
		// allows literal d1, d1 is not changed, sets muErr true if overflow
		{
			MultiJ t = new MultiJ(2); // temp
			MultiJD.SetTo1J(d1, t);
			RAddN(t, zI);
			MultiJD.DoneJ(ref t);
		}

		public static void Add1N(MultiJ x, int d1, int zI) // z = x + d1
		// allows literal d1, d1 is not changed, sets muErr true if overflow
		{
			Calc.SetToN(x, zI);
			RAdd1N(d1, zI);
		}

		public static void FormatN(StreamWriter outF, string name, MultiJ z)
		// output z as a formatted set of lines and pages
		// z is not modified
		{
			if (z.Rs == 0)
			{
				XICalLst.FormatN(outF, name, z);
				return;
			}
			bool wB = MultiID.writeBool;
			MultiID.writeBool = false;
			int bO = MultiID.baseO;
			MultiID.baseO = 10;
			string nLc = MultiID.nLc;
			MultiID.nLc = "-" + MultiID.nL;
			MultiID.WriteLn(null, outF, "@Echo Off");
			MultiID.WriteLn(null, outF, "Rem  Code file for matrix " + name);
			int rs = z.Rs;
			int cs = z.Cs;
			MultiID.WriteLn(null, outF);
			MultiID.WriteLn(null, outF, name + " = Mat(" + rs + ", " + cs + ");");
			for (int r = 0; r < rs; r++)
				for (int c = 0; c < cs; c++)
				{
					int tot = 0;
					MultiID.WriteLn(null, outF);
					MultiID.WriteT(null, outF, name + " = Set(" + name + ", " + (r + uFirst) + ", " + (c + uFirst) + ", ", ref tot);
					MultiID.WriteSI(null, outF, z.E[r, c], ref tot, 0, false);
					MultiID.WriteTLn(null, outF, ");", ref tot);
				}
			MultiID.writeBool = wB;
			MultiID.baseO = bO;
			MultiID.nLc = nLc;
		} // FormatN

		public static void RepUnitTN(MultiJ x, MultiJ y, int zI) // z = x -- y
		{
			if (x.Rs == 0 && y.Rs == 0)
				XICalLst.RepUnitTSIN(x, y, zI);
			else if ((x.Rs != 0 && y.Rs != 0) && (x.Rs != y.Rs || x.Cs != y.Cs))
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "RepUnitTN-tI", out tI);
				if (x.Rs == 0)
				{
					MultiSI xL = new MultiSI();
					Calc.SetToNM(x, xL);
					if (Common.nList[zI].Rs != y.Rs || Common.nList[zI].Cs != y.Cs)
						MatrixN(y.Rs, y.Cs, zI);
					for (int r = 0; r < y.Rs; r++)
						for (int c = 0; c < y.Cs; c++)
						{
							XICalLst.RepUnitTSIN(xL, y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else if (y.Rs == 0)
				{
					MultiSI yL = new MultiSI();
					Calc.SetToNM(y, yL);
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICalLst.RepUnitTSIN(x.E[r, c], yL, tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else
				{
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICalLst.RepUnitTSIN(x.E[r, c], y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				Common.LDelete(tI);
			}
		} // RepUnitTN

		public static void CompN(MultiJ x, MultiJ y, string op, string op2, int zI) // z = x op y
		{
			if (x.Rs == 0 && y.Rs == 0)
				XICalLst.CompSIN(x, y, op, op2, zI);
			else if ((x.Rs != 0 && y.Rs != 0) && (x.Rs != y.Rs || x.Cs != y.Cs))
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "CompN-tI", out tI);
				if (x.Rs == 0)
				{
					MultiSI xL = new MultiSI();
					Calc.SetToNM(x, xL);
					if (Common.nList[zI].Rs != y.Rs || Common.nList[zI].Cs != y.Cs)
						MatrixN(y.Rs, y.Cs, zI);
					for (int r = 0; r < y.Rs; r++)
						for (int c = 0; c < y.Cs; c++)
						{
							XICalLst.CompSIN(xL, y.E[r, c], op, op2, tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else if (y.Rs == 0)
				{
					MultiSI yL = new MultiSI();
					Calc.SetToNM(y, yL);
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICalLst.CompSIN(x.E[r, c], yL, op, op2, tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else
				{
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICalLst.CompSIN(x.E[r, c], y.E[r, c], op, op2, tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				Common.LDelete(tI);
			}
		} // CompN

		public static void AndN(MultiJ x, MultiJ y, int zI) // z = x & y
		{
			if (x.Rs == 0 && y.Rs == 0)
				XICalLst.AndSIN(x, y, zI);
			else if ((x.Rs != 0 && y.Rs != 0) && (x.Rs != y.Rs || x.Cs != y.Cs))
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "AndN-tI", out tI);
				if (x.Rs == 0)
				{
					MultiSI xL = new MultiSI();
					Calc.SetToNM(x, xL);
					if (Common.nList[zI].Rs != y.Rs || Common.nList[zI].Cs != y.Cs)
						MatrixN(y.Rs, y.Cs, zI);
					for (int r = 0; r < y.Rs; r++)
						for (int c = 0; c < y.Cs; c++)
						{
							XICalLst.AndSIN(xL, y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else if (y.Rs == 0)
				{
					MultiSI yL = new MultiSI();
					Calc.SetToNM(y, yL);
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICalLst.AndSIN(x.E[r, c], yL, tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else
				{
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICalLst.AndSIN(x.E[r, c], y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				Common.LDelete(tI);
			}
		} // AndN

		public static void OrN(MultiJ x, MultiJ y, int zI) // z = x & y
		{
			if (x.Rs == 0 && y.Rs == 0)
				XICalLst.OrSIN(x, y, zI);
			else if ((x.Rs != 0 && y.Rs != 0) && (x.Rs != y.Rs || x.Cs != y.Cs))
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "OrN-tI", out tI);
				if (x.Rs == 0)
				{
					MultiSI xL = new MultiSI();
					Calc.SetToNM(x, xL);
					if (Common.nList[zI].Rs != y.Rs || Common.nList[zI].Cs != y.Cs)
						MatrixN(y.Rs, y.Cs, zI);
					for (int r = 0; r < y.Rs; r++)
						for (int c = 0; c < y.Cs; c++)
						{
							XICalLst.OrSIN(xL, y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else if (y.Rs == 0)
				{
					MultiSI yL = new MultiSI();
					Calc.SetToNM(y, yL);
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICalLst.OrSIN(x.E[r, c], yL, tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else
				{
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICalLst.OrSIN(x.E[r, c], y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				Common.LDelete(tI);
			}
		} // OrN

		public static void RMul1N(int d1, int zI) // z = z * d1
		// Multi-precision one or two super digit multiply d1 is not changed
		// muErr set true if z overflows
		{
			MultiJ t = new MultiJ(2);
			MultiJD.SetTo1J(d1, t);
			RMulN(t, zI);
			MultiJD.DoneJ(ref t);
		}

		public static void Mul1N(MultiSI x, int d1, int zI) // z = x * d1
		// Multi-precision one or two super digit multiply x and d1 are not changed
		// muErr set true if z overflows
		{
			Calc.SetToN(x, zI);
			RMul1N(d1, zI);
		}

		public static void MulN(MultiJ x, MultiJ y, int zI) // z = x @ y, Matrix multiply
		// x or y or both may have the same address in memory as z
		// x, y are not changed unless they share memory with z
		// sets muErr true if overflow
		{
			if (x.Rs == 0 && y.Rs == 0)
				XICalLst.MulSIN(x, y, zI);
			else if (x.Rs != 0 && y.Rs != 0 && x.Cs != y.Rs)
				MultiJD.ErrIncompatible();
			else if (x.Rs == 0)
			{
				MultiJ z = new MultiJ(0, y.Rs, y.Cs);
				MultiJD.MulM(x, y, z);
				Calc.SetToN(z, zI);
			}
			else if (y.Rs == 0)
			{
				MultiJ z = new MultiJ(0, x.Rs, x.Cs);
				MultiJD.MulM(x, y, z);
				Calc.SetToN(z, zI);
			}
			else
			{
				MultiJ z = new MultiJ(0, x.Rs, y.Cs);
				MultiJD.MulM(x, y, z);
				Calc.SetToN(z, zI);
			}
		} // MulN

		public static void MulEN(MultiJ x, MultiJ y, int zI) // z = x * y, element by element multiply
		// x or y or both may have the same address in memory as z
		// x, y are not changed unless they share memory with z
		// sets muErr true if overflow
		{
			if (x.Rs == 0 && y.Rs == 0)
				XICalLst.MulSIN(x, y, zI);
			else if ((x.Rs != 0 && y.Rs != 0) && (x.Rs != y.Rs || x.Cs != y.Cs))
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "MulEN-tI", out tI);
				if (x.Rs == 0)
				{
					if (Common.nList[zI].Rs != y.Rs || Common.nList[zI].Cs != y.Cs)
						MatrixN(y.Rs, y.Cs, zI);
					for (int r = 0; r < y.Rs; r++)
						for (int c = 0; c < y.Cs; c++)
						{
							XICalLst.MulSIN(x, y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else if (y.Rs == 0)
				{
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICalLst.MulSIN(x.E[r, c], y, tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else
				{
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICalLst.MulSIN(x.E[r, c], y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				Common.LDelete(tI);
			}
		} // MulEN

		public static void RMulN(MultiJ x, int zI) // z = z * x
		// x may have the same address in memory as z
		// x is not changed unless it shares memory with z
		// sets muErr true if overflow
		{
			MulN(Common.nList[zI], x, zI);
		}

		public static void PowN(MultiJ x, MultiJ y, int zI) // z = x^y
		{
			if (x.Rs == 0 && y.Rs == 0)
				XICalLst.PowSIN(x, y, zI);
			else if ((x.Rs != 0 && y.Rs != 0) && (x.Rs != y.Rs || x.Cs != y.Cs))
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "PowN-tI", out tI);
				if (x.Rs == 0)
				{
					MultiSI xL = new MultiSI();
					Calc.SetToNM(x, xL);
					if (Common.nList[zI].Rs != y.Rs || Common.nList[zI].Cs != y.Cs)
						MatrixN(y.Rs, y.Cs, zI);
					for (int r = 0; r < y.Rs; r++)
						for (int c = 0; c < y.Cs; c++)
						{
							XICalLst.PowSIN(xL, y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else if (y.Rs == 0)
				{
					MultiSI yL = new MultiSI();
					Calc.SetToNM(y, yL);
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICalLst.PowSIN(x.E[r, c], yL, tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else
				{
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICalLst.PowSIN(x.E[r, c], y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				Common.LDelete(tI);
			}
		} // PowN

		public static void PowMN(MultiJ x, MultiJ y, int zI) // z = x^y % FMB
		{
			int fMBI;
			if (!Common.FMBIok(zI, out fMBI))
				return;
			else if (x.Rs == 0 && y.Rs == 0)
				XICalLst.PowMSIN(x, y, zI);
			else if ((x.Rs != 0 && y.Rs != 0) && (x.Rs != y.Rs || x.Cs != y.Cs))
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "PowMN-tI", out tI);
				if (x.Rs == 0)
				{
					MultiSI xL = new MultiSI();
					Calc.SetToNM(x, xL);
					if (Common.nList[zI].Rs != y.Rs || Common.nList[zI].Cs != y.Cs)
						MatrixN(y.Rs, y.Cs, zI);
					for (int r = 0; r < y.Rs; r++)
						for (int c = 0; c < y.Cs; c++)
						{
							XICalLst.PowMSIN(xL, y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else if (y.Rs == 0)
				{
					MultiSI yL = new MultiSI();
					Calc.SetToNM(y, yL);
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICalLst.PowMSIN(x.E[r, c], yL, tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else
				{
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICalLst.PowMSIN(x.E[r, c], y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				Common.LDelete(tI);
			}		
		} // PowMN

		public static void DivN(MultiJ u, MultiJ d, int zI) // z = u / d
		// u or d or both may have the same address in memory as z
		// u, d are not changed unless they share memory with z
		// sets muErr true if d = 0
		{
			MultiJ z;
			if (u.Rs == 0)
				z = new MultiJ(Calc.fMC, d.Rs, d.Cs);
			else
				z = new MultiJ(Calc.fMC, u.Rs, u.Cs);
			MultiJD.DivM(u, d, z);
			Calc.SetToN(z, zI);
		} // DivN

		public static void DivRemN(MultiJ x, MultiJ y, int zI) // zij = Floor(xij / yij), Reij = remainder
		// x or y or both may have the same address in memory as z
		// x, y are not changed unless they share memory with z
		// sets muErr true if y = 0
		{
			if (x.Rs == 0 && y.Rs == 0)
				XICalLst.DivRemSIN(x, y, zI);
			else if ((x.Rs != 0 && y.Rs != 0) && (x.Rs != y.Rs || x.Cs != y.Cs))
				MultiJD.ErrIncompatible();
			else
			{
				int rI;
				Common.LFindAdd("Re", out rI);
				if (rI == zI || rI == Common.iTop)
				{
					MultiID.MuWriteErr(
						"Cannot set same location to both quotient and remainder, continuing...");
					return;
				}
				else if (x.Rs == 0)
				{
					MultiJ zM = new MultiJ(Calc.fMC, y.Rs, y.Cs);
					MultiJ rM = new MultiJ(Calc.fMC, y.Rs, y.Cs);
					for (int r = 0; r < y.Rs; r++)
						for (int c = 0; c < y.Cs; c++)
						{
							MultiID.DivQRSI(x, y.E[r, c], rM.E[r, c], zM.E[r, c]);
							if (MultiID.muErr)
							{
								Calc.SetToN(MultiJD.mZero, rI);
								Calc.SetToN(MultiJD.mZero, zI);
								return;
							} 
						}
					Calc.SetToN(zM, zI);
					Calc.SetToN(rM, rI);
				}
				else if (y.Rs == 0)
				{
					MultiJ zM = new MultiJ(Calc.fMC, x.Rs, x.Cs);
					MultiJ rM = new MultiJ(Calc.fMC, x.Rs, x.Cs);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							MultiID.DivQRSI(x.E[r, c], y, rM.E[r, c], zM.E[r, c]);
							if (MultiID.muErr)
							{
								Calc.SetToN(MultiJD.mZero, rI);
								Calc.SetToN(MultiJD.mZero, zI);
								return;
							}
						}
					Calc.SetToN(zM, zI);
					Calc.SetToN(rM, rI);
				}
				else
				{
					MultiJ zM = new MultiJ(Calc.fMC, x.Rs, x.Cs);
					MultiJ rM = new MultiJ(Calc.fMC, x.Rs, x.Cs);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							MultiID.DivQRSI(x.E[r, c], y.E[r, c], rM.E[r, c], zM.E[r, c]);
							if (MultiID.muErr)
							{
								Calc.SetToN(MultiJD.mZero, rI);
								Calc.SetToN(MultiJD.mZero, zI);
								return;
							}
						}
					Calc.SetToN(zM, zI);
					Calc.SetToN(rM, rI);
				}
			}
		} // DivRemN

		public static void Primorial(int zI) // z = Primorial(z)
		{
			if (Common.nList[zI].Rs == 0)
				XICommon.Primorial(zI);
			else
			{
				int tI;
				Common.InitNAdd(1, "Primorial-tI", out tI);
				for (int r = 0; r < Common.nList[zI].Rs; r++)
					for (int c = 0; c < Common.nList[zI].Cs; c++)
					{
						Calc.SetToNM(Common.nList[zI].E[r, c], Common.nList[tI]);
						XICommon.Primorial(tI);
						Calc.SetToNM(Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // Primorial

		public static void FacSN(MultiJ x, int zI, bool MFlag, int fMBI)
		// z = x! = 1*2*3*...*x % FMB
		// x may have the same address in memory as zI
		// x is not changed unless it shares memory with zI
		// sets muErr true if overflow or error
		{
			if (x.Rs == 0)
				XICalLst.FacSSIN(x, zI, MFlag, fMBI);
			else
			{
				int tI;
				Common.InitNAdd(1, "FacN-tI", out tI);
				if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
					MatrixN(x.Rs, x.Cs, zI);
				for (int r = 0; r < x.Rs; r++)
					for (int c = 0; c < x.Cs; c++)
					{
						XICalLst.FacSSIN(x.E[r, c], tI, MFlag, fMBI);
						if (MultiID.muErr)
						{
							Common.LDelete(tI);
							return;
						}
						Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // FacSN

		public static void Fac2N(MultiJ x, MultiJ y, int zI, bool MFlag)
		// z = Fac(x) / Fac(y) % fMB by binary splitting
		// x or y or both may have the same address in memory as z
		// x, y are not changed unless they share memory with z
		// sets muErr true if y = 0
		{
			if (x.Rs == 0 && y.Rs == 0)
				XICalLst.Fac2SIN(x, y, zI, MFlag);
			else if ((x.Rs != 0 && y.Rs != 0) && (x.Rs != y.Rs || x.Cs != y.Cs))
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "Fac2N-tI", out tI);
				if (x.Rs == 0)
				{
					MultiSI xL = new MultiSI();
					Calc.SetToNM(x, xL);
					if (Common.nList[zI].Rs != y.Rs || Common.nList[zI].Cs != y.Cs)
						MatrixN(y.Rs, y.Cs, zI);
					for (int r = 0; r < y.Rs; r++)
						for (int c = 0; c < y.Cs; c++)
						{
							XICalLst.Fac2SIN(xL, y.E[r, c], tI, MFlag);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else if (y.Rs == 0)
				{
					MultiSI yL = new MultiSI();
					Calc.SetToNM(y, yL);
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICalLst.Fac2SIN(x.E[r, c], yL, tI, MFlag);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else
				{
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICalLst.Fac2SIN(x.E[r, c], y.E[r, c], tI, MFlag);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				Common.LDelete(tI);
			}
		} // Fac2N

		public static void FacN(MultiJ x, int zI, bool MFlag, int fMBI)
		// z = x! = 1*2*3*...*x % FMB
		// x may have the same address in memory as zI
		// x is not changed unless it shares memory with zI
		// sets muErr true if overflow or error
		{
			if (x.Rs == 0)
				XICalLst.FacSIN(x, zI, MFlag, fMBI);
			else
			{
				int tI;
				Common.InitNAdd(1, "FacN-tI", out tI);
				if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
					MatrixN(x.Rs, x.Cs, zI);
				for (int r = 0; r < x.Rs; r++)
					for (int c = 0; c < x.Cs; c++)
					{
						XICalLst.FacSIN(x.E[r, c], tI, MFlag, fMBI);
						if (MultiID.muErr)
						{
							Common.LDelete(tI);
							return;
						}
						Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // FacN

		public static void FibN(MultiJ x, int zI)
		// z = Fibonacci number of Int(x)
		// x may have the same address in memory as zI
		// x is not changed unless it shares memory with zI
		// sets muErr true if overflow or error
		{
			if (x.Rs == 0)
				Calc.FibN(x, zI);
			else
			{
				int tI;
				Common.InitNAdd(1, "FibN-tI", out tI);
				if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
					MatrixN(x.Rs, x.Cs, zI);
				for (int r = 0; r < x.Rs; r++)
					for (int c = 0; c < x.Cs; c++)
					{
						Calc.FibN(x.E[r, c], tI);
						if (MultiID.muErr)
						{
							Common.LDelete(tI);
							return;
						}
						Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // FibN


		public static void GamN(MultiJ x, int zI)
		// z = gamma(x)
		// x may have the same address in memory as zI
		// x is not changed unless it shares memory with zI
		// sets muErr true if overflow or error
		{
			if (x.Rs == 0)
				Calc.GamN(x, zI);
			else
			{
				int tI;
				Common.InitNAdd(1, "GamN-tI", out tI);
				if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
					MatrixN(x.Rs, x.Cs, zI);
				for (int r = 0; r < x.Rs; r++)
					for (int c = 0; c < x.Cs; c++)
					{
						Calc.GamN(x.E[r, c], tI);
						if (MultiID.muErr)
						{
							Common.LDelete(tI);
							return;
						}
						Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
		} // GamN

		public static void RFacN(int zI) // z = z! = 1*2*3*...*x
		// sets muErr true if overflow or error
		{
			int fMBI = 1;
			FacN(Common.nList[zI], zI, false, fMBI);
		}

		public static void RFacSN(int zI) // z = z! = 1*2*3*...*x
		// sets muErr true if overflow or error
		{
			int fMBI = 1;
			FacSN(Common.nList[zI], zI, false, fMBI);
		}

		public static void RFacMBN(int zI) // z = z! = 1*2*3*...*x % FMB
		// sets muErr true if overflow or error
		{
			int fMBI;
			if (Common.FMBIok(zI, out fMBI))
				FacN(Common.nList[zI], zI, true, fMBI);
		}

		public static void RFacMBSN(int zI) // z = z! = 1*2*3*...*x % FMB
		// sets muErr true if overflow or error
		{
			int fMBI;
			if (Common.FMBIok(zI, out fMBI))
				FacSN(Common.nList[zI], zI, true, fMBI);
		}

		public static void SqRootN(MultiJ x, int zI) // z = SqRt(x), Re = remainder
		// x is not changed, it is ok for x and z to have the same location in memory
		// but then x will be changed.  Sets muErr true if error
		{
			if (x.Rs == 0)
				XICommon.SqRtRemN(x, zI);
			else
			{
				int rI;
				Common.LFindAdd("Re", out rI);
				if (rI == zI || rI == Common.iTop)
				{
					MultiID.MuWriteErr("Cannot set same location to both cube root and remainder, continuing...");
					return;
				}
				if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
					MatrixN(x.Rs, x.Cs, zI);
				MatrixN(x.Rs, x.Cs, rI);
				for (int r = 0; r < x.Rs; r++)
					for (int c = 0; c < x.Cs; c++)
					{
						XICommon.ReSizeN(4, x.E[r, c], Common.nList[rI].E[r, c]);
						XICommon.ReSizeN(4, x.E[r, c], Common.nList[zI].E[r, c]);
						MultiID.SqRtRemSI(x.E[r, c], Common.nList[rI].E[r, c], Common.nList[zI].E[r, c]);
						if (MultiID.muErr)
							return;
					}
				MultiJD.NormM(Common.nList[rI]);
				MultiJD.NormM(Common.nList[zI]);
			}
		} // SqRootN

		public static void SolveN(MultiJ x, MultiJ y, MultiJ n, int zI) // Solve the modular linear equation x * z = y Mod n for z
		{
			int caseNo = 0;
			if (x.Rs != 0) caseNo += 4;
			if (y.Rs != 0) caseNo += 2;
			if (n.Rs != 0) caseNo += 1;
			if (caseNo == 0)
				XICommon.SolveN(x, y, n, zI);
			else if (caseNo == 1)
			{
				int tI;
				Common.InitNAdd(1, "SolveN-tI", out tI);
				MultiSI xL = new MultiSI();
				Calc.SetToNM(x, xL);
				MultiSI yL = new MultiSI();
				Calc.SetToNM(y, yL);
				if (Common.nList[zI].Rs != n.Rs || Common.nList[zI].Cs != n.Cs)
					MatrixN(n.Rs, n.Cs, zI);
				for (int r = 0; r < n.Rs; r++)
					for (int c = 0; c < n.Cs; c++)
					{
						XICommon.SolveN(xL, yL, n.E[r, c], tI);
						Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
			else if (caseNo == 2)
			{
				int tI;
				Common.InitNAdd(1, "SolveN-tI", out tI);
				MultiSI xL = new MultiSI();
				Calc.SetToNM(x, xL);
				MultiSI nL = new MultiSI();
				Calc.SetToNM(n, nL);
				if (Common.nList[zI].Rs != y.Rs || Common.nList[zI].Cs != y.Cs)
					MatrixN(y.Rs, y.Cs, zI);
				for (int r = 0; r < y.Rs; r++)
					for (int c = 0; c < y.Cs; c++)
					{
						XICommon.SolveN(xL, y.E[r, c], nL, tI);
						Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
			else if (caseNo == 3)
			{
				if (y.Rs != n.Rs || y.Cs != n.Cs)
					MultiJD.ErrIncompatible();
				else
				{
					int tI;
					Common.InitNAdd(1, "SolveN-tI", out tI);
					MultiSI xL = new MultiSI();
					Calc.SetToNM(x, xL);
					if (Common.nList[zI].Rs != n.Rs || Common.nList[zI].Cs != n.Cs)
						MatrixN(n.Rs, n.Cs, zI);
					for (int r = 0; r < n.Rs; r++)
						for (int c = 0; c < n.Cs; c++)
						{
							XICommon.SolveN(xL, y.E[r, c], n.E[r, c], tI);
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
					Common.LDelete(tI);
				}
			}
			else if (caseNo == 4)
			{
				int tI;
				Common.InitNAdd(1, "SolveN-tI", out tI);
				MultiSI yL = new MultiSI();
				Calc.SetToNM(y, yL);
				MultiSI nL = new MultiSI();
				Calc.SetToNM(n, nL);
				if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
					MatrixN(x.Rs, x.Cs, zI);
				for (int r = 0; r < x.Rs; r++)
					for (int c = 0; c < x.Cs; c++)
					{
						XICommon.SolveN(x.E[r, c], yL, nL, tI);
						Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}
			else if (caseNo == 5)
			{
				if (x.Rs != n.Rs || x.Cs != n.Cs)
					MultiJD.ErrIncompatible();
				else
				{
					int tI;
					Common.InitNAdd(1, "SolveN-tI", out tI);
					MultiSI yL = new MultiSI();
					Calc.SetToNM(y, yL);
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICommon.SolveN(x.E[r, c], yL, n.E[r, c], tI);
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
					Common.LDelete(tI);
				}
			}
			else if (caseNo == 6)
			{
				if (x.Rs != y.Rs || x.Cs != y.Cs)
					MultiJD.ErrIncompatible();
				else
				{
					int tI;
					Common.InitNAdd(1, "SolveN-tI", out tI);
					MultiSI nL = new MultiSI();
					Calc.SetToNM(n, nL);
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICommon.SolveN(x.E[r, c], y.E[r, c], nL, tI);
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
					Common.LDelete(tI);
				}
			}
			else if (caseNo == 7)
			{
				if (x.Rs != y.Rs || x.Rs != n.Rs || y.Rs != n.Rs || x.Cs != y.Cs || x.Cs != n.Cs || y.Cs != n.Cs)
					MultiJD.ErrIncompatible();
				else
				{
					int tI;
					Common.InitNAdd(1, "SolveN-tI", out tI);
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICommon.SolveN(x.E[r, c], y.E[r, c], n.E[r, c], tI);
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
					Common.LDelete(tI);
				}
			}
		} // SolveN

		public static int Get1InJ(int low, int high, MultiJ z)
		{
			if (z.Rs != 0) return low;
			int d1;
			if (z.N > 1)
			{
				d1 = 100000000;
				if (z.S) d1 = -d1;
			}
			else
				MultiID.Get1SI(out d1, z);
			if (d1 < low) return low;
			if (d1 > high) return high;
			return d1;
		} // Get1InJ

		public static void SortC(MultiJ x) // x = sorted columns of x
		{
			MultiJ rightJ = new MultiJ(2);
			MultiJD.SetTo1J(x.Rs, rightJ);
			SortCS(x, MultiJD.mOne, rightJ);
		} // SortC

		public static void SortCS(MultiJ x, MultiJ leftJ, MultiJ rightJ) // x = sorted columns of x
		{
			swapped = false;
			if (x.Rs == 0)
				return;
			int left = Get1InJ(1, x.Rs, leftJ);
			left -= uFirst;
			int right = Get1InJ(1, x.Rs, rightJ);
			if (rightJ.Rs != 0)
				right = x.Rs;
			right -= uFirst;
			if (left > right)
			{
				int swap = left;
				left = right;
				right = swap;
			}
			for (int c = 0; c < x.Cs; c++)
				QuickSortC(x, left, right, c);
			if (swapped && !Common.quiet) MultiID.WriteLn("SortC: Some elements were swapped");
			if (!swapped && !Common.quiet) MultiID.WriteLn("SortC: No elements were swapped");
		} // SortCS

		public static void QuickSortC(MultiJ x, int left, int right, int i) // sort column i of x
		{
			if (right <= left)
				return;
			int leftIndex = left;
			int rightIndex = right;
			// Arbitrarily establishing partition element as the midpoint of the array.
			MultiSI partionElement = new MultiSI();
			partionElement &= x.E[(left + right) / 2, i];
			// loop through the array until indices cross
			while (leftIndex <= rightIndex)
			{
				// find the first element that is greater than or equal to
				// the partionElement starting from the leftIndex.
				while ((leftIndex < right) && (x.E[(leftIndex), i] < partionElement))
					leftIndex = leftIndex + 1;
				// find an element that is smaller than or equal to
				// the partionElement starting from the rightIndex.
				while ((rightIndex > left) && (x.E[(rightIndex), i] > partionElement))
					rightIndex = rightIndex - 1;
				// if the indexes have not crossed, swap
				if (leftIndex <= rightIndex)
				{
					if (leftIndex != rightIndex)
					{
						MultiID.SwapSI(ref x.E[(leftIndex), i], ref x.E[(rightIndex), i]);
						swapped = true;
					}
					leftIndex = leftIndex + 1;
					rightIndex = rightIndex - 1;
				}
			}
			// If the right index has not reached the left side of array
			// must now sort the left partition.
			if (left < rightIndex)
				QuickSortC(x, left, rightIndex, i);
			// If the left index has not reached the right side of array
			// must now sort the right partition.
			if (leftIndex < right)
				QuickSortC(x, leftIndex, right, i);
		} // QuickSortC

		public static void SortR(MultiJ x) // x = sorted rows of x
		{
			MultiJ rightJ = new MultiJ(2);
			MultiJD.SetTo1J(x.Cs, rightJ);
			SortRS(x, MultiJD.mOne, rightJ);
		} // SortR

		public static void SortRS(MultiJ x, MultiJ leftJ, MultiJ rightJ) // x = sorted rows of x
		{
			swapped = false;
			if (x.Rs == 0)
				return;
			int left = Get1InJ(1, x.Cs, leftJ);
			left -= uFirst;
			int right = Get1InJ(1, x.Cs, rightJ);
			if (rightJ.Rs != 0)
				right = x.Cs;
			right -= uFirst;
			if (left > right)
			{
				int swap = left;
				left = right;
				right = swap;
			}
			for (int r = 0; r < x.Rs; r++)
				QuickSortR(x, left, right, r);
			if (swapped && !Common.quiet) MultiID.WriteLn("SortR: Some elements were swapped");
			if (!swapped && !Common.quiet) MultiID.WriteLn("SortR: No elements were swapped");
		} // SortRS

		public static void QuickSortR(MultiJ x, int left, int right, int i) // sort row i of x
		{
			if (right <= left)
				return;
			int leftIndex = left;
			int rightIndex = right;
			// Arbitrarily establishing partition element as the midpoint of the array.
			MultiSI partionElement = new MultiSI();
			partionElement &= x.E[i, (left + right) / 2];
			// loop through the array until indices cross
			while (leftIndex <= rightIndex)
			{
				// find the first element that is greater than or equal to
				// the partionElement starting from the leftIndex.
				while ((leftIndex < right) && (x.E[i, (leftIndex)] < partionElement))
					leftIndex = leftIndex + 1;
				// find an element that is smaller than or equal to
				// the partionElement starting from the rightIndex.
				while ((rightIndex > left) && (x.E[i, (rightIndex)] > partionElement))
					rightIndex = rightIndex - 1;
				// if the indexes have not crossed, swap
				if (leftIndex <= rightIndex)
				{
					if (leftIndex != rightIndex)
					{
						MultiID.SwapSI(ref x.E[i, (leftIndex)], ref x.E[i, (rightIndex)]);
						swapped = true;
					}
					leftIndex = leftIndex + 1;
					rightIndex = rightIndex - 1;
				}
			}
			// If the right index has not reached the left side of array
			// must now sort the left partition.
			if (left < rightIndex)
				QuickSortR(x, left, rightIndex, i);
			// If the left index has not reached the right side of array
			// must now sort the right partition.
			if (leftIndex < right)
				QuickSortR(x, leftIndex, right, i);
		} // QuickSortR

		public static void ModFMBN(MultiJ x, int zI) // z = x % FMB
		{
			int fMBI;
			if (Common.FMBIok(zI, out fMBI) && (!Common.ZTestN(fMBI) || Common.nList[fMBI].Rs != 0))
				RModN(Common.nList[fMBI], zI);
		}

		public static void AbsN(MultiJ x, int zI) // z = |x|
		{
			Calc.SetToN(x, zI);
			if (x.Rs == 0)
				Common.nList[zI].S = false;
			else
				for (int r = 0; r < x.Rs; r++)
					for (int c = 0; c < x.Cs; c++)
						Common.nList[zI].E[r, c].S = false;
		} // AbsN

		public static void CubeRtN(int xI, int zI) // z = CubeRt(x) = x^(1/3)
		{
			if (Common.nList[xI].Rs == 0)
				XICommon.CubeRtRemN(xI, zI);
			else
			{
				int rI;
				Common.LFindAdd("Re", out rI);
				if (rI == zI || rI == Common.iTop)
				{
					MultiID.MuWriteErr("Cannot set same location to both cube root and remainder, continuing...");
					return;
				}
				int rs = Common.nList[xI].Rs;
				int cs = Common.nList[xI].Cs;
				if (Common.nList[zI].Rs != rs || Common.nList[zI].Cs != cs)
					MatrixN(rs, cs, zI);
				MultiJ re = new MultiJ(1, rs, cs);
				for (int r = 0; r < rs; r++)
					for (int c = 0; c < cs; c++)
						XICommon.CubeRtSIR(Common.nList[xI].E[r, c], Common.nList[zI].E[r, c], re.E[r, c]);
				Calc.SetToN(re, rI);
			}
		} //* CubeRtN

		public static void DigDN(MultiJ x, int zI) // z = number of digits in x, base 10
		{
			DigN(x, MultiJD.mTen, zI);
		}

		public static void DigN(MultiJ x, MultiJ y, int zI) // z = number of digits in x, base y
		{
			if (x.Rs == 0 && y.Rs == 0)
				XICommon.DigN(x, y, zI);
			else if ((x.Rs != 0 && y.Rs != 0) && (x.Rs != y.Rs || x.Cs != y.Cs))
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "DigN-tI", out tI);
				if (x.Rs == 0)
				{
					MultiSI xL = new MultiSI();
					Calc.SetToNM(x, xL);
					if (Common.nList[zI].Rs != y.Rs || Common.nList[zI].Cs != y.Cs)
						MatrixN(y.Rs, y.Cs, zI);
					for (int r = 0; r < y.Rs; r++)
						for (int c = 0; c < y.Cs; c++)
						{
							XICommon.DigN(xL, y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else if (y.Rs == 0)
				{
					MultiSI yL = new MultiSI();
					Calc.SetToNM(y, yL);
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICommon.DigN(x.E[r, c], yL, tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else
				{
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICommon.DigN(x.E[r, c], y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				Common.LDelete(tI);
			}
		} // DigN

		public static void SumDN(MultiJ x, MultiJ y, int zI) // z = sum of digits in x, base y
		{
			if (x.Rs == 0 && y.Rs == 0)
				XICommon.SumDN(x, y, zI);
			else if ((x.Rs != 0 && y.Rs != 0) && (x.Rs != y.Rs || x.Cs != y.Cs))
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "SumDN-tI", out tI);
				if (x.Rs == 0)
				{
					MultiSI xL = new MultiSI();
					Calc.SetToNM(x, xL);
					if (Common.nList[zI].Rs != y.Rs || Common.nList[zI].Cs != y.Cs)
						MatrixN(y.Rs, y.Cs, zI);
					for (int r = 0; r < y.Rs; r++)
						for (int c = 0; c < y.Cs; c++)
						{
							XICommon.SumDN(xL, y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else if (y.Rs == 0)
				{
					MultiSI yL = new MultiSI();
					Calc.SetToNM(y, yL);
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICommon.SumDN(x.E[r, c], yL, tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else
				{
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICommon.SumDN(x.E[r, c], y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				Common.LDelete(tI);
			}
		} // SumDN

		public static void MagN(MultiJ x, MultiJ y, int zI)	// z = SqRt(Sq(x) + Sq(y))
		{
			if (x.Rs == 0 && y.Rs == 0)
				Calc.MagN(x, y, zI);
			else if ((x.Rs != 0 && y.Rs != 0) && (x.Rs != y.Rs || x.Cs != y.Cs))
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "MagN-tI", out tI);
				int rTI;
				Common.InitNAdd(1, "MagN-rTI", out rTI);
				if (x.Rs == 0)
				{
					int rI;
					Common.LFindAdd("Re", out rI);
					MultiSI xL = new MultiSI();
					Calc.SetToNM(x, xL);
					if (Common.nList[zI].Rs != y.Rs || Common.nList[zI].Cs != y.Cs)
						MatrixN(y.Rs, y.Cs, zI);
					if (Common.nList[rI].Rs != y.Rs || Common.nList[rI].Cs != y.Cs)
						MatrixN(y.Rs, y.Cs, rI);
					for (int r = 0; r < y.Rs; r++)
						for (int c = 0; c < y.Cs; c++)
						{
							Calc.MagNRem(xL, y.E[r, c], tI, rTI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
							Calc.SetToNM((MultiSI)Common.nList[rTI], Common.nList[rI].E[r, c]);
						}
				}
				else if (y.Rs == 0)
				{
					int rI;
					Common.LFindAdd("Re", out rI);
					MultiSI yL = new MultiSI();
					Calc.SetToNM(y, yL);
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					if (Common.nList[rI].Rs != x.Rs || Common.nList[rI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, rI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							Calc.MagNRem(x.E[r, c], yL, tI, rTI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
							Calc.SetToNM((MultiSI)Common.nList[rTI], Common.nList[rI].E[r, c]);
						}
				}
				else
				{
					int rI;
					Common.LFindAdd("Re", out rI);
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					if (Common.nList[rI].Rs != x.Rs || Common.nList[rI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, rI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							Calc.MagNRem(x.E[r, c], y.E[r, c], tI, rTI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
							Calc.SetToNM((MultiSI)Common.nList[rTI], Common.nList[rI].E[r, c]);
						}
				}
				Common.LDelete(rTI);
				Common.LDelete(tI);
			}
		} // MagN

		public static void MaxN(MultiJ x, MultiJ y, int zI)	// z = max of x and y
		{
			if (x.Rs == 0 && y.Rs == 0)
				Calc.MaxSIN(x, y, zI);
			else if ((x.Rs != 0 && y.Rs != 0) && (x.Rs != y.Rs || x.Cs != y.Cs))
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "LCM-tI", out tI);
				if (x.Rs == 0)
				{
					MultiSI xL = new MultiSI();
					Calc.SetToNM(x, xL);
					if (Common.nList[zI].Rs != y.Rs || Common.nList[zI].Cs != y.Cs)
						MatrixN(y.Rs, y.Cs, zI);
					for (int r = 0; r < y.Rs; r++)
						for (int c = 0; c < y.Cs; c++)
						{
							Calc.MaxSIN(xL, y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else if (y.Rs == 0)
				{
					MultiSI yL = new MultiSI();
					Calc.SetToNM(y, yL);
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							Calc.MaxSIN(x.E[r, c], yL, tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else
				{
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							Calc.MaxSIN(x.E[r, c], y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				Common.LDelete(tI);
			}
		} //MaxN

		public static void MinN(MultiJ x, MultiJ y, int zI)	// z = min of x and y
		{
			if (x.Rs == 0 && y.Rs == 0)
				Calc.MinSIN(x, y, zI);
			else if ((x.Rs != 0 && y.Rs != 0) && (x.Rs != y.Rs || x.Cs != y.Cs))
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "LCM-tI", out tI);
				if (x.Rs == 0)
				{
					MultiSI xL = new MultiSI();
					Calc.SetToNM(x, xL);
					if (Common.nList[zI].Rs != y.Rs || Common.nList[zI].Cs != y.Cs)
						MatrixN(y.Rs, y.Cs, zI);
					for (int r = 0; r < y.Rs; r++)
						for (int c = 0; c < y.Cs; c++)
						{
							Calc.MinSIN(xL, y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else if (y.Rs == 0)
				{
					MultiSI yL = new MultiSI();
					Calc.SetToNM(y, yL);
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							Calc.MinSIN(x.E[r, c], yL, tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else
				{
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							Calc.MinSIN(x.E[r, c], y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				Common.LDelete(tI);
			}
		} //MinN

		public static void ModN(MultiJ x, MultiJ y, int zI)	// z = Mod(x, y))
		// same as FORTRAN definition: z = Mod(x, y) = x - (Floor(x/y) * y)
		// sign(z) = sign(y) or z = 0
		{
			if (x.Rs == 0 && y.Rs == 0)
				XICalLst.ModSIN(x, y, zI);
			else if ((x.Rs != 0 && y.Rs != 0) && (x.Rs != y.Rs || x.Cs != y.Cs))
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "ModN-tI", out tI);
				if (x.Rs == 0)
				{
					MultiSI xL = new MultiSI();
					Calc.SetToNM(x, xL);
					if (Common.nList[zI].Rs != y.Rs || Common.nList[zI].Cs != y.Cs)
						MatrixN(y.Rs, y.Cs, zI);
					for (int r = 0; r < y.Rs; r++)
						for (int c = 0; c < y.Cs; c++)
						{
							XICalLst.ModSIN(xL, y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else if (y.Rs == 0)
				{
					MultiSI yL = new MultiSI();
					Calc.SetToNM(y, yL);
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICalLst.ModSIN(x.E[r, c], yL, tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				else
				{
					if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
						MatrixN(x.Rs, x.Cs, zI);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICalLst.ModSIN(x.E[r, c], y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
						}
				}
				Common.LDelete(tI);
			}
		} // ModN

		public static void RModN(MultiJ x, int zI) // z = Mod(z, x))
		{
			ModN(Common.nList[zI], x, zI);
		}

		public static void GCD(MultiJ x, MultiJ y, MultiJ z) // z = GCD(x, y))
		{
			if (x.Rs == 0 && y.Rs == 0)
				XICalLst.GCD(x, y, z);
			else if ((x.Rs != 0 && y.Rs != 0) && (x.Rs != y.Rs || x.Cs != y.Cs))
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "GCD-tI", out tI);
				if (x.Rs == 0)
				{
					MultiSI xL = new MultiSI();
					Calc.SetToNM(x, xL);
					if (z.Rs != y.Rs || z.Cs != y.Cs)
						Matrix(y.Rs, y.Cs, z);
					for (int r = 0; r < y.Rs; r++)
						for (int c = 0; c < y.Cs; c++)
						{
							XICalLst.GCDSIN(xL, y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], z.E[r, c]);
						}
				}
				else if (y.Rs == 0)
				{
					MultiSI yL = new MultiSI();
					Calc.SetToNM(y, yL);
					if (z.Rs != x.Rs || z.Cs != x.Cs)
						Matrix(x.Rs, x.Cs, z);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICalLst.GCDSIN(x.E[r, c], yL, tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], z.E[r, c]);
						}
				}
				else
				{
					if (z.Rs != x.Rs || z.Cs != x.Cs)
						Matrix(x.Rs, x.Cs, z);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICalLst.GCDSIN(x.E[r, c], y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], z.E[r, c]);
						}
				}
				Common.LDelete(tI);
			}
		} // GCD

		public static void GCDN(MultiJ x, MultiJ y, int zI) // z = GCD(x, y))
		{
			MultiJ zz = new MultiJ();
			GCD(x, y, zz);
			Calc.SetToN(zz, zI);
		} // GCDN

		public static void GCDeN(int xI, int yI, int zI) // extended GCD
		// z = GCD(x, y) >= 0, also computes x1 and y1 such that x*x1 + y*y1 = GCD(x, y)
		// x and or y may be <= 0
		{
			int xRs = Common.nList[xI].Rs;
			int xCs = Common.nList[xI].Cs;
			int yRs = Common.nList[yI].Rs;
			int yCs = Common.nList[yI].Cs;
			if (xRs == 0 && yRs == 0)
				XICommon.GCDeN(xI, yI, zI);
			else if ((xRs != 0 && yRs != 0) && (xRs != yRs || xCs != yCs))
				MultiJD.ErrIncompatible();
			else
			{
				int x1I;
				Common.LFindAdd("X1", out x1I);
				int y1I;
				Common.LFindAdd("Y1", out y1I);
				if (x1I == zI || y1I == zI)
				{
					MultiID.MuWriteErr("Cannot set same location to GCD and extended GCD, continuing...");
					return;
				}
				else if (xRs == 0)
				{
					MultiJ zM = new MultiJ(Calc.fMC, yRs, yCs);
					MultiJ x1M = new MultiJ(Calc.fMC, yRs, yCs);
					MultiJ y1M = new MultiJ(Calc.fMC, yRs, yCs);
					for (int r = 0; r < yRs; r++)
						for (int c = 0; c < yCs; c++)
							Calc.GCDeSIr(Common.nList[xI], Common.nList[yI].E[r, c], x1M.E[r, c], y1M.E[r, c], zM.E[r, c]);
					Calc.SetToN(zM, zI);
					Calc.SetToN(x1M, x1I);
					Calc.SetToN(y1M, y1I);
				}
				else if (yRs == 0)
				{
					MultiJ zM = new MultiJ(Calc.fMC, xRs, xCs);
					MultiJ x1M = new MultiJ(Calc.fMC, xRs, xCs);
					MultiJ y1M = new MultiJ(Calc.fMC, xRs, xCs);
					for (int r = 0; r < xRs; r++)
						for (int c = 0; c < xCs; c++)
							Calc.GCDeSIr(Common.nList[xI].E[r, c], Common.nList[yI], x1M.E[r, c], y1M.E[r, c], zM.E[r, c]);
					Calc.SetToN(zM, zI);
					Calc.SetToN(x1M, x1I);
					Calc.SetToN(y1M, y1I);
				}
				else
				{
					MultiJ zM = new MultiJ(Calc.fMC, xRs, xCs);
					MultiJ x1M = new MultiJ(Calc.fMC, xRs, xCs);
					MultiJ y1M = new MultiJ(Calc.fMC, xRs, xCs);
					for (int r = 0; r < xRs; r++)
						for (int c = 0; c < xCs; c++)
							Calc.GCDeSIr(Common.nList[xI].E[r, c], Common.nList[yI].E[r, c], x1M.E[r, c], y1M.E[r, c], zM.E[r, c]);
					Calc.SetToN(zM, zI);
					Calc.SetToN(x1M, x1I);
					Calc.SetToN(y1M, y1I);
				}
			}
		} // GCDeN

		public static void LCM(MultiJ x, MultiJ y, MultiJ z) // z = LCM(x, y))
		// LCM(x, y) = x * y / GCD(x, y)
		{
			if (x.Rs == 0 && y.Rs == 0)
				XICalLst.LCM(x, y, z);
			else if ((x.Rs != 0 && y.Rs != 0) && (x.Rs != y.Rs || x.Cs != y.Cs))
				MultiJD.ErrIncompatible();
			else
			{
				int tI;
				Common.InitNAdd(1, "LCM-tI", out tI);
				if (x.Rs == 0)
				{
					MultiSI xL = new MultiSI();
					Calc.SetToNM(x, xL);
					if (z.Rs != y.Rs || z.Cs != y.Cs)
						Matrix(y.Rs, y.Cs, z);
					for (int r = 0; r < y.Rs; r++)
						for (int c = 0; c < y.Cs; c++)
						{
							XICalLst.LCMSIN(xL, y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], z.E[r, c]);
						}
				}
				else if (y.Rs == 0)
				{
					MultiSI yL = new MultiSI();
					Calc.SetToNM(y, yL);
					if (z.Rs != x.Rs || z.Cs != x.Cs)
						Matrix(x.Rs, x.Cs, z);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICalLst.LCMSIN(x.E[r, c], yL, tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], z.E[r, c]);
						}
				}
				else
				{
					if (z.Rs != x.Rs || z.Cs != x.Cs)
						Matrix(x.Rs, x.Cs, z);
					for (int r = 0; r < x.Rs; r++)
						for (int c = 0; c < x.Cs; c++)
						{
							XICalLst.LCMSIN(x.E[r, c], y.E[r, c], tI);
							if (MultiID.muErr)
							{
								Common.LDelete(tI);
								return;
							}
							Calc.SetToNM((MultiSI)Common.nList[tI], z.E[r, c]);
						}
				}
				Common.LDelete(tI);
			}
		} // LCM

		public static void LCMN(MultiJ x, MultiJ y, int zI) // z = LCM(x, y))
		// LCM(a, b) = a * b / GCD(a, b)
		{
			MultiJ zz = new MultiJ();
			LCM(x, y, zz);
			Calc.SetToN(zz, zI);
		} // LCMN

		public static void ChSMN(int zI) // change sign, n = - n
		{
			MultiJD.ChSM(Common.nList[zI]);
		}

		public static void Matrix(int rs, int cs, MultiJ z)
		// z = zero matrix with rs rows and cs columns
		{
			MultiJ zz;
			MultiJD.Matrix(0, rs, cs, out zz);
			Calc.SetToNM(zz, z);
		} // Matrix

		public static void MatrixN(MultiJ rsF, MultiJ csF, int zI)
		// z = zero matrix with rs rows and cs columns
		{
			if (rsF.Rs != 0 || csF.Rs != 0)
				MultiJD.ErrNotScalar();
			else
			{
				int rs;
				int cs;
				MultiJD.Get1J(out rs, rsF);
				MultiJD.Get1J(out cs, csF);
				MatrixN(rs, cs, zI);
			}
		} // MatrixN

		public static void MatrixN(int rs, int cs, int zI)
		// z = zero matrix with rs rows and cs columns
		{
			MultiJ z;
			MultiJD.Matrix(0, rs, cs, out z);
			Calc.SetToN(z, zI);
		} // MatrixN

		public static void SetAll(MultiJ m, MultiJ x, int zI) // z = m, z[r, c] = x for all z[r, c]
		{
			if (x.Rs != 0)
				MultiJD.ErrNotScalar();
			else if (m.Rs == 0)
				MultiJD.ErrElementOfScalar();
			else
			{
				for (int r = 0; r < m.Rs; r++)
					for (int c = 0; c < m.Cs; c++)
					{
						Calc.SetToNM((MultiSI)x, m.E[r, c]);
					}
				Calc.SetToN(m, zI);
			}
		} // SetAll

		public static void SetElement(MultiJ m, MultiJ rF, MultiJ cF, MultiJ x, int zI)	// z = m, z[r, c] = x
		{
			if (rF.Rs != 0 || cF.Rs != 0 || x.Rs != 0)
				MultiJD.ErrNotScalar();
			else if (m.Rs == 0)
				MultiJD.ErrElementOfScalar();
			else
			{
				int r;
				int c;
				MultiJD.Get1J(out r, rF);
				MultiJD.Get1J(out c, cF);
				r -= uFirst;
				c -= uFirst;
				Calc.SetToN(m, zI);
				if (r >= 0 && c >= 0 && r < Common.nList[zI].Rs && c < Common.nList[zI].Cs)
					Calc.SetToNM(x, Common.nList[zI].E[r, c]);
				else
					MultiJD.ErrOutside();
			}
		} // SetElement

		public static void GetElement(MultiJ x, MultiJ rF, MultiJ cF, int zI) // z = x[r, c]
		{
			if (rF.Rs != 0 || cF.Rs != 0)
				MultiJD.ErrNotScalar();
			else
			{
				int r;
				int c;
				MultiJD.Get1J(out r, rF);
				MultiJD.Get1J(out c, cF);
				GetElement(x, r, c, zI);
			}
		} // GetElement

		public static void GetElement(MultiJ x, int ri, int ci, int zI)	// z = x[r, c]
		{
			if (x.Rs == 0)
				MultiJD.ErrNotMatrix();
			else
			{
				int r = ri - uFirst;
				int c = ci - uFirst;
				if (r >= 0 && c >= 0 && r < x.Rs && c < x.Cs)
				{
					MultiJ z = new MultiJ();
					Calc.SetToNM(x.E[r, c], z);
					Calc.SetToN(z, zI);
				}
				else
					MultiJD.ErrOutside();
			}
		} // GetElement

		public static MultiJ Transpose(MultiJ x)	// return x'
		{
			if (x.Rs == 0)
			{
				MultiJD.ErrNotMatrix();
				return MultiJD.mZero;
			}
			else
			{
				MultiJ z = new MultiJ(0, x.Cs, x.Rs);
				for (int r = 0; r < x.Rs; r++)
					for (int c = 0; c < x.Cs; c++)
						Calc.SetToNM(x.E[r, c], z.E[c, r]);
				return z;
			}
		} // Transpose

		public static void TransposeN(int zI)	// z = z'
		{
				Calc.SetToN(Transpose(Common.nList[zI]), zI);
		} // TransposeN

		public static void InitPerm(int n)	// initialize permutation generator
		{
			vPerm = new int[n];				// resize permutation vector
			nowPerm = -1;
			for (int i = 0; i < n; i++)
				vPerm[i] = 0;
			countPerm = 0;
		} // InitPerm

		public static void VisitPerm(int k, MultiJ x, MultiSI z)	// permutation generator
		// Computes all the permutations of the first n integers.
		// by Alexander Bogomolyn's unordered permutation algorithm
		// See http://www.bearcave.com/random_hacks/permute.html
		// and the book ALGORITHMS by Robert Sedgewick, Addison-Wesley, 1983, page 521.
		{
			int i;
			int n = vPerm.Length;
			nowPerm++;
			vPerm[k] = nowPerm;
			if (nowPerm == n)
			{
				countPerm++;
				DeterminantMul(x, z);
			}
			else
				for (i = 0; i < n; i++)
					if (vPerm[i] == 0)
						VisitPerm(i, x, z);
			nowPerm--;
			vPerm[k] = 0;
		} // VisitPerm

		public static void DeterminantMul(MultiJ x, MultiSI z)	// z = |x|
		{
			MultiSI t = new MultiSI();
			int rem;
			Math.DivRem(countPerm, 4, out rem);
			t &= MultiID.sOne;
			if (rem > 1)
				t.S = true;
			int n = vPerm.Length;
			if (MultiID.debug)
			{
				MultiID.Write(countPerm.ToString() + ") Permitation = ");
				for (int i = 0; i < n; i++)
					MultiID.Write(vPerm[i].ToString() + ' ');
				MultiID.Write("Sign = ");
				int tot = 0;
				MultiID.WriteSILn(t, ref tot);
			}
			for (int i = 0; i < n; i++)
				t *= x.E[i, vPerm[i] - 1];
			MultiID.SetToSI(z + t, z); // resizes z
		} // DeterminantMul

		public static void Determinant(MultiJ x, MultiSI z)	// z = |x|
		{
			MultiID.ClearSI(z);
			int np = x.Rs;
			InitPerm(np);
			VisitPerm(0, x, z);
		} // Determinant

		public static void DeterminantN(MultiJ x, int zI)	// z = |x|
		{
			int rs = x.Rs;
			int cs = x.Cs;
			if (rs == 0 || rs != cs)
				MultiJD.ErrNotSquare();
			else
			{
				MultiSI z = new MultiSI();
				Determinant(x, z);
				Calc.SetToN(z, zI);
			}
		} // DeterminantN

		public static void NotN(MultiJ x, int zI)   // z = !x
		{
			if (x.Rs == 0)
				XICalLst.NotSIN(x, zI);
			else
			{
				int tI;
				Common.InitNAdd(1, "NotN-tI", out tI);
				if (Common.nList[zI].Rs != x.Rs || Common.nList[zI].Cs != x.Cs)
					MatrixN(x.Rs, x.Cs, zI);
				for (int r = 0; r < x.Rs; r++)
					for (int c = 0; c < x.Cs; c++)
					{
						XICalLst.NotSIN(x.E[r, c], tI);
						if (MultiID.muErr)
						{
							Common.LDelete(tI);
							return;
						}
						Calc.SetToNM((MultiSI)Common.nList[tI], Common.nList[zI].E[r, c]);
					}
				Common.LDelete(tI);
			}		
		} // NotN

		public static void RNotN(int zI) // z = !z
		{
			NotN(Common.nList[zI], zI);
		}

		public static bool swapped;		// true if SortC or SortR swapped elements
		public static int countPerm;	// permutation count, 1 thru n!
		public static int nowPerm;		// permutation variable
		public static int uFirst;		// the user index of the first element in a matrix
		public static int[] vPerm;		// permutation vector on first n integers
	}
}
